/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file cs_info_after.c
 *   @brief This is the Module file to act AFTER the ChanServ INFO Command.
 *
 *   It contains all functions which are neccessary to handle<br/>
 *   operations after performing the ChanServ INFO command.
 *
 *   The ChanServ INFO Command allows the user to dump<br/>
 *   some Information about the Channel.
 *
 *   Syntax is: /msg ChanServ INFO #channel
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "global.h"

#define SERVICE "cs"
#define MODUL "info"
#define MODUL_FILE "cs_info"
#define MODUL_VERSION "1.0"

MOD_BOOL cs_info(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_info_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_info_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	
	/* What does it do!?
	--------------------
		ModInfo Module					- It holds a pointer to the modul we just created and added to the modules list
		ModuleInit(
			1st Parameter "thread" 		- It holds a pointer to the thread which is executing the module
										In this thread we store: ChannelInformation, UserInformation and thread handling stuff..
										
			2nd Parameter "dl_handle" 	- The handle for messing with the modulfile itself (used to unload the module again)
			
			3rd Parameter "MOD_TAIL"	- (MOD_HEAD, MOD_UNIQUE, MOD_TAIL)
			
										If you want to create a new module (complete independent from others) you use MOD_UNIQUE:
											i.e. a module using "getinfo" - for ChanServ - as IRC keyword)
											IRC Keyword and Modulname.c are INDEPENDENT!
									
									Executing BEFORE the core module
										If you want a modul to be executed BEFORE i.e. ChanServ "info" you use MOD_HEAD:
											i.e. you add the new ChanServ "info" (must be the same then) as MOD_HEAD
											it will be executed by the IRC Keyword "info" and executed before the core module.
											(Allows you i.e. to modify the permissions for that Module)
											
									Executing AFTER the core module
										If you want a modul to be executed AFTER i.e. ChanServ "info" you use MOD_TAIL:
											i.e. you add the new ChanServ "info" (must be the same then) as MOD_TAIL
											it will be executed by the IRC Keyword "info" and executed after the core module.
											(Allows you i.e. to provide more information about a channel as the core module per default)
			
			4th Parameter "SERVICE"		- The Service for which you want to add this module (if it's now MOD_UNIQUE) STICK
										to the service where the original Module belongs to.
													
			5th Parameter "MODUL_FILE"	- The modules filename (without the .*)
			
			6th Parameter "MODUL"		- That's the IRC-Keyword orion should be reacting on (i.e. /msg ChanServ "info")
			
			7th Parameter "info"		- CommandProtoype
										You are able to put here whatever you want it tells orion about how your command looks like
										examples: 
											"info" tells orion that there is only one IRC Keyword which is part of the Command
											"register" does the same (but it wouldn't have any other effect if i rename it to "IRCKEYWORD")
											"keyword1 * keyword2" tells orion that the real Command consists of keyword1 and keyword2
												-> it will be processed to "keyword1 keyword2" the "*" tells orion to ignore the parameter between the two keywords,
												and so it wouldn't be part of the command!!!
											
											Compare "ns_set" and "ns_saset" for more details ;)
											
			8th Parameter "info"		- It's the same like the 7th Parameter but it's not the Commands prototype, it's the help commands prototype (HelpCommandProtoype)
										Use it the same way like the 7th Parameter.
		);*/
	/* Do it the same way it is done in "cs_info", you are able to copy & paste the initialisation */
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_TAIL,SERVICE,MODUL_FILE,MODUL,"info","info");
	

	/* This adds an author for the module, (PROJECTNAME and YEAR are orion defines)
		if you like others define them i.e in your module-head */
	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	
	/* This adds the version text and number for the module:
		(PROJECTNAME, VERSION_NUMBER, MODUL, MODUL_VERSION) are orion defines.
		VERSION_NUMBER,MODUL,MODUL_VERSION are already visible the head of the file (just change them to fit your needs) */
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	
	/* Here you are able to add options to your module.
		addOptions(
			thread		- The threadhandle.
			&Module		- The address of were we stored our module.
			2			- This tells us how many options you gave us - TAKE CARE if you type 2 orion will only read 2 options
						if you sepcified more and said you only like two orion will only read 2 of them and the module won't run properly.
			
			LOGINONLY,OPER_IF_DEFCON,OPERONLY,READONLY
						These are all available Options.
						LOGINONLY: Module will only be executed it the user is logged in (identified with nickserv)
						OPER_IF_DEFCON: Module will only be executed if the user is an IRCOp (if defcon is >=3)
										If Defcon is <3 users will also be able to execute it.
						OPERONLY: Module can ONLY be executed by IRCOps
						READONLY: The module won't be executed if "readonly" is on (global variable)
		);*/
	addOptions(thread,&Module,2,LOGINONLY,OPER_IF_DEFCON);
	
	/* Adds an Accesstype for your module.
		Available types are: USER, ADMIN, BOTH
		USER: Only users are able to execute that module.
		ADMIN: Only admins are able to execute that module.
		BOTH: Both are able to execute it.
		
		(you even have to check rights then while executing)
		This is just to help Dynamic_Command_Level(); and ModuleFile_DynamicCommandLevel();
		help to decide what level they have to chose by proofing rights later. */
	addAccess (thread,&Module,BOTH);
	
	/* Specify your module type here.
		Available types are: [typedef enum { CORE, THIRD, EVENT, ENCRYPT } MODType;] (modules.h)
		CORE: You shouldn't use it (it's only designed for CORE modules
		THIRD: You are able to use that flag to tell us it's a third party module
		EVENT: Tells us that you add an Eventmodule (you will use that if you make a new client)
			look to "ns_events", "bs_events" and all other "*_events" for more details (each client MUST have an *_events file)
		ENCRYPT: It's an encryption module (you won't build a new one? =P) */
	addMODType(thread,&Module,CORE);

	/* Adding Commands: (the main command)
		addCommand(
			1st Parameter "thread" 				- It holds a pointer to the thread which is executing the module
												In this thread we store: ChannelInformation, UserInformation and thread handling stuff..
										
			2nd Parameter "&Module"				- The address of were we stored our module.
			
			3rd parameter "info"				- This is the keyword for the IRC Command (could also be i.e. "set password" - ns_set)
												How the keyword looks like is now depending on the CommandProtoype!!
										
			4th Parameter "cs_info"				- The function to execute if we get the keyword (TAKE CARE this should
												be already declared in the modulefiles head
			
			---------------------------------
					   -- Rights --
			---------------------------------

			5th Parameter "is_services_admin"	- The least admin access level for that command. 
												(isn't proofed before executing just stored to the module and helps you
													later by proofing access rights)
													
			6th Parameter "is_services_user"	- The least user access level for that command. 
												(isn't proofed before executing just stored to the module and helps you
													later by proofing access rights)	
													
												NOTE: If you are writing an admin only command set it to "INFINIT"
													and the addAccess(); to "ADMIN"							
			
			---------------------------------
					-- Parameters --
			---------------------------------
			IRCD input: (with params) - ever count it like that
				:shock PRIVMSG ChanServ :info #channel
				   0	   1	  2		   3	 4
			
			7th Parameter "4"					- The for the Command at minimum required parameters, they are checked before the module is executed
												(In this case shouldn't be less than 4 - if there are less than 4 parameters
													the execution of the module is skipped and a reply like: "missing parameter(s)" is generated)
												
			8th Parameter "4"					- The maximum parameters for that Command, they are checked before the module is executed
												If the last parameter is a reason or something else of what we don't know how many parameters it may have
												use "0" here - that means: starting from Parameter 4 op to X parameters (so there can be less but never too much)
												
												(In this case shouldn't be higher than 4 - if there are more than 4 parameters
													the execution of the module is skipped and a reply like: "too many parameters" is generated)	*/
	addCommand(thread,&Module,"info",cs_info,is_services_admin,is_services_user,4,4);
	
	/* Adding Commands: (this adds a specific help command, returns the help for a specific command)
		addCommand(
			1st Parameter "thread" 				- It holds a pointer to the thread which is executing the module
												In this thread we store: ChannelInformation, UserInformation and thread handling stuff..
										
			2nd Parameter "&Module"				- The address of were we stored our module.
			
			3rd parameter "help info"			- This is the keyword for the IRC Command (could also be i.e. "set password" - ns_set)
												How the keyword looks like is now depending on the HelpCommandProtoype!!
										
			4th Parameter "cs_info_help"		- The function to execute if we get the keyword (TAKE CARE this should
												be already declared in the modulefiles head
			
			---------------------------------
					   -- Rights --
			---------------------------------

			5th Parameter "is_services_user"	- The least admin access level for that command. 
												(isn't proofed before executing just stored to the module and helps you
													later by proofing access rights)
													
			6th Parameter "is_services_user"	- The least user access level for that command. 
												(isn't proofed before executing just stored to the module and helps you
													later by proofing access rights)	
													
												NOTE: If you are writing an admin only command set it to "INFINIT"
													and the addAccess(); to "ADMIN"							
			
			---------------------------------
					-- Parameters --
			---------------------------------
			IRCD input: (with params) - ever count it like that
				:shock PRIVMSG ChanServ :help info
				   0	   1	  2		   3	4
			
			7th Parameter "4"					- The for the Command at minimum required parameters, they are checked before the module is executed
												(In this case shouldn't be less than 4 - if there are less than 4 parameters
													the execution of the module is skipped and a reply like: "missing parameter(s)" is generated)
												
			8th Parameter "4"					- The maximum parameters for that Command, they are checked before the module is executed
												If the last parameter is a reason or something else of what we don't know how many parameters it may have
												use "0" here - that means: starting from Parameter 4 op to X parameters (so there can be less but never too much)
												
												(In this case shouldn't be higher than 4 - if there are more than 4 parameters
													the execution of the module is skipped and a reply like: "too many parameters" is generated)*/
	addCommand(thread,&Module,"help info",cs_info_help,is_services_user,is_services_user,4,4);
	
	/* Adding Commands: (this adds a specific help command, returns the help for a specific command)
		addCommand(
			1st Parameter "thread" 				- It holds a pointer to the thread which is executing the module
												In this thread we store: ChannelInformation, UserInformation and thread handling stuff..
										
			2nd Parameter "&Module"				- The address of were we stored our module.
			
			3rd parameter "help"				- This is the keyword for the IRC Command (could also be i.e. "set password" - ns_set)
												The keyword "help" functions are executed if a Service is begged for giving an overview of it's commands
												(/msg ChanServ help) - it is controlled from cs_help in this case.
												(if you write a new Service you need a servicetag_help and a servicetag_events module)
										
			4th Parameter "cs_info_help_listing"- The function to execute if we get the keyword (TAKE CARE this should
													be already declared in the modulefiles head)
			
			---------------------------------
					   -- Rights --
			---------------------------------

			5th Parameter "is_services_user"	- The least admin access level for that command. 
												(isn't proofed before executing just stored to the module and helps you
													later by proofing access rights)
													
			6th Parameter "is_services_user"	- The least user access level for that command. 
												(isn't proofed before executing just stored to the module and helps you
													later by proofing access rights)	
													
												NOTE: If you are writing an admin only command set it to "INFINIT"
													and the addAccess(); to "ADMIN"							
			
			---------------------------------
					-- Parameters --
			---------------------------------
			IRCD input: (with params) - ever count it like that
				:shock PRIVMSG ChanServ :help
				   0	   1	  2		   3
			
			NOTE: In this case we add a command with "help".
				Depending on the service this is executed i.e. with /msg ChanServ help
				min_params and max_params are always "3" here.
			
			7th Parameter "3"					- The for the Command at minimum required parameters, they are checked before the module is executed
												(In this case shouldn't be less than 3 - if there are less than 3 parameters
													the execution of the module is skipped and a reply like: "missing parameter(s)" is generated)
												
			8th Parameter "3"					- The maximum parameters for that Command, they are checked before the module is executed
												If the last parameter is a reason or something else of what we don't know how many parameters it may have
												use "0" here - that means: starting from Parameter 4 op to X parameters (so there can be less but never too much)
												
												(In this case shouldn't be higher than 3 - if there are more than 3 parameters
													the execution of the module is skipped and a reply like: "too many parameters" is generated)*/
	addCommand(thread,&Module,"help",cs_info_help_listing,is_services_user,is_services_user,3,3);

	/* The given function is called on unloading the module - 
		could be useful to clear things, change database settings one last time
		or delete any other dependencies */
	addFinish(thread,&Module,orion_Fini);
	
	/* This is to tell our module what levels are the lowest (adminlevel and userlevel)
		helps orion to decide if he could skip the command before executing or not */
	VerifyLevels(thread,&Module);
	
	/* Return that Informations and our possibly received Errors,
		which are held in Module.ModErr - ModuleError(Module.ModErr) returns a more detailled Error information */
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the ChanServ INFO Command.
 * 
 * The ChanServ INFO Command allows the user to dump<br/>
 *   some Information about the Channel.
 *
 * Syntax is: /msg ChanServ INFO #channel
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_info(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */

	/* Initialise Variables to (NULL) */

	/*
	 * Info Command
	 */
	 
	/* After "/msg ChanServ info #channel" is executed (as specified in orion_Init) we do a global
		THIS IS JUST TO ILLUSTRATE HOW STUFF WORKS HERE (kinda useless function ^^) */
	do_global(thread,snick,"cs_info was executed successfully");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the ChanServ INFO Command (/msg ChanServ HELP INFO)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP INFO,<br/> 
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP INFO
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_info_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Info Help
	 */
	 
	 /* Here we show you how to check rights:
			ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help info") :
				It returns the required level for that Command.
				(if you specified:
					"ADMIN" in addAccess() : it will return the modules rootlevel
					"BOTH" in addAccess() : it will return the modules userlevel
					"USER" in addAccess() : it will return the modules userlevel
				)
				NOTE: If you have module where you use BOTH (like most modules)
				you often need to check for admin levels (cauze they are possibly allowed to override settings)
				so you should proof with
					ModuleFile_RootCommandLevel()
				there to ensure the user is an admin.
			
			could also be ...
			
			ModuleFile_RootCommandLevel(thread,MODUL_FILE,"help info")
				It returns the required admin level for that Command.
				
			ModuleFile_UserCommandLevel(thread,MODUL_FILE,"help info")
				It returns the required user level for that Command.
				
			Check for example ns_drop for more details.
	*/
	/* IMPORTANT NOTE: 
		thread->sql.access.rights could be (0-4)
		0: is_user
		1: is_services_user (known & identifed)
		2: is_services_oper
		3: is_services_admin
		4: is_services_root
		(This is used for NickServ, ChanServ, MemoServ, HostServ, BotServ)
		
		O/OperServ, orion, Q and sockscontrol use other levels to make it easier to allow/disallow certain users certain commands.
		
		(proof with the specific levels there!! - you could use "thread->sql.access.rights" too, but that is not what they were designed for!)
		-------------------------------------------
		O/OperServ: 	"thread->sql.access.olevel"
		orion: 			"thread->sql.access.olevel"
		Q:				"thread->sql.access.qlevel"
		sockscontrol:	"thread->sql.access.slevel"
		-------------------------------------------*/
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help info"))
	
	/* Use send_lang to give any reply the users specified language */
	/* For more Langreplies than "CS_HELP_INFO" look to "lang/en.c" for example ;) */
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_INFO");

	else send_lang(thread,language,howtosend,snick,nick,"CS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief ChanServ Help List Command - INFO part (/msg ChanServ help)
 * 
 * When the user tries to get general ChanServ help, using /msg ChanServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the INFO command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_info_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* These two lines are ever very similar to these in other modules
		You should alread know what ModuleFile_DynamicCommandLevel(); is for and how it's used
		**************************************************************************************
		
		thread->sql.aliases.active: holds the users identify status (1: identified, 0: not)
		thread->sql.access.rights: holds the users global access level (only valid if user is identifed properly)
		------------------------------------------------------------------------------------
		Users_CS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	*/
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_CS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Info (Help listing)
	 */
	if(Users_CS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_CMD_INFO");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return MOD_BOOL
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
