/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 *
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by the Free Software Foundation; either version 3
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
 *  the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  if not, see <http://www.gnu.org/licenses/>.
 *
 *  ************************************************************************************************
 *
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 *
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file ns_set_profession.c
 *   @brief This is the module file to manage NickServ's SET Profession
 *
 *   It contains only the NickServ SET profession command
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "ns"
#define MODUL "set"
#define MODUL_FILE "ns_set_profession"
#define MODUL_VERSION "1.0"

MOD_BOOL ns_set_profession(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

void addLanguage(Threads *thread, ModInfo *Module);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/* Initialize a module for commands like /ns "set display" NEWDISPLAY (Command has TWO keywords) */

/**
 * @brief Initialize the module in the orion DB
 *
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_TAIL,SERVICE,MODUL_FILE,MODUL,"set option[1]","set option[1]");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,LOGINONLY,OPER_IF_DEFCON,READONLY);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addLeastModuleLevels(thread,&Module,"set",is_services_user,is_services_user);

	addCommand(thread,&Module,"set profession",ns_set_profession,is_services_user,is_services_user,5,0);

	/* you can load your own configs from etc/ (where services.conf should be too) */
	/* NOTE: we can't check what you require or not - so all is required and all is a string! */
	moduleLoadconfig(thread,Module.module,"ns_set_profession.conf");
	addLanguage(thread,&Module);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);

	printf("Module.ModErr = %d\n",Module.ModErr);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SET Profession function for the NickServ SET Command.
 *
 * The NickServ SET Profession Command allows the user to change it's Profession<br/>
 *
 * Syntax is: /msg NickServ SET PROFESSION < profession >
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_set_profession(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	char *profession = NULL;
	/* Initialise Variables to (NULL) */

	/*
	 * Set Profession Command
	 */
	/* Proof if the command executors GLOBAL level is high enough to use this command */
	if(getConfigVariable(thread,"EnabledProfession")){
		if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set profession")){
			if(!param[5]){
				domysql(thread,"UPDATE `orion_ns_core` SET `profession`='' WHERE `authnick`='%s'",mysql_quote(thread,thread->sql.aliases.authnick));
				moduleSendLang(thread, thread->sql.access.language, howtosend, snick, nick, "NS_SET_PROFESSION_NONE");
			}else{
				profession = _dstrjoin(' ',param,5);
				domysql(thread,"UPDATE `orion_ns_core` SET `profession`='%s' WHERE `authnick`='%s'",mysql_quote(thread,profession),mysql_quote(thread,thread->sql.aliases.authnick));
				moduleSendLang(thread, thread->sql.access.language, howtosend, snick, nick, "NS_SET_CHANGED_PROFESSION",profession);
			}
		}else{
		/*
	 	*  Error Messages
	 	*/
			send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
		}
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

void addLanguage(Threads *thread, ModInfo *Module){
	/************ ENGLISH ****************/
	addModuleLanguage(thread,Module,EN,"NS_SET_PROFESSION_NONE","Profession deleted");
	addModuleLanguage(thread,Module,EN,"NS_SET_CHANGED_PROFESSION","Your profession is now: \002%s\002");
	/************ GERMAN **************+**/
	addModuleLanguage(thread,Module,DE,"NS_SET_PROFESSION_NONE","Du hast jetzt keinen Beruf mehr");
	addModuleLanguage(thread,Module,DE,"NS_SET_CHANGED_PROFESSION","Du bist nun von Beruf \002%s\002");
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 *
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return MOD_BOOL
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
