/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file modules.h
 *   @brief This is the modules header.
 *
 *   It contains all necessary functions and structs to handle modules (used all over the program).
 */
/*  ************************************************************************************************  */

#ifndef MODULES_H
#define MODULES_H

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#ifndef _WIN32
  #include <dlfcn.h>
#endif

#include "datatypes.h"

#ifdef _WIN32
  #ifdef MODULE_COMPILE
    #define DLL __declspec( dllimport )
  #else
    #define DLL __declspec( dllexport )
  #endif
#else
  #define DLL extern
#endif

/************ Defines needed for the modules ************/
#define OPERONLY					0
#define READONLY					1
#define LOGINONLY					2
#define OPER_IF_DEFCON				3
#define WEBACCESS					4

#define OPERONLY_BIT				0x00000001
#define READONLY_BIT				0x00000002
#define LOGINONLY_BIT				0x00000004
#define OPER_IF_DEFCON_BIT			0x00000008
#define WEBACCESS_BIT				0x00000010

#define INFINIT 					30000

/* Defines for handling Events */
#define EVENT_PROTOTYPE 			"\"*\""
#define _ExecuteEvent(x,y,z)		ExecuteEvent(thread,x,y,z);

/************ What do we do on Unix, what on Windows? ************/

#ifdef _WIN32
  #define _dynopen(file)			LoadLibrary(file)
  #define _dynsym(file, symbol)		(void *)GetProcAddress((HMODULE)file, symbol)
  #define _dynclose(file)			FreeLibrary((HMODULE)file) ? 0 : 1
  #define _dyn_clear_error()		SetLastError(0)
  #define MODULE_EXT				".dll"
  
  #define RTLD_LAZY 	0
  #define RTLD_NOW	 	1
#else
  #ifdef HAS_RTLD_LOCAL
    #define _dynopen(file)			dlopen(file, RTLD_LAZY|RTLD_LOCAL)
  #else
    #define _dynopen(file) 			dlopen(file, RTLD_LAZY)
  #endif
  
  #define _dynerror()				dlerror()
  #define _dynsym(file, symbol)		dlsym(file, symbol)
  #define _dynclose(file)			dlclose(file)
  #define _dyn_clear_error()		errno = 0
  #define MODULE_EXT				".so"
#endif

/*****************************************************************/

void *dynopen(Threads *thread, char *lib, int flags);
DLL void *dynsym (Threads *thread, void *handle, char *func);
void dynclose(Threads *threadg, void *handle);

/* List of Commands */
struct commands_ {
	char call[MAX_COMMAND_CALL_LEN];
	/* The pointer to the function we want to execute if "call" matched (Callback) - pointer to our modulefunction */
	MOD_BOOL (*func) (Threads *thread, void* language, bool howtosend, char *snick, char *nick, char *param[]);
	/* The required admin level to access that command */
	int root_level;
	/* The required user level to access that command */
 	int user_level;
 	/* The minimum of required parameters */
	size_t low_param;
 	/* The maximum of wanted parameters (if we don't know - reasons, descriptions - we add it with "0") */
	size_t high_param;
	
	struct commands_ *next, *prev;
};

/* List of Modules */
struct modules_ {
	/* Here we save the author of our module */
	char Author[MAX_MODUL_AUTHOR_LEN];
	/* Here we save the versionstring of our module */
	char Version[MAX_MODUL_VERSION_LEN];
	MODType type;
	MODACCESS access;
	/* Moduleoptions. (bitmask)
	 *	Available options are: LOGINONLY,OPER_IF_DEFCON,OPERONLY,READONLY */
	int options;
	/* The Servicetag (TAKE CARE: the servicetag is not the Nick!!
	 * known Servicetags are: Q,O,sockscontrol,ns,cs,ms,hs,bs,global
	 * Module files MUST EVER have the tag in front of the modulename !!! */
	char servicetag[MAX_NICK_LEN];
	/* Command to call in IRC, example: /cs register has "register" here */
	char modulename[MAX_MODULENAME_LEN];
	/* Filename, example: q_chanlev */
	char filename[MAX_MODULEFILE_LEN];
	/* Dynamic Library Handle - used to handle (close/symbol search) opened librarys */
	void *dl_handle;
	/* Command Prototype - makes orion know what of the input parameters is part of the Command */
	char prototype[MAX_COMMANDNAME_LEN*4];
	/* HelpCommand Prototype - makes orion know what of the input parameters is part of the help Command */
	char helpprototype[MAX_COMMANDNAME_LEN*4];

	/* The configfile the module is using (if it's using any) */
	char configfile[MAX_FILEPATH_LEN];

	/* Finish function - is called before unloading the module */
	MOD_BOOL (*func_finish) (Threads *thread, DO_ACTION Action);
	/* VerifyLevels(); finishes initializing and sets the values for them (DON'T EVER FORGET IT) */
	int LeastUserLevel;							//Least user level to execute something in that module
	int LeastRootLevel;							//Least admin level to execute something in that module

	/* List of Commands for that module */
	struct commands_ *commands;

	struct modules_ *next, *prev;
};

/* The ModInfo struct */
struct ModInfo_ {
	struct modules_ *module;	/* - holds a pointer to the active module */
	MOD_ERROR 		ModErr;		/* - holds the last occured error (used to grab errors on loading) */
};

/* ModCheck struct (to check for errors while: initializing and executing modules */
struct ModCheck_ {
	int 		executed;
	MOD_BOOL 	status;
	MOD_ERROR 	ModErrParams;
	MOD_ERROR 	ModErrOptions;
};

/* Moduleconfig struct */
struct moduleconfig_ {
	char modulefile[MAX_MODULEFILE_LEN];
	char item[MAX_CONFSTRING_LEN];
	char value[MAX_CONFIGVALUE_LEN];
	struct moduleconfig_ *next, *prev;
};

/* Language struct */
struct language_ {
	char modulefile[MAX_MODULEFILE_LEN];
	char item[MAX_LANGSTRING_LEN];
	char *value;
	struct language_ *next, *prev;
};

/* Initialize languages */
DLL void init_langs(Threads *thread);
DLL void reinit_langs(Threads *thread);

/* Return module errors */
char *ModuleError(MOD_ERROR ModErr);

/* check for matching prototype */
bool is_same_prototype(char *prototype_a, char *prototype_b);

/* Loading / Unloading API functions */
DLL ModInfo load_modules	(Threads *thread, char *servicetag);
DLL ModInfo reload_modules	(Threads *thread, char *servicetag);
DLL void unload_modules		(Threads *thread, char *servicetag);

/* Loading modules */
ModInfo OrionLoadModule			(Threads *thread,  char *modulefile);
DLL ModInfo IRCLoadModule		(Threads *thread, void* language, bool howtosend, char *snick, char *nick, char *param[], char *servicetag, char *modulname);

/* Unloading modules */
MOD_ERROR OrionUnloadModule		(Threads *thread, char *modulefile, DO_ACTION unloadtype);
DLL MOD_ERROR IRCUnloadModule	(Threads *thread, void* language, bool howtosend, char *snick, char *nick, char *param[], char *servicetag, char *modulname, DO_ACTION unloadtype);

/* Search for a module */
DLL ModCo 	*FilefindModule	(Threads *thread, char *modulefile);
DLL ModCo 	*findModule		(Threads *thread, char *servicetag, char *modulename);
DLL ModCo 	*findModule_next(Threads *thread, char *servicetag, char *modulename, ModCo *module_start);

/* Search for a command in a specific module */
DLL CMD 	*findCommand	(Threads *thread, ModCo *module, char *calling);

/* Initaialize a module */
DLL ModInfo ModuleInit(Threads *thread, void *dl_handle, MODposition ModPos, char *Modul, char *Filepath, char *CallbyName, char *prototype, char *helpprototype);

/* Add various things to a module */
DLL MOD_ERROR addOptions(Threads *thread, ModInfo *ModHandle, int parameters, ...);
DLL MOD_ERROR addMODType(Threads *thread, ModInfo *ModHandle, MODType itype);
DLL MOD_ERROR addAuthor (Threads *thread, ModInfo *ModHandle, const char *Author, ...);
DLL MOD_ERROR addVersion(Threads *thread, ModInfo *ModHandle, const char *Version, ...);
DLL MOD_ERROR addAccess (Threads *thread, ModInfo *ModHandle, MODACCESS access);
DLL MOD_ERROR addFinish (Threads *thread, ModInfo *ModHandle, MOD_BOOL (*Finish_Func) (Threads *thread, DO_ACTION Action));
DLL MOD_ERROR addCommand(Threads *thread, ModInfo *ModHandle,
	const char *call,
	MOD_BOOL (*func) (Threads *thread, void* language, bool howtosend, char *snick, char *nick, char *param[]),
	int root_level,
	int user_level,
	size_t low_param,
	size_t high_param
);

/* Used to describe the least module levels (admin / user) */
DLL MOD_ERROR addLeastModuleLevels(Threads *thread, ModInfo *ModHandle,
	const char *call,
	int root_level,
	int user_level
);

/* Add an Event Hook (used in eventhandlers - each Service has one!) */
DLL MOD_ERROR addEventHook(Threads *thread, ModInfo *ModHandle,
	const char *call,
	MOD_BOOL (*func) (Threads *thread, void* language, bool howtosend, char *snick, char *nick, char *param[])
);

/*MOD_ERROR addEventCallback(Threads *thread, ModInfo *ModHandle, MODposition ModPos,
	const char *call,
	MOD_BOOL (*func) (Threads *thread, void* language, bool howtosend, char *snick, char *nick, char *param[])
);*/

DLL bool addModuleConfig	(Threads *thread, char *desc, char *value, char *to_modulefile);
DLL void delModuleConfig	(Threads *thread, ModCo *module);
DLL void rehashModuleConfigs(Threads *thread);
DLL char *getConfigVariable	(Threads *thread, char *item);

/* Delete various things from a module, or even the module itself */
MOD_ERROR delModule			(Threads *thread, char *servicetag, char *modulename);
MOD_ERROR delCommand		(Threads *thread, char *servicetag, char *modulename, char *command);
MOD_ERROR delModulebyFile	(Threads *thread, char *modulefile);
MOD_ERROR delCommandbyFile	(Threads *thread, char *modulefile, char *command);

/* Mess with the modules Levels requirements */
DLL int User_Command_Level   			(Threads *thread, char *servicetag, char *modulename, char *command);
DLL int Root_Command_Level   			(Threads *thread, char *servicetag, char *modulename, char *command);
DLL int Dynamic_Command_Level			(Threads *thread, char *servicetag, char *modulename, char *command);
DLL int ModuleFile_UserCommandLevel		(Threads *thread, char *modulefile, char *command);
DLL int ModuleFile_RootCommandLevel		(Threads *thread, char *modulefile, char *command);
DLL int ModuleFile_DynamicCommandLevel	(Threads *thread, char *modulefile, char *command);
DLL MOD_ERROR VerifyLevels(Threads *thread, ModInfo *ModHandle);

/* Mess with the modules Parameter requirements */
DLL bool 	CheckOption			(Threads *thread, char *snick, char *rnick, int option);
MOD_REPLY 	CheckOptions		(Threads *thread, ModCo *module, char *snick, char *rnick);
MOD_REPLY 	CheckParamters		(Threads *thread, ModCo *module, char *command, char *param[], char *snick, char *rnick);
DLL bool 	ModReply			(Threads *thread, ModCheck *modcheck, char *snick, char *nick, char *param[]);
	
/* Running commands */
DLL ModCheck call_command		(Threads *thread, ModCo *module, char *cmd, char *snick, char *nick, char *param[]);
DLL ModCheck call_helpcommand	(Threads *thread, ModCo *module, char *cmd, char *snick, char *nick, char *param[]);
DLL void call_wildview			(Threads *thread, char *snick, char *nick, char *servicetag, char *help_overview_tag, char *param[]);
DLL void call_overviewhelp		(Threads *thread, void *language, bool howtosend, char *snick, char *service, char *help_overview_tag, char *nick, char *param[]);

/* Executing Events */
DLL void ExecuteEvent				(Threads *thread, char *nick, char *param[], char *event);
DLL MOD_ERROR ExecuteSingleEvent	(Threads *thread, char *nick, char *param[], char *servicetag, char *event);

/* Dumping a List of modules (on IRC) */
DLL void ListModules (Threads *thread, char* snick, char *rnick, char *servicetag);
/* Dumping a List of modules (on Console) */
DLL void DumpModules(Threads *thread);
#endif
