/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file cs_access.c
 *   @brief This is the Module file for ChanServ ACCESS Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the ChanServ ACCESS command.
 *
 *   The ChanServ ACCESS Command allows the user to control the channels<br/>
 *   accesslist. (Can add / delete users from the channels accesslist)
 *
 *   Syntax is: /msg ChanServ ACCESS #channel { ADD | DEL | LIST | CLEAR }
 *
 *<table border="0">
 *    <tr><td> ADD   </td><td> Syntax is: /msg ChanServ ACCESS #channel ADD		</td><td> < nick > < level >	</td></tr>
 *    <tr><td> DEL   </td><td> Syntax is: /msg ChanServ ACCESS #channel DEL		</td><td> < nick > 				</td></tr>
 *    <tr><td> LIST  </td><td> Syntax is: /msg ChanServ ACCESS #channel LIST	</td><td>						</td></tr>
 *    <tr><td> CLEAR </td><td> Syntax is: /msg ChanServ ACCESS #channel CLEAR	</td><td>						</td></tr>
 *</table>
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "webform.h"
#include "global.h"

#define SERVICE "cs"
#define MODUL "access"
#define MODUL_FILE "cs_access"
#define MODUL_VERSION "1.2"

MOD_BOOL cs_access_add(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_access_del(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_access_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_access_clear(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL cs_access_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_access_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"access * option[1]","access");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,WEBACCESS,LOGINONLY,OPER_IF_DEFCON);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addLeastModuleLevels(thread,&Module,"access",is_services_admin,is_services_user); /* just to add a displaylevel for /msg cs help
															  this should always be the lowest used Module level! */
															  
	addCommand(thread,&Module,"access add",cs_access_add,is_services_admin,is_services_user,7,7);
	addCommand(thread,&Module,"access del",cs_access_del,is_services_admin,is_services_user,6,6);
	addCommand(thread,&Module,"access list",cs_access_list,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"access clear",cs_access_clear,is_services_admin,is_services_user,5,5);
	
	addCommand(thread,&Module,"help access",cs_access_help,is_services_user,is_services_user,4,4);
	addCommand(thread,&Module,"help",cs_access_help_listing,is_services_user,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief CS ACCESS Add function for the ChanServ ACCESS Command.
 * 
 * The ChanServ ACCESS Add Command allows the user to add other users to the<br/>
 *   Access-List with a specific Level
 *  
 * Syntax is: /msg ChanServ ACCESS #channel ADD < nick > < level >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_access_add(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;
	//Aliases aliases5;
	int sender_level = 0, nicks_level = 0;
	int NSAccess = 0, CSAccess = 0;
	
	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));
	//bzero(&aliases5,sizeof(aliases5));
	
	/*
	 * Access Add Command
	 */
	// :sagara PRIVMSG chanserv@services.deepirc :access #ircops add Ally 10
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"access add")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],param[6]);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv") && (~thread->c.chan.settings & CS_XOP_BIT)){
			if(CheckOption(thread,snick,nick,READONLY)){
                if(digitcol(param[7]) && stoi(param[7])>=ACCESS_UNKNOWN_END){
					if(stoi(param[7])>ACCESS_FOUNDER){
						send_lang(thread,language,howtosend,snick,nick,"ACCESS_IN_CHANNEL_LOW",param[4]);
						return MOD_CONT;
					}
    				//aliases5     = sql_aliases(thread,param[6]);
					sender_level = stoi(thread->c.chan.access.ownlevel);
					nicks_level  = stoi(thread->c.chan.access.targetlevel);
					
					NSAccess     = stoi(mysql_sel_var(thread,"SELECT count(1) FROM `orion_cs_access` WHERE `authnick`='%s'",mysql_quote(thread,thread->c.aliases.authnick)));
					CSAccess     = stoi(mysql_sel_var(thread,"SELECT count(1) FROM `orion_cs_access` WHERE `channel`='%s'",mysql_quote(thread,param[4])));
					
					if(thread->c.aliases.fields && thread->c.aliases.authnick[0]!='\0'){
    					if((NSAccess<m->NSAccessMax || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"access add")) && CSAccess<m->CSAccessMax){
    	    				if(thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"access") || (sender_level>nicks_level && sender_level>=thread->c.chan.levels.accchange && stoi(param[7])<=sender_level) || streqnc(thread->c.chan.founder,thread->sql.aliases.authnick) || streqnc(thread->c.chan.successor,thread->sql.aliases.authnick)){
    	                        if(nicks_level>IDENTIFIED_NO_ACCESS){
    	        					domysql(thread,"UPDATE `orion_cs_access` LEFT OUTER JOIN `orion_cs_core` ON (`orion_cs_access`.`channel`=`orion_cs_core`.`channel`) SET `orion_cs_core`.`last_used`='%ld', `orion_cs_access`.`level`='%d' WHERE `orion_cs_access`.`serv`='ChanServ' AND `orion_cs_access`.`authnick`='%s' AND `orion_cs_access`.`channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,thread->c.aliases.authnick),mysql_quote(thread,param[4]));
    	        					
    	        					if(!thread->Webform)
    	                        		do_logchan(thread,snick,NULL,NULL,NULL,"%s!%s@%s level (%d) set access level %d to %s (group %s) on channel %s",nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,sender_level,stoi(param[7]),thread->c.aliases.nick,thread->c.aliases.authnick,thread->c.chan.name);
    	                        	else
    	                        		do_logchan(thread,snick,NULL,NULL,NULL,"WEB: %s level (%d) set access level %d to %s (group %s) on channel %s",nick,sender_level,stoi(param[7]),thread->c.aliases.nick,thread->c.aliases.authnick,thread->c.chan.name);
    							   
									send_lang(thread,language,howtosend,snick,nick,"CS_ACCESS_ACCESS_CHANGED",thread->c.aliases.authnick,thread->c.chan.name,stoi(param[7]));
    							    setWebSuccess(thread); /* Indentify webform that changes were applied */
    	                        }
    	                        else{
    	                            domysql(thread,"INSERT INTO `orion_cs_access` (`serv`,`authnick`,`channel`,`level`) VALUES ('ChanServ','%s','%s','%d')",mysql_quote(thread,thread->c.aliases.authnick),mysql_quote(thread,thread->c.chan.name),stoi(param[7]));

									if(!thread->Webform)
    	                        		do_logchan(thread,snick,NULL,NULL,NULL,"%s!%s@%s level (%d) set access level %d to %s (group %s) on channel %s",nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,sender_level,stoi(param[7]),thread->c.aliases.nick,thread->c.aliases.authnick,thread->c.chan.name);
    	                        	else
    	                        		do_logchan(thread,snick,NULL,NULL,NULL,"WEB: %s level (%d) set access level %d to %s (group %s) on channel %s",nick,sender_level,stoi(param[7]),thread->c.aliases.nick,thread->c.aliases.authnick,thread->c.chan.name);
    	                        		
    							    send_lang(thread,language,howtosend,snick,nick,"CS_ACCESS_ACCESS_ADDED",thread->c.aliases.authnick,thread->c.chan.name,stoi(param[7]));
    							    setWebSuccess(thread); /* Indentify webform that changes were applied */
    	    					}
    	    					dynncpy(thread->c.chan.access.targetlevel,param[7],sizeof(thread->c.chan.access.targetlevel));
    	    				}
    	                    else send_lang(thread,language,howtosend,snick,nick,"ACCESS_IN_CHANNEL_TOO_LOW",sender_level,param[4]);
    					}else send_lang(thread,language,howtosend,snick,nick,"ACCESS_LIMIT_X_FOR_NICK_OR_X_CHANNEL_REACHED",m->NSAccessMax,m->CSAccessMax);
                    }else send_lang(thread,language,howtosend,snick,nick,"NO_AUTHNICK");
                }else send_lang(thread,language,howtosend,snick,nick,"MISSING_PARAMETER_OR_NONE_DIGIT");
			}else send_lang(thread,language,howtosend,snick,nick,"READONLY_MODE");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_OR_XOP",snick);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS ACCESS Del function for the ChanServ ACCESS Command.
 * 
 * The ChanServ ACCESS Del Command allows the user to delete other users from<br/>
 *   the Access-List.
 *  
 * Syntax is: /msg ChanServ ACCESS #channel DEL < nick >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_access_del(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;
	//Aliases aliases5;
	int sender_level = 0, nicks_level = 0;
	
	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));
	//bzero(&aliases5,sizeof(aliases5));
	
	/*
	 * Access Del Command
	 */
	// :sagara PRIVMSG chanserv@services.deepirthread->c.chan.net :access #ircops del Ally
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"access del")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],param[6]);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv") && (~thread->c.chan.settings & CS_XOP_BIT)){
			if(CheckOption(thread,snick,nick,READONLY)){
				//aliases5     = sql_aliases(thread,param[6]);
				sender_level = stoi(thread->c.chan.access.ownlevel);
				nicks_level  = stoi(thread->c.chan.access.targetlevel);
				
				if(nicks_level>IDENTIFIED_NO_ACCESS && thread->c.aliases.fields && ((sender_level>nicks_level && sender_level>=thread->c.chan.levels.accchange) || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"access del") || streqnc(thread->c.chan.founder,thread->sql.aliases.authnick) || streqnc(thread->c.chan.successor,thread->sql.aliases.authnick))){
					domysql(thread,"DELETE FROM `orion_cs_access` WHERE `serv`='ChanServ' AND `authnick`='%s' AND `channel`='%s'",mysql_quote(thread,thread->c.aliases.authnick),mysql_quote(thread,param[4]));
					
					if(!thread->Webform)
                    	do_logchan(thread,snick,NULL,NULL,NULL,"%s!%s@%s level (%d) deleted %s (group %s) on channel %s",nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,sender_level,thread->c.aliases.nick,thread->c.aliases.authnick,thread->c.chan.name);
                    else
                    	do_logchan(thread,snick,NULL,NULL,NULL,"WEB: %s level (%d) deleted %s (group %s) on channel %s",nick,sender_level,thread->c.aliases.nick,thread->c.aliases.authnick,thread->c.chan.name);
                    	
					send_lang(thread,language,howtosend,snick,nick,"CS_ACCESS_ACCESS_DELETED",thread->c.aliases.authnick,thread->c.chan.name);
					bzero(&thread->c.chan.access.targetlevel,sizeof(thread->c.chan.access.targetlevel));
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}
                else send_lang(thread,language,howtosend,snick,nick,"ACCESS_IN_CHANNEL_TOO_LOW_OR_USER_NOT_REGISTERED",sender_level,param[4],param[6]);
			}else send_lang(thread,language,howtosend,snick,nick,"READONLY_MODE");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_OR_XOP",snick);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS ACCESS List function for the ChanServ ACCESS Command.
 * 
 * The ChanServ ACCESS List Command allows the user to dump the<br/>
 *   Access-List for a specific Channel
 *  
 * Syntax is: /msg ChanServ ACCESS #channel LIST
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_access_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	deepSQL Chanaccess;
	//ChannelDB c;
	int sender_level = 0;
	int i = 0;
	
	/* Initialise Variables to (NULL) */
	bzero(&Chanaccess,sizeof(Chanaccess));
	//bzero(&c,sizeof(c));
	
	/*
	 * Access List Command
	 */
	// :sagara PRIVMSG chanserv@services.deepirthread->c.chan.net :access #ircops list
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"access list")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv")){
            sender_level = stoi(thread->c.chan.access.ownlevel);
                
    		if(sender_level>=thread->c.chan.levels.acclist || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"access list") || streqnc(thread->c.chan.founder,thread->sql.aliases.authnick) || streqnc(thread->c.chan.successor,thread->sql.aliases.authnick)){
        		Chanaccess.res    = doSelectMySQL(thread,"SELECT `authnick`,`level` FROM `orion_cs_access` WHERE `channel`='%s' ORDER BY `authnick` ASC",mysql_quote(thread,param[4]));
				Chanaccess.fields = mysql_num_rows(Chanaccess.res);
				i = 0;
        		    
        		if(Chanaccess.fields>0){
					for(i=0;i<Chanaccess.fields;i++){
						Chanaccess.row = mysql_fetch_row(Chanaccess.res);
							
						send_lang(thread,language,howtosend,snick,nick,"CS_ACCESS_LISTINGS",i+1,Chanaccess.row[0],stoi(Chanaccess.row[1]));
					}
        			send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");
        		}
                else send_lang(thread,language,howtosend,snick,nick,"NO_LIST_AVAIL");
        		    
        		mysql_free_result_myr(Chanaccess.res); //free MySQL result
        		
        		setWebSuccess(thread); /* Indentify webform that changes were applied */
        	}
			else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_CHANSERV");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS ACCESS Clear function for the ChanServ ACCESS Command.
 * 
 * The ChanServ ACCESS Clear Command allows the user clear the<br/>
 *   Access-List for a specific Channel
 *  
 * Syntax is: /msg ChanServ ACCESS #channel CLEAR
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_access_clear(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;
	//int sender_level = 0;
	
	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));
	
	/*
	 * Access Clear Command
	 */
	// :sagara PRIVMSG chanserv@services.deepirthread->c.chan.net :access #ircops clear
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"access clear")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv")){
			if(CheckOption(thread,snick,nick,READONLY)){
                //sender_level = stoi(thread->c.chan.access.ownlevel);
                
    			if(thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"access clear") || streqnc(thread->c.chan.founder,thread->sql.aliases.authnick) || streqnc(thread->c.chan.successor,thread->sql.aliases.authnick)){
                    domysql(thread,"DELETE FROM `orion_cs_access` WHERE `serv`='ChanServ' AND `channel`='%s'",mysql_quote(thread,param[4]));

					if(!thread->Webform)
                		do_logchan(thread,snick,NULL,NULL,NULL,"%s!%s@%s level (%d) cleared access list for channel %s",nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,stoi(thread->c.chan.access.ownlevel),thread->c.chan.name);
                	else
                		do_logchan(thread,snick,NULL,NULL,NULL,"WEB: %s level (%d) cleared access list for channel %s",nick,stoi(thread->c.chan.access.ownlevel),thread->c.chan.name);
                	
				    send_lang(thread,language,howtosend,snick,nick,"CS_ACCESS_CLEARED_FOR_CHANNEL_X",param[4]);
				    bzero(&thread->c.chan.access.targetlevel,sizeof(thread->c.chan.access.targetlevel));
				    bzero(&thread->c.chan.access.ownlevel,sizeof(thread->c.chan.access.ownlevel));
				    
				    setWebSuccess(thread); /* Indentify webform that changes were applied */
        		}
                else send_lang(thread,language,howtosend,snick,nick,"ACCESS_IN_CHANNEL_TOO_LOW",thread->c.chan.access.ownlevel,param[4]);
			}else send_lang(thread,language,howtosend,snick,nick,"READONLY_MODE");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_CHANSERV");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the ChanServ ACCESS Command (/msg ChanServ HELP ACCESS)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP ACCESS,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP ACCESS
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_access_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Access Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help access")){
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_ACCESS",snick,snick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}

	else send_lang(thread,language,howtosend,snick,nick,"CS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief ChanServ Help List Command - ACCESS part (/msg ChanServ help)
 * 
 * When the user tries to get general ChanServ help, using /msg ChanServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the ACCESS command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_access_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_CS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Access (Help listing)
	 */
	if(Users_CS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_CMD_ACCESS");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
