/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file cs_devoice.c
 *   @brief This is the Module file for ChanServ DEVOICE Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the ChanServ DEVOICE command.
 *
 *   The ChanServ DEVOICE Command allows the user to devoice ...<br/>
 *   1. Himself in all channels, he has the rights to take himself voice (no parameters)<br/>
 *   2. Himself in a specific channel, if he has enough rights in that channel (the channel parameter)<br/>
 *   3. Another User in a specific channel, if he has enough rights to devoice target user in target channel (both parameters)
 *  
 *   Syntax is: /msg ChanServ DEVOICE [#channel] [nick]
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "webform.h"
#include "global.h"

#define SERVICE "cs"
#define MODUL "devoice"
#define MODUL_FILE "cs_devoice"
#define MODUL_VERSION "1.2"

MOD_BOOL cs_devoice(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_devoice_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_devoice_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"devoice","devoice");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,WEBACCESS,LOGINONLY,OPER_IF_DEFCON);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"devoice",cs_devoice,is_services_oper,is_services_user,3,5);
	addCommand(thread,&Module,"help devoice",cs_devoice_help,is_services_user,is_services_user,4,4);
	addCommand(thread,&Module,"help",cs_devoice_help_listing,is_services_user,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the ChanServ DEVOICE Command.
 * 
 * The ChanServ DEVOICE Command allows the user to devoice ...<br/>
 *  1. Himself in all channels, he has the rights to take himself voice (no parameters)<br/>
 *  2. Himself in a specific channel, if he has enough rights in that channel (the channel parameter)<br/>
 *  3. Another User in a specific channel, if he has enough rights to devoice target user in target channel (both parameters)
 *  
 * Syntax is: /msg ChanServ DEVOICE [#channel] [nick]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_devoice(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	Aliases aliases5;
	deepSQL devoice;
	int i = 0;
	
	/* Initialise Variables to (NULL) */
	bzero(&aliases5,sizeof(aliases5));
	bzero(&devoice,sizeof(devoice));
	
	/*
	 * Devoice Command
	 */
	// :shock PRIVMSG chanserv :devoice #channel nick
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"devoice")){
		if(param[4]){
			if(streqnc(param[5],nick) || !param[5]) // devoice myself in a certain channel
				devoice.res = doSelectMySQL(thread,"SELECT `level`,`orion_cs_core`.`cs_voiceme` AS `voice`,`orion_cs_core`.`datecreate`,`orion_cs_core`.`botnick`,`orion_cs_core`.`bs_set` FROM `orion_cs_access` LEFT JOIN `orion_cs_core` ON `orion_cs_access`.`channel`=`orion_cs_core`.`channel` WHERE `orion_cs_access`.`serv`='ChanServ' AND `orion_cs_access`.`channel`='%s' AND `orion_cs_access`.`authnick`='%s' LIMIT 0,1",mysql_quote(thread,param[4]),mysql_quote(thread,thread->sql.aliases.authnick));
			else // devoice someone else in a certain channel
				devoice.res = doSelectMySQL(thread,"SELECT `level`,`orion_cs_core`.`cs_voice`,`orion_cs_core`.`datecreate`,`orion_cs_core`.`settings`,`orion_cs_core`.`botnick`,`orion_cs_core`.`bs_set` FROM `orion_cs_access` LEFT JOIN `orion_cs_core` ON `orion_cs_access`.`channel`=`orion_cs_core`.`channel` WHERE `orion_cs_access`.`serv`='ChanServ' AND `orion_cs_access`.`channel`='%s' AND `orion_cs_access`.`authnick`='%s'  LIMIT 0,1",mysql_quote(thread,param[4]),mysql_quote(thread,thread->sql.aliases.authnick));

			devoice.fields = mysql_num_rows(devoice.res);
			if(devoice.fields){
				devoice.row = mysql_fetch_row(devoice.res);
				// devoice.row[0]: own chanaccess level, devoice.row[1]: voiceme/voice level...
				if(stoi(devoice.row[0])>=stoi(devoice.row[1]) || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"devoice")){
					if(param[5] && !streqnc(param[5],nick)){
						aliases5 = sql_aliases(thread,param[5]);
						
						if(thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"devoice") || (~stol(devoice.row[3])&CS_PEACE_BIT || (stol(devoice.row[3])&CS_PEACE_BIT && stoi(devoice.row[0])>stoi(acclevel(thread,"ChanServ",param[4],aliases5.authnick))))){
							do_mode(thread,CSCheckBotInPlace(devoice.row[4],stol(devoice.row[5])),param[4],(time_t)stol(devoice.row[2]),"-v %s",param[5]);
							send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
							
							setWebSuccess(thread); /* Indentify webform that changes were applied */
						}
						else send_lang(thread,language,howtosend,snick,nick,"CS_PERMISSION_DENIED");
					}else{
						do_mode(thread,CSCheckBotInPlace(devoice.row[3],stol(devoice.row[4])),param[4],(time_t)stol(devoice.row[2]),"-v %s",nick);
						send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
						
						setWebSuccess(thread); /* Indentify webform that changes were applied */
					}
				}else send_lang(thread,language,howtosend,snick,nick,"ACCESS_IN_CHANNEL_LOW",param[4]);
			}
			mysql_free_result_myr(devoice.res);
		}else{
			devoice.res    = doSelectMySQL(thread,"SELECT `orion_cs_access`.`channel`,`orion_cs_access`.`level`,`orion_cs_core`.`cs_voiceme`,`orion_cs_core`.`datecreate`,`orion_cs_core`.`botnick`,`orion_cs_core`.`bs_set` FROM `orion_cs_access` LEFT JOIN (`orion_cs_core`,`orion_chan_users`) ON (`orion_cs_access`.`channel` = `orion_cs_core`.`channel` AND `orion_cs_access`.`channel`=`orion_chan_users`.`channel`) WHERE `orion_cs_access`.`authnick`='%s' AND `orion_chan_users`.`nick` ='%s' AND `orion_cs_access`.`serv`='ChanServ'",mysql_quote(thread,thread->sql.aliases.authnick),mysql_quote(thread,nick));
			devoice.fields = mysql_num_rows(devoice.res);
			
			for(i=0;i<devoice.fields;i++){
				devoice.row = mysql_fetch_row(devoice.res);
				// devoice.row[1]: own chanaccess level, devoice.row[2]: voiceme level
				if(stoi(devoice.row[1])>=stoi(devoice.row[2])) do_mode(thread,CSCheckBotInPlace(devoice.row[4],stol(devoice.row[5])),devoice.row[0],(time_t)stol(devoice.row[3]),"-v %s",nick);
			}
			send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
			mysql_free_result_myr(devoice.res);
			
			setWebSuccess(thread); /* Indentify webform that changes were applied */
		}
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the ChanServ DEVOICE Command (/msg ChanServ HELP DEVOICE)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP DEVOICE,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP DEVOICE
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_devoice_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Devoice Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help devoice")){
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_DEVOICE");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}

	else send_lang(thread,language,howtosend,snick,nick,"CS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief ChanServ Help List Command - DEVOICE part (/msg ChanServ help)
 * 
 * When the user tries to get general ChanServ help, using /msg ChanServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the DEVOICE command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_devoice_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_CS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Devoice (Help listing)
	 */
	if(Users_CS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_CMD_DEVOICE");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
