/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file cs_forbid.c
 *   @brief This is the Module file for ChanServ FORBID Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the ChanServ FORBID command.
 *
 *   The ChanServ FORBID Command allows an Admin to forbid target #channel.<br/>
 *   That Channel can't be used (joined) or accessed by users anymore.<br/>
 *   It also prevents that the Channel will be registered at a later time (if it's not now)<br/>
 *   (Limited to Services Admins).
 *
 *   Syntax is: /msg ChanServ FORBID #channel [reason]
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"
#include "webform.h"

#define SERVICE "cs"
#define MODUL "forbid"
#define MODUL_FILE "cs_forbid"
#define MODUL_VERSION "1.2"

MOD_BOOL cs_forbid(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_forbid_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_forbid_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by <br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"forbid","forbid");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,5,WEBACCESS,LOGINONLY,OPER_IF_DEFCON,OPERONLY,READONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"forbid",cs_forbid,is_services_admin,INFINIT,4,0);
	addCommand(thread,&Module,"help forbid",cs_forbid_help,is_services_admin,INFINIT,4,4);
	addCommand(thread,&Module,"help",cs_forbid_help_listing,is_services_admin,INFINIT,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the ChanServ FORBID Command.
 * 
 * The ChanServ FORBID Command allows an Admin to forbid target #channel.<br/>
 *   That Channel can't be used (joined) or accessed by users anymore.<br/>
 *   It also prevents that the Channel will be registered at a later time (if it's not now)<br/>
 *   (Limited to Services Admins).
 *
 * Syntax is: /msg ChanServ FORBID #channel [reason]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_forbid(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//Channel c;
	
	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));
	
	/*
	 * Forbid Command
	 */
	 //Syntax: SUSPEND channel [reason]
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"forbid")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_FORBID_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_FORBIDDEN",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
        
		else if(/*thread->c.chan.serv[0]=='\0' && */~thread->c.chan.settings & CS_FORBID_BIT){
			if(!thread->c.chan.fields) 
				dynncpy(thread->c.chan.name,param[4],sizeof(thread->c.chan.name));
				
			thread->c.chan.settings |= CS_FORBID_BIT;
			
			if(m->WallForbid)
				do_global(thread,snick,"%s used forbid for channel %s",nick,thread->c.chan.name);
				
			if(thread->c.chan.fields){
				if(m->ForceForbidReason && param[5]){
					dynncpy(thread->c.chan.reason,_dstrjoin(' ',param,5),sizeof(thread->c.chan.reason));
					thread->c.chan.expire = 0;
					
					domysql(thread,"UPDATE `orion_cs_core` SET `settings`='%d',`reason`='%s',`serv`='ChanServ',`expire`='0' WHERE `channel`='%s'",thread->c.chan.settings,mysql_quote(thread,thread->c.chan.reason),mysql_quote(thread,thread->c.chan.name));
					send_lang(thread,language,howtosend,snick,nick,"CHANNEL_X_FORBIDDEN",thread->c.chan.name,thread->c.chan.reason);
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else if(!m->ForceForbidReason){
					if(param[5]){
						dynncpy(thread->c.chan.reason,_dstrjoin(' ',param,5),sizeof(thread->c.chan.reason));
						
						domysql(thread,"UPDATE `orion_cs_core` SET `settings`='%d',`reason`='%s',`serv`='ChanServ',`expire`='0' WHERE `channel`='%s'",thread->c.chan.settings,mysql_quote(thread,thread->c.chan.reason),mysql_quote(thread,thread->c.chan.name));
						send_lang(thread,language,howtosend,snick,nick,"CHANNEL_X_FORBIDDEN",thread->c.chan.name,thread->c.chan.reason);
					}else{
						dynncpy(thread->c.chan.reason,"None",sizeof(thread->c.chan.reason));
						
						domysql(thread,"UPDATE `orion_cs_core` SET `settings`='%d',`reason`='None',`serv`='ChanServ',`expire`='0' WHERE `channel`='%s'",thread->c.chan.settings,mysql_quote(thread,thread->c.chan.name));
						send_lang(thread,language,howtosend,snick,nick,"CHANNEL_X_FORBIDDEN",thread->c.chan.name,"");
					}
					thread->c.chan.expire = 0;
					dynncpy(thread->c.chan.serv,"ChanServ",sizeof(thread->c.chan.serv));
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}
				else send_lang(thread,language,howtosend,snick,nick,"CS_FORBID_NO_REASON_GIVEN");
			}else{
				if(m->ForceForbidReason && param[5]){
					dynncpy(thread->c.chan.reason,_dstrjoin(' ',param,5),sizeof(thread->c.chan.reason));
					thread->c.chan.expire = 0;
					dynncpy(thread->c.chan.serv,"ChanServ",sizeof(thread->c.chan.serv));
					
					domysql(thread,"INSERT INTO `orion_cs_core` (`settings`,`reason`,`serv`,`channel`,`expire`) VALUES ('%d','%s','ChanServ','%s','0')",thread->c.chan.settings,mysql_quote(thread,thread->c.chan.reason),mysql_quote(thread,thread->c.chan.name));
					send_lang(thread,language,howtosend,snick,nick,"CHANNEL_X_FORBIDDEN",thread->c.chan.name,thread->c.chan.reason);
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else if(!m->ForceForbidReason){
					if(param[5]){
						dynncpy(thread->c.chan.reason,_dstrjoin(' ',param,5),sizeof(thread->c.chan.reason));
						
						domysql(thread,"INSERT INTO `orion_cs_core` (`settings`,`reason`,`serv`,`channel`,`expire`) VALUES ('%d','%s','ChanServ','%s','0')",thread->c.chan.settings,mysql_quote(thread,thread->c.chan.reason),mysql_quote(thread,thread->c.chan.name));
						send_lang(thread,language,howtosend,snick,nick,"CHANNEL_X_FORBIDDEN",thread->c.chan.name,thread->c.chan.reason);
					}else{
						dynncpy(thread->c.chan.reason,"None",sizeof(thread->c.chan.reason));
						
						domysql(thread,"INSERT INTO `orion_cs_core` (`settings`,`reason`,`serv`,`channel`,`expire`) VALUES ('%d','None','ChanServ','%s','0')",thread->c.chan.settings,mysql_quote(thread,thread->c.chan.name));
						send_lang(thread,language,howtosend,snick,nick,"CHANNEL_X_FORBIDDEN",thread->c.chan.name,"");
					}
					thread->c.chan.expire = 0;
					dynncpy(thread->c.chan.serv,"ChanServ",sizeof(thread->c.chan.serv));
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}
				else send_lang(thread,language,howtosend,snick,nick,"CS_FORBID_NO_REASON_GIVEN");
			}
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_ALREADY_REGISTERED",thread->c.chan.serv);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the ChanServ FORBID Command (/msg ChanServ HELP FORBID)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP FORBID,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP FORBID
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_forbid_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Forbid Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help forbid")){
		send_lang(thread,language,howtosend,snick,nick,"CS_SERVADMIN_HELP_FORBID",snick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	else send_lang(thread,language,howtosend,snick,nick,"CS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief ChanServ Help List Command - FORBID part (/msg ChanServ help)
 * 
 * When the user tries to get general ChanServ help, using /msg ChanServ help, all loaded commands with <br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the FORBID command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_forbid_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_CS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Forbid (Help listing)
	 */
	if(Users_CS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_CMD_FORBID");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
