/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file cs_sop.c
 *   @brief This is the Module file for ChanServ SOP Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the ChanServ SOP command.
 *
 *   The ChanServ SOP Command allows the user to control the channels<br/>
 *   AutoSuperOP-List. (ChanServ modifies/dumps the sop list)<br/>
 *    AutoSuperOp = AutoOp + AutoProtect (+ao @ UnrealIRCd)
 *
 *   Syntax is: /msg ChanServ SOP #channel { ADD | DEL | LIST | CLEAR } [nick]
 *
 *<table border="0">
 *    <tr><td> ADD		</td><td> Syntax is: /msg ChanServ SOP #channel ADD		</td><td> < nick >	</td></tr>
 *    <tr><td> DEL  	</td><td> Syntax is: /msg ChanServ SOP #channel DEL		</td><td> < nick >	</td></tr>
 *    <tr><td> LIST 	</td><td> Syntax is: /msg ChanServ SOP #channel LIST	</td><td>			</td></tr>
 *    <tr><td> CLEAR	</td><td> Syntax is: /msg ChanServ SOP #channel CLEAR	</td><td>			</td></tr>
 *</table>

 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "webform.h"
#include "global.h"

#define SERVICE "cs"
#define MODUL "sop"
#define MODUL_FILE "cs_sop"
#define MODUL_VERSION "1.2"

MOD_BOOL cs_sop_add(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_sop_del(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_sop_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_sop_clear(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL cs_sop_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_sop_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"sop * option[1]","sop");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,WEBACCESS,LOGINONLY,OPER_IF_DEFCON);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);
	
	addLeastModuleLevels(thread,&Module,"sop",is_services_admin,is_services_user); /* just to add a displaylevel for /msg cs help
															  this should always be the lowest used Module level! */

	addCommand(thread,&Module,"sop add",cs_sop_add,is_services_admin,is_services_user,6,6);
	addCommand(thread,&Module,"sop del",cs_sop_del,is_services_admin,is_services_user,6,6);
	addCommand(thread,&Module,"sop list",cs_sop_list,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"sop clear",cs_sop_clear,is_services_admin,is_services_user,5,5);
	
	addCommand(thread,&Module,"help sop",cs_sop_help,is_services_user,is_services_user,4,4);
	addCommand(thread,&Module,"help",cs_sop_help_listing,is_services_user,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief CS SOP Add function for the ChanServ SOP Command.
 * 
 * The ChanServ SOP Add Command allows the user to add another user to<br/>
 *    the Channel's Access-List with Level SOP (ONLY SUCCESSES IF XOP IS ENABLED!)
 *  
 * Syntax is: /msg ChanServ SOP #channel ADD < nick >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_sop_add(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//Channel c;
	//Aliases aliases5;
	int sender_level = 0, nicks_level = 0;
	int NSAccess = 0, CSAccess = 0;

	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));
	//bzero(&aliases5,sizeof(aliases5));

	/*
	 * Sop Add Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"sop add")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],param[6]);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv") && (thread->c.chan.settings & CS_XOP_BIT)){
			if(CheckOption(thread,snick,nick,READONLY)){
				//aliases5     = sql_aliases(thread,param[6]);
				sender_level = stoi(thread->c.chan.access.ownlevel);
				nicks_level  = stoi(thread->c.chan.access.targetlevel);
								
				NSAccess     = stoi(mysql_sel_var(thread,"SELECT count(1) FROM `orion_cs_access` WHERE `authnick`='%s'",mysql_quote(thread,thread->c.aliases.authnick)));
				CSAccess     = stoi(mysql_sel_var(thread,"SELECT count(1) FROM `orion_cs_access` WHERE `channel`='%s'",mysql_quote(thread,param[4])));

				if(thread->c.aliases.fields && thread->c.aliases.authnick[0]!='\0'){
					if(NSAccess<m->NSAccessMax && CSAccess<m->CSAccessMax){
						if((sender_level>=nicks_level && sender_level>=ACCESS_SOP) || streqnc(thread->c.chan.founder,thread->sql.access.authnick) || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"sop add")){
							if(nicks_level>IDENTIFIED_NO_ACCESS){
								domysql(thread,"UPDATE `orion_cs_access` LEFT OUTER JOIN `orion_cs_core` ON (`orion_cs_access`.`channel`=`orion_cs_core`.`channel`) SET `orion_cs_core`.`last_used`='%ld', `orion_cs_access`.`level`='%d' WHERE `orion_cs_access`.`serv`='ChanServ' AND `orion_cs_access`.`authnick`='%s' AND `orion_cs_access`.`channel`='%s'",time(NULL),ACCESS_SOP,mysql_quote(thread,thread->c.aliases.authnick),mysql_quote(thread,param[4]));

								if(!thread->Webform)
									do_logchan(thread,snick,NULL,NULL,NULL,"%s!%s@%s level (%d) set access level %d to %s (group %s) on channel %s",nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,sender_level,ACCESS_SOP,thread->c.aliases.nick,thread->c.aliases.authnick,thread->c.chan.name);
								else
									do_logchan(thread,snick,NULL,NULL,NULL,"WEB: %s level (%d) set access level %d to %s (group %s) on channel %s",nick,sender_level,ACCESS_SOP,thread->c.aliases.nick,thread->c.aliases.authnick,thread->c.chan.name);
								
								send_lang(thread,language,howtosend,snick,nick,"CS_SOP_MOVED",thread->c.aliases.authnick,thread->c.chan.name);
								setWebSuccess(thread); /* Indentify webform that changes were applied */
							}
							else{
								domysql(thread,"INSERT INTO `orion_cs_access` (`serv`,`channel`,`authnick`,`level`) VALUES ('ChanServ','%s','%s','%d')",mysql_quote(thread,thread->c.chan.name),mysql_quote(thread,thread->c.aliases.authnick),ACCESS_SOP);

								if(!thread->Webform)
									do_logchan(thread,snick,NULL,NULL,NULL,"%s!%s@%s level (%d) set access level %d to %s (group %s) on channel %s",nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,sender_level,ACCESS_SOP,thread->c.aliases.nick,thread->c.aliases.authnick,thread->c.chan.name);
								else
									do_logchan(thread,snick,NULL,NULL,NULL,"WEB: %s level (%d) set access level %d to %s (group %s) on channel %s",nick,sender_level,ACCESS_SOP,thread->c.aliases.nick,thread->c.aliases.authnick,thread->c.chan.name);
									
								send_lang(thread,language,howtosend,snick,nick,"CS_SOP_ADDED",thread->c.aliases.authnick,thread->c.chan.name);
								setWebSuccess(thread); /* Indentify webform that changes were applied */
							}
							dynncpy(thread->c.chan.access.targetlevel,_dynadd("%d",ACCESS_SOP),sizeof(thread->c.chan.access.targetlevel));
						}
						else send_lang(thread,language,howtosend,snick,nick,"ACCESS_IN_CHANNEL_TOO_LOW",sender_level,param[4]);
					}else send_lang(thread,language,howtosend,snick,nick,"ACCESS_LIMIT_X_FOR_NICK_OR_X_CHANNEL_REACHED",m->NSAccessMax,m->CSAccessMax);
				}else send_lang(thread,language,howtosend,snick,nick,"NO_AUTHNICK");
			}else send_lang(thread,language,howtosend,snick,nick,"READONLY_MODE");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_OR_NO_XOP",snick);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SOP Del function for the ChanServ SOP Command.
 * 
 * The ChanServ SOP Del Command allows the user to delete another user from<br/>
 *    the Channel's SOP-List (ONLY SUCCESSES IF XOP IS ENABLED!)
 *  
 * Syntax is: /msg ChanServ SOP #channel DEL < nick >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_sop_del(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//Channel c;
	//Aliases aliases5;
	int sender_level = 0, nicks_level = 0;

	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));
	//bzero(&aliases5,sizeof(aliases5));

	/*
	 * Sop Del Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"sop del")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],param[6]);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv") && (thread->c.chan.settings & CS_XOP_BIT)){
			if(CheckOption(thread,snick,nick,READONLY)){
				//aliases5     = sql_aliases(thread,param[6]);
				sender_level = stoi(thread->c.chan.access.ownlevel);
				nicks_level  = stoi(thread->c.chan.access.targetlevel);
				
				if((sender_level>=nicks_level && sender_level>=ACCESS_SOP) || streqnc(thread->c.chan.founder,thread->sql.access.authnick) || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"sop del")){
					if(nicks_level==ACCESS_SOP){
						domysql(thread,"DELETE FROM `orion_cs_access` WHERE `serv`='ChanServ' AND `authnick`='%s' AND `channel`='%s'",mysql_quote(thread,thread->c.aliases.authnick),mysql_quote(thread,param[4]));
						
						if(!thread->Webform)
							do_logchan(thread,snick,NULL,NULL,NULL,"%s!%s@%s level (%d) deleted %s (group %s) on channel %s",nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,sender_level,thread->c.aliases.nick,thread->c.aliases.authnick,thread->c.chan.name);
						else
							do_logchan(thread,snick,NULL,NULL,NULL,"WEB: %s level (%d) deleted %s (group %s) on channel %s",nick,sender_level,thread->c.aliases.nick,thread->c.aliases.authnick,thread->c.chan.name);

						send_lang(thread,language,howtosend,snick,nick,"CS_SOP_DELETED",thread->c.aliases.authnick,param[4]);
						bzero(&thread->c.chan.access.targetlevel,sizeof(thread->c.chan.access.targetlevel));
						
						setWebSuccess(thread); /* Indentify webform that changes were applied */
					}else send_lang(thread,language,howtosend,snick,nick,"CS_SOP_LEVEL_USERS_LEVEL_ISNOT_SOP");
				}
				else send_lang(thread,language,howtosend,snick,nick,"ACCESS_IN_CHANNEL_TOO_LOW",sender_level,param[4]);
			}else send_lang(thread,language,howtosend,snick,nick,"READONLY_MODE");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_OR_NO_XOP",snick);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SOP List function for the ChanServ SOP Command.
 * 
 * The ChanServ SOP List Command allows the user to dump the List of<br/>
 *    the Channel's SOP-List (ONLY SUCCESSES IF XOP IS ENABLED!)
 *  
 * Syntax is: /msg ChanServ SOP #channel LIST
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_sop_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	deepSQL sops;
	//Channel c;
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&sops,sizeof(sops));
	//bzero(&c,sizeof(c));

	/*
	 * Sop List Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"sop list")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv") && (thread->c.chan.settings & CS_XOP_BIT)){
			
			if(stoi(thread->c.chan.access.ownlevel)>=ACCESS_AOP || streqnc(thread->c.chan.founder,thread->sql.access.authnick) || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"sop list")){
				sops.res    = doSelectMySQL(thread,"SELECT `authnick` FROM `orion_cs_access` WHERE `level`='%d'",ACCESS_SOP);
				sops.fields = mysql_num_rows(sops.res);
				
				send_lang(thread,language,howtosend,snick,nick,"CS_SOP_LIST_HEADER",param[4]);
				for(i=0;i<sops.fields;i++){
					sops.row = mysql_fetch_row(sops.res);
					
					send_lang(thread,language,howtosend,snick,nick,"CS_SOP_LIST_ENTRY",i,sops.row[0],ACCESS_SOP);
				}

				mysql_free_result_myr(sops.res);
				send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");
				
				setWebSuccess(thread); /* Indentify webform that changes were applied */
			}else send_lang(thread,language,howtosend,snick,nick,"ACCESS_IN_CHANNEL_TOO_LOW",stoi(thread->c.chan.access.ownlevel),param[4]);
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_OR_NO_XOP",snick);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SOP Clear function for the ChanServ SOP Command.
 * 
 * The ChanServ SOP Clear Command allows the user to clear<br/>
 *    the Channel's SOP-List (ONLY SUCCESSES IF XOP IS ENABLED!)
 *  
 * Syntax is: /msg ChanServ SOP #channel CLEAR
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_sop_clear(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	int sender_level = 0;
	
	/* Initialise Variables to (NULL) */

	/*
	 * Sop Clear Command
	 */
 	// :shock PRIVMSG chanserv :sop #channel clear
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"sop clear")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv") && (thread->c.chan.settings & CS_XOP_BIT)){
			if(CheckOption(thread,snick,nick,READONLY)){
				sender_level = stoi(thread->c.chan.access.ownlevel);
				
				if(sender_level>=ACCESS_FOUNDER || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"sop clear") || streqnc(thread->c.chan.founder,thread->sql.access.authnick)){
					domysql(thread,"DELETE FROM `orion_cs_access` WHERE `serv`='ChanServ' AND `level`='%d' AND `channel`='%s'",ACCESS_SOP,mysql_quote(thread,param[4]));

					if(!thread->Webform)
						do_logchan(thread,snick,NULL,NULL,NULL,"%s!%s@%s level (%d) cleared SOP-List on channel %s",nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,sender_level,thread->c.chan.name);
					else
						do_logchan(thread,snick,NULL,NULL,NULL,"%s level (%d) cleared SOP-List on channel %s",nick,sender_level,thread->c.chan.name);

					send_lang(thread,language,howtosend,snick,nick,"CS_SOP_CLEARED",thread->sql.access.authnick,param[4]);
					bzero(&thread->c,sizeof(thread->c));
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}
				else send_lang(thread,language,howtosend,snick,nick,"ACCESS_IN_CHANNEL_TOO_LOW",sender_level,param[4]);
			}else send_lang(thread,language,howtosend,snick,nick,"READONLY_MODE");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_OR_NO_XOP",snick);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the ChanServ SOP Command (/msg ChanServ HELP SOP)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP SOP,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP SOP
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_sop_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Sop Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help sop")){
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_SOP",snick,snick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"CS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief ChanServ Help List Command - SOP part (/msg ChanServ help)
 * 
 * When the user tries to get general ChanServ help, using /msg ChanServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the SOP command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_sop_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_CS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Sop (Help listing)
	 */
	if(Users_CS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_CMD_SOP");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
