/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file cs_unban.c
 *   @brief This is the Module file for ChanServ UNBAN Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the ChanServ UNBAN command.
 *
 *   The ChanServ UNBAN Command allows the user to remove all bans preventing<br/>
 *   him from entering the given channel.
 *
 *   Syntax is: /msg ChanServ UNBAN #channel
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"
#include "webform.h"

#define SERVICE "cs"
#define MODUL "unban"
#define MODUL_FILE "cs_unban"
#define MODUL_VERSION "1.2"

MOD_BOOL cs_unban(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_unban_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_unban_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"unban","unban");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,LOGINONLY,OPER_IF_DEFCON,READONLY);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"unban",cs_unban,is_services_admin,is_services_user,4,5);
	addCommand(thread,&Module,"help unban",cs_unban_help,is_services_user,is_services_user,4,4);
	addCommand(thread,&Module,"help",cs_unban_help_listing,is_services_user,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the ChanServ UNBAN Command.
 * 
 * The ChanServ UNBAN Command allows the user to remove all bans preventing<br/>
 *   him from entering the given channel.
 *
 * Syntax is: /msg ChanServ UNBAN #channel
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_unban(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	char CBot[MAX_NICK_LEN];
	deepSQL bans;
	Users *IRCUser;

	int i=0;
	int j=0, pos=0;
	char nicks[BUFFER];
	char mode[4];
	char host[MAX_HOST_LEN+MAX_NICK_LEN+MAX_IDENT_LEN+1+1+1];

	/* Initialise Variables to (NULL) */
	bzero(&bans,sizeof(bans));
	bzero(&CBot,sizeof(CBot));
	bzero(&mode,sizeof(mode));
	bzero(&nicks,sizeof(nicks));
	bzero(&host,sizeof(host));
	IRCUser = NULL;

	/*
	 * Unban Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"unban")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		/* Set ChanServ to our Bot, if BotServ is not loaded, or no Bot is assigned, or if FANTASY is OFF */
		CSCheckBot(CBot,thread->c.chan.bot,thread->c.chan.botserv.set,sizeof(CBot));
		
		IRCUser = GetUser(nick);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv")){
			if(IRCUser!=NULL){
				if(thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"unban") || stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.unban){
				
					bzero(&host,sizeof(host));
					straddns(host,IRCUser->nick,sizeof(host));
					straddnc(host,'!',sizeof(host));
					straddns(host,IRCUser->ident,sizeof(host));
					straddnc(host,'@',sizeof(host));
					straddns(host,IRCUser->vhost,sizeof(host));
				
					//msend(thread,0,cs->nick,nick,"SELECT `ban` FROM `orion_chan_bans` WHERE '%s' LIKE REPLACE(`ban`,'*','%c')",host,'%');
				
					bans.res	= doSelectMySQL(thread,"SELECT `ban` FROM `orion_chan_bans` WHERE '%s' LIKE REPLACE(`ban`,'*','%c')",host,'%');
					bans.fields = mysql_num_rows(bans.res);

					if(bans.fields){
						for(j=0;j<bans.fields;j++){
							bans.row = mysql_fetch_row(bans.res);
						
							if(i==0){
								bzero(&mode,sizeof(mode));
								bzero(&nicks,sizeof(nicks));
							}
							if(i<3){
								straddnc(mode,IRC.BanMode,sizeof(mode));
								straddns(nicks,bans.row[0],sizeof(nicks));
								straddnc(nicks,' ',sizeof(nicks));
								i++;
							}
							if(i==3 || pos >= (bans.fields-1)){
								do_mode(thread,CBot,thread->c.chan.name,thread->c.chan.datecreate,_dynadd("-%s %s",mode,nicks));
								i=0;
							}
							pos++;
						}
						send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
					}else send_lang(thread,language,howtosend,snick,nick,"CS_UNBAN_NO_BANS",thread->c.chan.name);
				
					mysql_free_result_myr(bans.res);
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_CONNECTED_TO_IRCSERVER");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_CHANSERV");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the ChanServ UNBAN Command (/msg ChanServ HELP UNBAN)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP UNBAN,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP UNBAN
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_unban_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Unban Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help unban")){
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_UNBAN");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}

	else send_lang(thread,language,howtosend,snick,nick,"CS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief ChanServ Help List Command - UNBAN part (/msg ChanServ help)
 * 
 * When the user tries to get general ChanServ help, using /msg ChanServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the UNBAN command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_unban_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_CS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Unban (Help listing)
	 */
	if(Users_CS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_CMD_UNBAN");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
