/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Chrisostomos Adamos (alias: Ally), Email: Ally@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file hs_setall.c
 *   @brief This is the Module file for HostServ SETALL Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the HostServ SETALL command.
 *
 *   The HostServ SET Command allows the user to set the given hostmask as the new vHost<br/>
 *   for all nicks in the same group as that of the given nick.<br/>
 *   If your IRCD supports vIdents, then using SETALL < nick > < ident >@< hostmask ><br/>
 *   will set Idents for users as well as vHosts.
 *
 *   Syntax is: /msg HostServ SETALL < nick > < hostmask >
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"
#include "webform.h"

#define SERVICE "hs"
#define MODUL "setall"
#define MODUL_FILE "hs_setall"
#define MODUL_VERSION "1.2"

MOD_BOOL hs_setall(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL hs_setall_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL hs_setall_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module;
	
	if(!IRC.VHost){
		Module.ModErr = MOD_ERR_CANCELLED;
		Module.module = NULL;
		return Module;
	}
	
	Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"setall","setall");

	addAuthor (thread,&Module,"Ally, cLiEbiG, (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,5,WEBACCESS,LOGINONLY,OPER_IF_DEFCON,OPERONLY,READONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"setall",hs_setall,is_services_oper,INFINIT,5,5);
	addCommand(thread,&Module,"help setall",hs_setall_help,is_services_oper,INFINIT,4,4);
	addCommand(thread,&Module,"help",hs_setall_help_listing,is_services_oper,INFINIT,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

//: shock PRIVMSG HostServ :setall bla
/*  ************************************************************************************************  */

/**
 * @brief Main function for the HostServ SETALL Command.
 * 
 * The HostServ SET Command allows the user to set the given hostmask as the new vHost<br/>
 *   for all nicks in the same group as that of the given nick.<br/>
 *   If your IRCD supports vIdents, then using SETALL < nick > < ident >@< hostmask ><br/>
 *   will set Idents for users as well as vHosts.
 *
 * Syntax is: /msg HostServ SETALL < nick > < hostmask >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL hs_setall(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
    deepSQL getaliases;
    deepSQL vhostgroup;
    deepSQL checkvhost;
    Aliases aliases5;

    char *HostParts[2];
    char* vIdent = NULL;
    char* vHost  = NULL;
    char tempnick[MAX_IDENT_LEN+MAX_HOST_LEN+1];
    char virthost[MAX_HOST_LEN];
    size_t parts = 0;
    bool ret;
    int i = 0;
    
	/* Initialise Variables to (NULL) */
	bzero(&getaliases,sizeof(getaliases));
	bzero(&vhostgroup,sizeof(vhostgroup));
	bzero(&checkvhost,sizeof(checkvhost));
	bzero(&aliases5,sizeof(aliases5));
	bzero(HostParts,sizeof(HostParts));
	bzero(&tempnick,sizeof(tempnick));
	bzero(&virthost,sizeof(virthost));
	
	/*
	 * Setall Command
	 */
	 //> :Ally PRIVMSG hostserv@services.deepirc.net :setall bla2 muh.kuh.ha
   	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"setall")){

		aliases5 = sql_aliases(thread,param[4]);
		
		vhostgroup.res    = doSelectMySQL(thread,"SELECT `nick` FROM `orion_hs_core` WHERE `display`='%s' ORDER BY `nick`",mysql_quote(thread,aliases5.authnick));
		vhostgroup.fields = mysql_num_rows(vhostgroup.res);
		
		getaliases.res    = doSelectMySQL(thread,"SELECT `nick` FROM `orion_ns_aliases` WHERE `display`='%s' ORDER BY `nick`",mysql_quote(thread,aliases5.authnick));
		getaliases.fields = mysql_num_rows(getaliases.res);
		
        checkvhost.res    = doSelectMySQL(thread,"SELECT `nick`, `active`, `vhost` FROM `orion_hs_core` WHERE `nick`='%s'",mysql_quote(thread,param[4]));
        checkvhost.fields = mysql_num_rows(checkvhost.res);
		checkvhost.row    = mysql_fetch_row(checkvhost.res);

		dynncpy(virthost,param[5],sizeof(virthost));
		parts = strsplit(virthost,HostParts,'@');
		if(parts==2){ /* vIdent + vHost */
			vIdent = HostParts[0];
			vHost = HostParts[1];
		}else if(parts==1){  /* nur vHost */
			vHost = HostParts[0];
		}

		if(vIdent && !IRC.SupportVIdents){
			printf("ircd does not allow vIdents");
		}else if(vIdent && IRC.SupportVIdents){
			if(dynlen(vIdent)>MAX_IDENT_LEN){
				send_lang(thread,language,howtosend,snick,nick,"HS_REQUEST_VIDENT_TOO_LONG",snick);
				//printf("ident too long!");

				/* mysql_free_result_myr(); - getaliases, vhostgroup, checkvhost */
				mysql_free_result_myr(getaliases.res);
				mysql_free_result_myr(vhostgroup.res);
				mysql_free_result_myr(checkvhost.res);
				return MOD_CONT;
			}

			//printf("VIDENT FOUND: %s\n",vIdent);
			ret = is_valid_chars(vIdent,"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-.");
			
			if(ret == 1){
				send_lang(thread,language,howtosend,snick,nick,"HS_REQUEST_VIDENT_INVALID",snick);

				/* mysql_free_result_myr(); - getaliases, vhostgroup, checkvhost */
				mysql_free_result_myr(getaliases.res);
				mysql_free_result_myr(vhostgroup.res);
				mysql_free_result_myr(checkvhost.res);
				return MOD_CONT;
			}
			snprintf(tempnick,sizeof(tempnick),"%s",vIdent);
		}
		if(vHost){
			if(dynlen(vHost)>MAX_HOST_LEN){
				send_lang(thread,language,howtosend,snick,nick,"HS_REQUEST_VHOST_TOO_LONG",snick);
				//printf("host too long!");

				/* mysql_free_result_myr(); - getaliases, vhostgroup, checkvhost */
				mysql_free_result_myr(getaliases.res);
				mysql_free_result_myr(vhostgroup.res);
				mysql_free_result_myr(checkvhost.res);
				return MOD_CONT;
			}
			
			printf("VHOST FOUND: %s\n",vHost);
			ret = is_valid_chars(vHost,"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-.");
			
			if(ret == 1){
				send_lang(thread,language,howtosend,snick,nick,"HS_REQUEST_VHOST_INVALID",snick);

				/* mysql_free_result_myr(); - getaliases, vhostgroup, checkvhost */
				mysql_free_result_myr(getaliases.res);
				mysql_free_result_myr(vhostgroup.res);
				mysql_free_result_myr(checkvhost.res);
				return MOD_CONT;
			}
			if(vIdent)
				snprintf(tempnick,sizeof(tempnick),"%s%s%s",tempnick,"@",vHost);
			else
				snprintf(tempnick,sizeof(tempnick),"%s",vHost);
		}


		if(checkvhost.fields){
			if(getaliases.fields == 1) domysql(thread,"UPDATE `orion_hs_core` SET `vhost` = '%s', `changetime`='%ld', `authorized`='%s', `active`='1' WHERE `display`='%s'",mysql_quote(thread,param[5]),time(NULL),mysql_quote(thread,nick),mysql_quote(thread,aliases5.authnick));
			else{
//			  vhostgroup.fields
				for(i = 0; i < getaliases.fields && vhostgroup.fields; i++){
					getaliases.row = mysql_fetch_row(getaliases.res);
					vhostgroup.row = mysql_fetch_row(vhostgroup.res);
                      
					if(streqnc(getaliases.row[0],vhostgroup.row[0])){
						domysql(thread,"UPDATE `orion_hs_core` SET `vhost` = '%s', `changetime`='%ld', `authorized`='%s', `active`='1' WHERE `display`='%s'",mysql_quote(thread,param[5]),time(NULL),mysql_quote(thread,thread->sql.aliases.authnick),mysql_quote(thread,aliases5.authnick));
					}else{
						domysql(thread,"INSERT INTO `orion_hs_core` (`nick`,`display`,`nick_type`,`vhost`,`authorized`,`changetime`,`active`) VALUES ('%s','%s','%d','%s','%s','%ld','1')",mysql_quote(thread,getaliases.row[0]),mysql_quote(thread,aliases5.authnick),0,mysql_quote(thread,tempnick),mysql_quote(thread,thread->sql.aliases.authnick),time(NULL));
					}

				}
			}

			if(stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_core_condata` WHERE `nick`='%s'",mysql_quote(thread,param[4])))){
				if(parts==2){ /* vIdent + vHost */
    				do_chgident(thread,snick,param[4],vIdent);
    				do_chghost(thread,snick,param[4],vHost);
					send_lang(thread,language,howtosend,snick,nick,"HS_ADD_SUCCESS",_dynadd("%s@%s",vIdent,vHost),snick);
				}else if(parts==1){  /* nur vHost */
    				do_chghost(thread,snick,param[4],vHost);
					send_lang(thread,language,howtosend,snick,nick,"HS_ADD_SUCCESS",vHost,param[4]);
				}
			}
			send_lang(thread,language,howtosend,snick,nick,"HS_SETALL_SUCCESS",param[5],aliases5.authnick);
			
			setWebSuccess(thread); /* Indentify webform that changes were applied */
		}else{
			for(i = 0; i < getaliases.fields; i++){
				getaliases.row = mysql_fetch_row(getaliases.res);
				domysql(thread,"INSERT INTO `orion_hs_core` (`nick`,`display`,`nick_type`,`vhost`,`authorized`,`changetime`,`active`) VALUES ('%s','%s','%d','%s','%s','%d','%d')",mysql_quote(thread,getaliases.row[0]),mysql_quote(thread,aliases5.authnick),0,mysql_quote(thread,tempnick),mysql_quote(thread,thread->sql.aliases.authnick),time(NULL),1);
			}

			if(GetUser(param[4])!=NULL){
				if(parts==2){ /* vIdent + vHost */
    				do_chgident(thread,snick,param[4],vIdent);
    				do_chghost(thread,snick,param[4],vHost);
    				
    				send_lang(thread,language,howtosend,snick,nick,"HS_SETALL_SUCCESS",_dynadd("%s@%s",vIdent,vHost),aliases5.authnick);
    				
    				setWebSuccess(thread); /* Indentify webform that changes were applied */
    			}else if(parts==1){  /* nur vHost */
    				do_chghost(thread,snick,param[4],vHost);
    				send_lang(thread,language,howtosend,snick,nick,"HS_SETALL_SUCCESS",vHost,aliases5.authnick);
    				
    				setWebSuccess(thread); /* Indentify webform that changes were applied */
    			}else
    				send_lang(thread,language,howtosend,snick,nick,"HS_SETALL_ERROR",tempnick,aliases5.authnick);
    		}
    		else send_lang(thread,language,howtosend,snick,nick,"NOT_KNOWN_USER",param[4]);
    	}
    	
		/* mysql_free_result_myr(); - getaliases, vhostgroup, checkvhost */
		mysql_free_result_myr(getaliases.res);
		mysql_free_result_myr(vhostgroup.res);
		mysql_free_result_myr(checkvhost.res);
    }else{
	/*
	 * Error Messages
	 */
 		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the HostServ SETALL Command (/msg HostServ HELP SETALL)
 * 
 * When the user tries to get help for this command, using /msg HostServ HELP SETALL,<br/>
 * this function is executed.
 * 
 * Syntax: /msg HostServ HELP SETALL
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL hs_setall_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Setall Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help setall"))
		send_lang(thread,language,howtosend,snick,nick,"HS_HELP_SETALL",snick);
	else
	    send_lang(thread,language,howtosend,snick,nick,"HS_NO_HELP_AVAIL",snick);
		
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief HostServ Help List Command - SETALL part (/msg HostServ help)
 * 
 * When the user tries to get general HostServ help, using /msg HostServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the SETALL command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL hs_setall_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_HS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Setall (Help Listing)
	 */
	if(Users_HS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"HS_HELP_CMD_SETALL");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
