/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file ns_getpass.c
 *   @brief This is the Module file for NickServ GETPASS Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the NickServ GETPASS command.
 *
 *   The NickServ GETPASS Command allows an admin to dump the password for target < nick ><br/>
 *   (This use is limited to ServicesAdmins).
 *
 *   Syntax is: /msg NickServ GETPASS < nick >
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"
#include "webform.h"

#define SERVICE "ns"
#define MODUL "getpass"
#define MODUL_FILE "ns_getpass"
#define MODUL_VERSION "1.2"

MOD_BOOL ns_getpass(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_getpass_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_getpass_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"getpass","getpass");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,4,WEBACCESS,LOGINONLY,OPER_IF_DEFCON,OPERONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	if(streqnc(orion_check_encryption(),NO_ENCRYPT)){
		addCommand(thread,&Module,"getpass",ns_getpass,is_services_admin,INFINIT,4,4);
		addCommand(thread,&Module,"help getpass",ns_getpass_help,is_services_admin,INFINIT,4,4);
		addCommand(thread,&Module,"help",ns_getpass_help_listing,is_services_admin,INFINIT,3,3);
	}

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the NickServ GETPASS Command.
 * 
 * The NickServ GETPASS Command allows an admin to dump the password for target < nick ><br/>
 *   (This use is limited to ServicesAdmins).
 *
 * Syntax is: /msg NickServ GETPASS < nick >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_getpass(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	Aliases aliases5;
	Access access5;

	/* Initialise Variables to (NULL) */
	bzero(&aliases5,sizeof(aliases5));
	bzero(&access5,sizeof(access5));
	
	/*
	 * Getpass Command
	 */
	if(((!m->NSRestrictGetPass && thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"getpass")) || (m->NSRestrictGetPass && thread->sql.access.rights>=is_services_root))){
		aliases5 = sql_aliases(thread,param[4]);
			
		if(aliases5.fields && aliases5.authnick){
			bzero(&access5,sizeof(access5));
			access5 = sql_access(thread,aliases5.authnick);
			
			if(!m->NSSecureAdmins || (m->NSSecureAdmins && access5.rights<=is_services_oper)){	
				if(access5.olevel <= thread->sql.access.olevel && access5.authnick){
					send_lang(thread,language,howtosend,snick,nick,"NS_GETPASS_PASSWORD_AUTHNICK",param[4],aliases5.authnick,access5.password);
					if(m->WallGetpass)
						do_global(thread,snick,"%s used GETPASS on %s",nick,param[4]);
						
					if(!thread->Webform)
						do_logchan(thread,snick,nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,"used GETPASS on %s",param[4]);
					else
						do_logchan(thread,snick,nick,NULL,NULL,"used GETPASS on %s",param[4]);
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}
	            else send_lang(thread,language,howtosend,snick,nick,"NS_GETPASS_LEVEL_AUTHNICK_HIGHER_THAN_YOURS",param[4],aliases5.authnick);
			}else send_lang(thread,language,howtosend,snick,nick,"NS_GETPASS_CANT_GETPASS_SA_SRA_X",aliases5.authnick);
		}else send_lang(thread,language,howtosend,snick,nick,"NOT_KNOWN_USER");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the NickServ GETPASS Command (/msg NickServ HELP GETPASS)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP GETPASS,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP GETPASS
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_getpass_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Getpass Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help getpass")){
		send_lang(thread,language,howtosend,snick,nick,"NS_SERVADMIN_HELP_GETPASS");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief NickServ Help List Command - GETPASS part (/msg NickServ help)
 * 
 * When the user tries to get general NickServ help, using /msg NickServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the GETPASS command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_getpass_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_NS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Getpass (Help Listing)
	 */
	if(Users_NS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_CMD_GETPASS");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
