/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file o_admin.c
 *   @brief This is the Module file for O ADMIN Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the O ADMIN command.
 *
 *   The O ADMIN Command allows an admin to manage the AdminList.<br/>
 *   (add, delete, view ServicesAdmins - Add and Del are limited to Services Roots)
 *
 *   Syntax is: /msg O ADMIN { ADD | DEL | LIST } [< nick >]
 *
 *<table border="0">
 *    <tr><td> ADD	</td><td> Syntax is: /msg O ADMIN ADD	</td><td> < nick > { SRA | SA }	</td><td> (Allows a ServicesRoot to add a ServicesRoot / ServicesAdmin)		</td></tr>
 *    <tr><td> DEL	</td><td> Syntax is: /msg O ADMIN DEL	</td><td> < nick >				</td><td> (Allows a ServicesRoot to delete a ServicesRoot / ServicesAdmin)	</td></tr>
 *    <tr><td> LIST	</td><td> Syntax is: /msg O ADMIN LIST	</td><td>                       </td><td> (Allows a ServicesRoot / ServicesAdmin to view all Admins/Roots)	</td></tr>
 *</table>
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"
#include "webform.h"

#define SERVICE "O"
#define MODUL "admin"
#define MODUL_FILE "o_admin"
#define MODUL_VERSION "1.2"

MOD_BOOL O_admin_add(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_admin_del(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_admin_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL O_admin_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_admin_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"admin option[1]","admin");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,5,WEBACCESS,LOGINONLY,OPER_IF_DEFCON,OPERONLY,READONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addLeastModuleLevels(thread,&Module,"admin",O_SRA_LEVEL,INFINIT); /* just to add a displaylevel for /msg o help
															  this should always be the lowest used Module level! */
	
	addCommand(thread,&Module,"admin add",O_admin_add,O_SRA_LEVEL,INFINIT,6,6);
	addCommand(thread,&Module,"admin del",O_admin_del,O_SRA_LEVEL,INFINIT,5,5);
	addCommand(thread,&Module,"admin list",O_admin_list,O_SRA_LEVEL,INFINIT,4,4);
	
	addCommand(thread,&Module,"help admin",O_admin_help,O_SRA_LEVEL,INFINIT,4,4);
	addCommand(thread,&Module,"help",O_admin_help_listing,O_SRA_LEVEL,INFINIT,3,3); //100 - each Admin is able to see it.

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief O ADMIN Add function for the O ADMIN Command.
 * 
 * The O ADMIN Add Command allows and admin to add another Admin as an ServicesAdmin or ServicesRoot
 *  
 * Syntax is: /msg O ADMIN ADD < nick > { SRA | SA }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_admin_add(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	Aliases aliases5;
	Access  access5;

	/* Initialise Variables to (NULL) */
	bzero(&aliases5,sizeof(aliases5));
	bzero(&access5,sizeof(access5));

	/*
	 * Admin Add Command
	 */
	 //:shock PRIVMSG O :admin add shock SRA
	
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"admin add")){
		aliases5 = sql_aliases(thread,param[5]);
		access5  = sql_access(thread,aliases5.authnick);
			
		if(aliases5.authnick[0]!='\0'){
			if(streqnc(param[6],"sra")){
				if(access5.fields && thread->sql.access.olevel>=O_SRA_LEVEL && thread->sql.access.qlevel>=Q_SRA_LEVEL){
					if(access5.rights<is_services_root && access5.olevel<O_SRA_LEVEL && access5.qlevel<Q_SRA_LEVEL){
						domysql(thread,"UPDATE `orion_ns_core` SET `qlevel`='%d',`olevel`='%d',`llevel`='256',`slevel`='%d',`rights`='%d' WHERE `authnick`='%s'",Q_SRA_LEVEL,O_SRA_LEVEL,S_SRA_LEVEL,is_services_root,mysql_quote(thread,aliases5.authnick));
						send_lang(thread,language,howtosend,snick,nick,"O_ADMIN_ADD_ADDED_X_TO_SRA",aliases5.authnick,Q_SRA_LEVEL,O_SRA_LEVEL,S_SRA_LEVEL,nick);
						
						setWebSuccess(thread); /* Indentify webform that changes were applied */
					}else send_lang(thread,language,howtosend,snick,nick,"USERS_LEVEL_IS_HIGHER_THAN_SA");
				}else send_lang(thread,language,howtosend,snick,nick,"NOT_A_SERVICES_ROOT");
			}
			else if(streqnc(param[6],"sa")){
				if(thread->sql.access.olevel>=O_SA_LEVEL && thread->sql.access.qlevel>=Q_SA_LEVEL){
					if(access5.rights<is_services_admin && access5.olevel<O_SA_LEVEL && access5.qlevel<Q_SA_LEVEL){
						domysql(thread,"UPDATE `orion_ns_core` SET `qlevel`='%d',`olevel`='%d',`llevel`='256',`slevel`='%d',`rights`='%d' WHERE `authnick`='%s'",Q_SA_LEVEL,O_SA_LEVEL,S_SA_LEVEL,is_services_admin,mysql_quote(thread,aliases5.authnick));
						send_lang(thread,language,howtosend,snick,nick,"O_ADMIN_ADD_ADDED_X_TO_SA",aliases5.authnick,Q_SA_LEVEL,O_SA_LEVEL,S_SA_LEVEL,nick);
						
						setWebSuccess(thread); /* Indentify webform that changes were applied */
					}else send_lang(thread,language,howtosend,snick,nick,"USERS_LEVEL_IS_HIGHER_THAN_SA");
				}else send_lang(thread,language,howtosend,snick,nick,"NOT_A_SERVICES_ADMIN");
			}else send_lang(thread,language,howtosend,snick,nick,"INVALID_OPTION",&param[3][1]);
		}else send_lang(thread,language,howtosend,snick,nick,"NOT_KNOWN_USER",aliases5.nick);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O ADMIN Del function for the O ADMIN Command.
 * 
 * The O ADMIN Del Command allows and admin to delete another Admin from the Admin-List
 *  
 * Syntax is: /msg O ADMIN DEL < nick >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_admin_del(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	Aliases aliases5;
	Access  access5;

	/* Initialise Variables to (NULL) */
	bzero(&aliases5,sizeof(aliases5));
	bzero(&access5,sizeof(access5));

	/*
	 * Admin Del Command
	 */
	 //:shock PRIVMSG O :admin add shock SRA
	
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"admin del")){
		aliases5 = sql_aliases(thread,param[5]);
		access5  = sql_access(thread,aliases5.authnick);
			
		if(aliases5.authnick[0]!='\0'){
			if(access5.fields && thread->sql.access.olevel>=access5.olevel && thread->sql.access.qlevel>=access5.qlevel && thread->sql.access.rights>=access5.rights){
				domysql(thread,"UPDATE `orion_ns_core` SET `qlevel`='1',`olevel`='0',`llevel`='1',`slevel`='1',`rights`='1' WHERE `authnick`='%s'",mysql_quote(thread,aliases5.authnick));
				send_lang(thread,language,howtosend,snick,nick,"O_ADMIN_DEL_DELETED_X_FROM_ADMINS",aliases5.authnick,nick);
				
				setWebSuccess(thread); /* Indentify webform that changes were applied */
			}else send_lang(thread,language,howtosend,snick,nick,"LEVEL_NOT_HIGH_ENOUGH");
		}else send_lang(thread,language,howtosend,snick,nick,"NOT_KNOWN_USER",aliases5.authnick);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O ADMIN List function for the O ADMIN Command.
 * 
 * The O ADMIN List Command allows an admin to dump the List of known ServicesAdmins/ServicesRoots
 *  
 * Syntax is: /msg O ADMIN LIST
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_admin_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL adminlist;
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&adminlist,sizeof(adminlist));

	/*
	 * Admin List Command
	 */
	 //:shock PRIVMSG O :admin add shock SRA
	
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"admin list")){
		adminlist.res    = doSelectMySQL(thread,"SELECT `authnick`,`rights`,`qlevel`,`olevel`,`slevel` FROM `orion_ns_core` WHERE `rights`>='3' ORDER BY `rights` ASC");
		adminlist.fields = mysql_num_rows(adminlist.res);
		
		if(adminlist.fields){
			for(i=0;i<adminlist.fields;i++){
				adminlist.row = mysql_fetch_row(adminlist.res);
		
				if(adminlist.row[0][0]!='\0'){
					if(stoi(adminlist.row[1])==is_services_admin)	msend(thread,howtosend,snick,nick,"SA: %s Q:%d O:%d S:%d",adminlist.row[0],stoi(adminlist.row[2]),stoi(adminlist.row[3]),stoi(adminlist.row[4]));
					else											msend(thread,howtosend,snick,nick,"SRA: %s Q:%d O:%d S:%d",adminlist.row[0],stoi(adminlist.row[2]),stoi(adminlist.row[3]),stoi(adminlist.row[4]));
				}
			}
			send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");
		}else send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");

		/* MySQL Free Result */
		mysql_free_result_myr(adminlist.res);
		
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the O ADMIN Command (/msg O HELP ADMIN)
 * 
 * When the user tries to get help for this command, using /msg O HELP ADMIN,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP ADMIN
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_admin_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Admin Help
	 */ 
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help admin")){
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"admin"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_ADMIN",ns->nick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"O_NO_HELP_AVAIL");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief O Help List Command - ADMIN part (/msg O help)
 * 
 * When the user tries to get general O help, using /msg O help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the ADMIN command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_admin_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_O_Level=(thread->sql.aliases.active>0 ? thread->sql.access.olevel : is_user);
	/*
	 * Admin (Help listing)
	 */
	if(Users_O_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"O_SHOWCOMMANDS_ADMIN",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"admin"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
