/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file o_gline.c
 *   @brief This is the Module file for O GLINE Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the O GLINE command.
 *
 *   The O GLINE Command allows an admin to manage the Networkbans.<br/>
 *   (only normal Glines which are normally processes by the IRCd)
 *
 *   Syntax is: /msg O GLINE { ADD | DEL | MASK | BLOCK | LIST } ...
 *
 *<table border="0">
 *    <tr><td> ADD		</td><td> Syntax is: /msg O GLINE ADD	</td><td> < host >       < duration(i.e. 2d) > < reason >	</td></tr> 
 *    <tr><td> DEL		</td><td> Syntax is: /msg O GLINE DEL	</td><td> *host-pattern*									</td></tr> 
 *    <tr><td> MASK		</td><td> Syntax is: /msg O GLINE MASK	</td><td> *host-pattern* < duration(i.e. 2d) >				</td></tr> 
 *    <tr><td> BLOCK	</td><td> Syntax is: /msg O GLINE BLOCK	</td><td> *nick*         < duration(i.e. 2d) > [reason]		</td></tr> 
 *    <tr><td> LIST		</td><td> Syntax is: /msg O GLINE LIST	</td><td> 													</td></tr> 
 *</table>
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"
#include "webform.h"

#define SERVICE "O"
#define MODUL "gline"
#define MODUL_FILE "o_gline"
#define MODUL_VERSION "1.2"

MOD_BOOL O_gline_add(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_gline_del(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_gline_test(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_gline_block(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_gline_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL O_gline(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_gline_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_gline_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"gline option[1]","gline");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,5,WEBACCESS,LOGINONLY,OPER_IF_DEFCON,OPERONLY,READONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);
	
	addLeastModuleLevels(thread,&Module,"gline",700,INFINIT); /* just to add a displaylevel for /msg o help
															  this should always be the lowest used Module level! */

	addCommand(thread,&Module,"gline add",O_gline_add,700,INFINIT,6,0);
	addCommand(thread,&Module,"gline del",O_gline_del,700,INFINIT,4,7);
	addCommand(thread,&Module,"gline test",O_gline_test,900,INFINIT,5,5);
	addCommand(thread,&Module,"gline block",O_gline_block,700,INFINIT,6,0);	
	addCommand(thread,&Module,"gline list",O_gline_list,700,INFINIT,4,4);

	addCommand(thread,&Module,"help gline",O_gline_help,700,INFINIT,4,5);
	addCommand(thread,&Module,"help",O_gline_help_listing,700,INFINIT,3,3); //100 - each Admin is able to see it.

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief O GLINE Add function for the O GLINE Command.
 * 
 * The O GLINE Add Command allows an admin to<br/>
 *   add a Gline for target host / ip.
 *  
 * Syntax is: /msg O GLINE ADD < host > < duration(i.e. 2d) > < reason >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL O_gline_add(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	char ident[MAX_IDENT_LEN];
	char host[MAX_HOST_LEN];
	int ret = 0;
		
	/* Initialise Variables to (NULL) */
	bzero(&ident,sizeof(ident));
	bzero(&host,sizeof(host));
		
	/*
	 * Gline Add Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"gline add") && strcor(param[5],"@") && param[7]){
		if(check_expire(param[6])){
			if(dynlen(param[5])>MAX_IDENT_LEN+MAX_HOST_LEN+1){
				send_lang(thread,language,howtosend,snick,nick,"MASK_TOO_LONG",param[5],MAX_IDENT_LEN+MAX_HOST_LEN+1);
				return MOD_CONT;
			}
			
			if(is_valid_chars(param[5],"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ@-_.*")){
				send_lang(thread,language,howtosend,snick,nick,"NO_HOST_INPUT",param[5]);
				return MOD_CONT;
			}
			
			ret = sscanf(param[5],"%[^@]@%s",ident,host);
			if(ret!=2){
				send_lang(thread,language,howtosend,snick,nick,"NO_HOST_INPUT",param[5]);
				return MOD_CONT;
			}
			
			do_gline(thread,m->servername,param[5],_dynadd("%s!%s@%s",O->nick,O->ident,O->host),expire(param[6])+time(NULL),time(NULL),_dstrjoin(' ',param,7));
			send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
			
			setWebSuccess(thread); /* Indentify webform that changes were applied */
		}else
			send_lang(thread,language,howtosend,snick,nick,"INVALID_EXPIRESTRING",param[6]);
	}else{
	/*
	 *  Error Messages
	 */
		if(!param[7])					send_lang(thread,language,howtosend,snick,nick,"NO_REASON");
		else if(!strcor(param[5],"@"))	send_lang(thread,language,howtosend,snick,nick,"NO_HOST_INPUT",param[5]);
		else 							send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O GLINE Del function for the O GLINE Command.
 * 
 * The O GLINE Del Command allows an admin to delete<br/>
 *   a gline for target host / ip or target *hostmask* / *ipmask*
 *  
 * Syntax is: /msg O GLINE DEL *ident-pattern*@*host-pattern*
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL O_gline_del(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL gline;
	char rident[MAX_IDENT_LEN];
	char rhost[MAX_HOST_LEN];
	int i 	= 0;
	int ret = 0;

	/* Initialise Variables to (NULL) */
	bzero(&gline,sizeof(gline));
	bzero(&rident,sizeof(rident));
	bzero(&rhost,sizeof(rhost));

	/*
	 * Gline Del Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"gline del") && strcor(param[5],"@")){
		if(dynlen(param[5])>MAX_IDENT_LEN+MAX_HOST_LEN+1){
			send_lang(thread,language,howtosend,snick,nick,"MASK_TOO_LONG",param[5],MAX_IDENT_LEN+MAX_HOST_LEN+1);
			return MOD_CONT;
		}
			
		ret = sscanf(param[5],"%[^@]@%s",rident,rhost);
		if(ret!=2){
			send_lang(thread,language,howtosend,snick,nick,"NO_HOST_INPUT",param[5]);
			return MOD_CONT;
		}
		
		strrepc(rident,'*','%');
		strrepc(rhost,'*','%');

		gline.res    = doSelectMySQL(thread,"SELECT `gline` FROM `orion_core_glines` WHERE `gline` LIKE '%s@%s' AND `line`='G' AND `rng`='0'",mysql_quote(thread,rident),mysql_quote(thread,rhost));
		gline.fields = mysql_num_rows(gline.res);

		for(i=0;i<gline.fields;i++){
			gline.row = mysql_fetch_row(gline.res);
			
			do_ungline(thread,m->servername,gline.row[0],_dynadd("%s!%s@%s",O->nick,O->ident,O->host),time(NULL));
		}
		send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");

		/* MySQL Free Result */
		mysql_free_result_myr(gline.res);
		
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}else{
	/*
	 *  Error Messages
	 */
		if(!strcor(param[5],"@"))	send_lang(thread,language,howtosend,snick,nick,"NO_HOST_INPUT",param[5]);
		else 						send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O GLINE Test function for the O GLINE Command.
 * 
 * The O GLINE Test Command allows an admin to test a<br/>
 *   glines
 *  
 * Syntax is: /msg O GLINE TEST *ident-pattern*@*host-pattern* < duration(i.e. 2d) >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL O_gline_test(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL hosts;
	char reason[MAX_REASON_LEN];
	char rident[MAX_IDENT_LEN];
	char rhost[MAX_HOST_LEN];
	int i 	= 0;
	int ret = 0;

	/* Initialise Variables to (NULL) */
	bzero(&hosts,sizeof(hosts));
	bzero(&reason,sizeof(reason));
	bzero(&rident,sizeof(rident));
	bzero(&rhost,sizeof(rhost));
	
	/*
	 * Gline Test Command
	 */
	 //:shock PRIVMSG operserv :gline mask *.PPPoE-2799.sa.bih.net.ba*
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"gline test") && strcor(param[5],"@")){
		if(dynlen(param[5])>MAX_IDENT_LEN+MAX_HOST_LEN+1){
			send_lang(thread,language,howtosend,snick,nick,"MASK_TOO_LONG",param[5],MAX_IDENT_LEN+MAX_HOST_LEN+1);
			return MOD_CONT;
		}
		
		if(is_valid_chars(param[5],"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ@-_.*")){
			send_lang(thread,language,howtosend,snick,nick,"NO_HOST_INPUT",param[5]);
			return MOD_CONT;
		}
		
		ret = sscanf(param[5],"%[^@]@%s",rident,rhost);
		if(ret!=2){
			send_lang(thread,language,howtosend,snick,nick,"NO_HOST_INPUT",param[5]);
			return MOD_CONT;
		}
		
		strrepc(rident,'*','%');
		strrepc(rhost,'*','%');

		hosts.res    = doSelectMySQL(thread,"SELECT `nick`,`rident`,`rhost` FROM `orion_core_condata` WHERE `rhost`='%s' AND `rident`='%s' LIMIT 50",rhost,rident);
		hosts.fields = mysql_num_rows(hosts.res);
		
		for(i=0;i<hosts.fields;i++){
			hosts.row = mysql_fetch_row(hosts.res);
			
			if(i==0) 	send_lang(thread,language,howtosend,snick,nick,"O_GLINE_TEST_HEADER",param[5]);
			else 		send_lang(thread,language,howtosend,snick,nick,"O_GLINE_TEST_LIST_MATCHES",i+1,hosts.row[0],hosts.row[0],hosts.row[1],hosts.row[2]);
		}
		send_lang(thread,language,howtosend,snick,nick,"O_GLINE_TEST_FOOTER",hosts.fields<50 ? hosts.fields : 50,hosts.fields<50 ? hosts.fields : hosts.fields);

		mysql_free_result_myr(hosts.res);
		
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}else{
	/*
	 *  Error Messages
	 */
		if(strcor(param[5],"@"))	send_lang(thread,language,howtosend,snick,nick,"NO_HOST_INPUT",param[5]);
		else						send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O GLINE Block function for the O GLINE Command.
 * 
 * The O GLINE Block Command allows an admin to add a<br/>
 *   gline for target nick (The nicks realhost will be glined)
 *  
 * Syntax is: /msg O GLINE BLOCK *nick* < duration(i.e. 2d) > [reason]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL O_gline_block(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL rhost;
	char reason  [MAX_REASON_LEN];
	char pattern [BUFSIZE];
	time_t expiretime=0;
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&rhost,sizeof(rhost));
	bzero(&reason,sizeof(reason));
	bzero(&pattern,sizeof(pattern));

	/*
	 * Gline Block Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"gline block") && findchar(param[5],'*')==2 && dynlen(param[5])>=3){
		dynncpy(pattern,param[5],sizeof(pattern));
		
		if(param[5][0]=='*')
			strrepc(pattern,'*','%');

		expiretime   = expire(param[6])+time(NULL);
		rhost.res    = doSelectMySQL(thread,"SELECT `rhost` FROM `orion_core_condata` WHERE `nick` LIKE '%s'",pattern);
		rhost.fields = mysql_num_rows(rhost.res);

		if(!param[7])
			snprintf(reason,sizeof(reason),"blocking your connection for %s",param[6]);
		else
			dynncpy(reason,_dstrjoin(' ',param,7),sizeof(reason));
		

		for(i=0;i<rhost.fields;i++){
			rhost.row = mysql_fetch_row(rhost.res);
			
			do_gline(thread,m->servername,rhost.row[0],_dynadd("%s!%s@%s",O->nick,O->ident,O->host),expiretime,time(NULL),reason);
		}

		send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
		
		/* mysql_free_result_myr(); - rhost */
		mysql_free_result_myr(rhost.res);
		
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}else{
	/*
	 *  Error Messages
	 */
		if(!strcor(param[5],"*"))	send_lang(thread,language,howtosend,snick,nick,"WRONG_INPUT_PATTERN_INPUT",param[5]);
		else 						send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O GLINE List function for the O GLINE Command.
 * 
 * The O GLINE List Command allows an admin to<br/>
 *   dump all glines set by orion.
 *  
 * Syntax is: /msg O GLINE LIST
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL O_gline_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL gline;
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&gline,sizeof(gline));

	/*
	 * Gline List Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"gline list")){
		gline.res    = doSelectMySQL(thread,"SELECT `rng`,`line`,`gline`,`duration`,`addedby`,`reason` FROM `orion_core_glines` WHERE `rng`='0'");
		gline.fields = mysql_num_rows(gline.res);
		
		for(i=0;i<gline.fields;i++){
			gline.row = mysql_fetch_row(gline.res);
			
			if(stol(gline.row[3])!=0)
				send_lang(thread,language,howtosend,snick,nick,"O_GLINE_LIST_LIST_SET_GLINES_WHICH_EXPIRE_AT_A_GIVEN_TIME",gline.row[1],gline.row[2],get_localtime((time_t)stol(gline.row[3])),gline.row[4],gline.row[5]);
			else
				send_lang(thread,language,howtosend,snick,nick,"O_GLINE_LIST_LIST_PERMANENT_GLINES",gline.row[1],gline.row[2],gline.row[4],gline.row[5]);
		}
		send_lang(thread,language,howtosend,snick,nick,"END_OF_STATS_REPORT");

		mysql_free_result_myr(gline.res); //free MySQL result
		
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the O GLINE Command (/msg O HELP GLINE)
 * 
 * When the user tries to get help for this command, using /msg O HELP GLINE,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP GLINE
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_gline_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Gline Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help gline")) {
		if(streqnc(param[5],"add")){
			send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"gline add"));
			send_lang(thread,language,howtosend,snick,nick,"O_HELP_GLINE_ADD");
			setWebSuccess(thread); /* Indentify webform that changes were applied */
		}else if(streqnc(param[5],"del")){
			send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"gline del"));
			send_lang(thread,language,howtosend,snick,nick,"O_HELP_GLINE_DEL");
			setWebSuccess(thread); /* Indentify webform that changes were applied */
		}else if(streqnc(param[5],"test")){
			send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"gline test"));
			send_lang(thread,language,howtosend,snick,nick,"O_HELP_GLINE_TEST");
			setWebSuccess(thread); /* Indentify webform that changes were applied */
		}else if(streqnc(param[5],"block")){
			send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"gline block"));
			send_lang(thread,language,howtosend,snick,nick,"O_HELP_GLINE_BLOCK");
			setWebSuccess(thread); /* Indentify webform that changes were applied */
		}else if(streqnc(param[5],"list")){
			send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"gline list"));
			send_lang(thread,language,howtosend,snick,nick,"O_HELP_GLINE_LIST");
			setWebSuccess(thread); /* Indentify webform that changes were applied */
		}
		else if(!param[5]){
			send_lang(thread,language,howtosend,snick,nick,"O_HELP_GLINE",snick);
			setWebSuccess(thread); /* Indentify webform that changes were applied */
		}
			
		else 
			send_lang(thread,language,howtosend,snick,nick,"O_NO_HELP_AVAIL");

	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief O Help List Command - GLINE part (/msg O help)
 * 
 * When the user tries to get general O help, using /msg O help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the GLINE command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_gline_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_O_Level=(thread->sql.aliases.active>0 ? thread->sql.access.olevel : is_user);
	/*
	 * Gline (Help listing)
	 */
	if(Users_O_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"O_SHOWCOMMANDS_GLINE",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"gline"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
