/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file q_chanlev.c
 *   @brief This is the Module file for Q CHANLEV Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the Q CHANLEV command.
 *
 *   The Q CHANLEV Command allows the user to change/view<br/>
 *   the channels access list. (Master or Owner rights required)
 *
 *   Available ChanlevFlags are:<br/>
 *    +a = autoop/autovoice on join.<br/>
 *    +b = banned. (Only works if they've AUTH'd.)<br/>
 *    +h = halfop.<br/>
 *    +m = master. (Can make opers.)<br/>
 *    +n = owner. (Can make masters.)<br/>
 *    +o = op.<br/>
 *    +p = protect. (usermode +a.)<br/>
 *    +t = can set topics.<br/>
 *    +v = voice.
 *
 *   Syntax is: /msg Q CHANLEV #channel < nick > [ [+|-]Modes ]
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"
#include "webform.h"

#define SERVICE "Q"
#define MODUL "chanlev"
#define MODUL_FILE "q_chanlev"
#define MODUL_VERSION "1.2"

char Q_Avail_Usermodes[] = "abhmnoptv";

MOD_BOOL Q_chanlev(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL Q_chanlev_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL Q_chanlev_showcommands(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"chanlev","chanlev");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,WEBACCESS,LOGINONLY,OPER_IF_DEFCON);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"chanlev",Q_chanlev,550,1,4,6);
	addCommand(thread,&Module,"help chanlev",Q_chanlev_help,1,1,4,4);
	addCommand(thread,&Module,"showcommands",Q_chanlev_showcommands,1,1,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the Q CHANLEV Command.
 * 
 * The Q CHANLEV Command allows the user to change/view<br/>
 *   the channels access list. (Master or Owner rights required)
 *
 * Syntax is: /msg Q CHANLEV #channel < nick > [ [+|-]Modes ]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_chanlev(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global variables */
	deepSQL accesslist;

	char delmodes[MAX_MODES]; //delmodes - SIZE max. modes per channel (Q modes not IRCD modes)
	char addmodes[MAX_MODES]; //addmodes - SIZE max. modes per channel (Q modes not IRCD modes)
	char modes   [MAX_MODES*2];
	char type, chr;
	int total=0,owners=0,masters=0,ops=0,voices=0,halfops=0,bans=0;
	size_t i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&accesslist,sizeof(accesslist));
	bzero(&delmodes,sizeof(delmodes));
	bzero(&addmodes,sizeof(addmodes));
	bzero(&modes,sizeof(modes));
	type='\0';
	chr='\0';

	/*
	 * Chanlev
	 */
	if(thread->sql.access.qlevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"chanlev")){
        if(param[5])
			/* Updates the Threads Channel-Management */
			UpdateChannelInformation(thread,param[4],param[5]);
		else
			/* Updates the Threads Channel-Management */
			UpdateChannelInformation(thread,param[4],NULL);

		if(thread->c.chan.fields && streqnc(thread->c.chan.serv,"q")){
			if((thread->c.chan.settings & CS_FORBID_BIT))		send_lang(thread,language,howtosend,snick,nick,"CHANNEL_FORBIDDEN",thread->c.chan.serv,thread->c.chan.reason);
			else if((thread->c.chan.settings & CS_SUSPEND_BIT))	send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
			else{

				/*
				 * Chanlev list accounts
				 */
				if(!param[6] && param[5]){
					if(thread->c.aliases.authnick[0]!='\0')
						send_lang(thread,language,howtosend,snick,nick,"Q_CHANLEV_MODES_ON_CHANNEL",thread->sql.aliases.nick,param[4],thread->c.chan.access.ownlevel);
					else
						send_lang(thread,language,howtosend,snick,nick,"Q_CHANLEV_USER_NOT_KNOWN_ON_CHANNEL");
				}
				else if(!param[5]){
					if(thread->sql.access.qlevel>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"chanlev") || strcor(thread->c.chan.access.ownlevel,"mnov")){
						if(thread->c.chan.flags[0]!='\0'){
							send_lang(thread,language,howtosend,snick,nick,"Q_CHANFLAGS_CURRENT_CHANFLAGS",param[4],thread->c.chan.flags);
							if(strcor(thread->c.chan.flags,"c")) send_lang(thread,language,howtosend,snick,nick,"Q_CHANFLAGS_AUTOLIMIT",thread->c.chan.alimit);
							if(strcor(thread->c.chan.flags,"l")) send_lang(thread,language,howtosend,snick,nick,"Q_CHANFLAGS_LIMIT",thread->c.chan.limit);
						}
						else send_lang(thread,language,howtosend,snick,nick,"Q_NO_CHANFLAGS_SET");

						accesslist.res    = doSelectMySQL(thread,"SELECT `channel`,`authnick`,`level` FROM `orion_cs_access` WHERE `channel`='%s' AND `serv`='Q' ORDER BY `authnick` ASC",mysql_quote(thread,param[4]));
						accesslist.fields = mysql_num_rows(accesslist.res);

						for(i=0;i<(size_t)accesslist.fields;i++){
							accesslist.row = mysql_fetch_row(accesslist.res);
							
							if(strcor(accesslist.row[2],"n"))			owners++;
							else if(strcor(accesslist.row[2],"m"))		masters++;
							else if(strcor(accesslist.row[2],"o"))		ops++;
							else if(strcor(accesslist.row[2],"h"))		halfops++;
							else if(strcor(accesslist.row[2],"v"))		voices++;
							else if(strcor(accesslist.row[2],"b"))		bans++;
							send_lang(thread,language,howtosend,snick,nick,"Q_CHANNEL_USER_MODES",accesslist.row[1],accesslist.row[2]);
						}

						//total = owners+masters+ops+halfops+voices+bans;
						bans += stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_cs_akick` WHERE `channel`='%s'",mysql_quote(thread,param[4])));
						total = owners+masters+ops+halfops+voices+bans;
						
						if(accesslist.fields){
							send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");
							send_lang(thread,language,howtosend,snick,nick,"Q_CHANLEV_TOTAL_USERS",total,owners,masters,ops,halfops,voices,bans);
							
							setWebSuccess(thread); /* Indentify webform that changes were applied */
						}else send_lang(thread,language,howtosend,snick,nick,"Q_CHANLEV_NO_KNOWN_USERS",param[4]);

						mysql_free_result_myr(accesslist.res);
					}else send_lang(thread,language,howtosend,snick,nick,"Q_CHANLEV_CANT_DUMP_USERLIST",param[4]);
				}
				
				/*
				 * Chanlev modify levels
				 */
				else if(param[5] && param[6] && CheckOption(thread,snick,nick,READONLY)){
					//> shock PRIVMSG Q :chanlev #channel #KnT|PhAlAX +amn-otv
					addmodes[0]='\0'; //set AddModes to NULL
					delmodes[0]='\0'; //set DelModes to NULL
					
					if(thread->sql.access.qlevel>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"chanlev") || strcor(thread->c.chan.access.ownlevel,"n")){
						if(thread->c.aliases.fields && thread->c.aliases.authnick[0]!='\0'){
							type='\0';
							for(i=0;i<dynlen(param[6]);i++){
								chr=param[6][i];
								if(chr=='+' || chr=='-'){type=param[6][i];}
								else if(strcin(chr,Q_Avail_Usermodes)){
									if(type=='+' && !strcin(chr,addmodes) && !strcin(chr,thread->c.chan.access.targetlevel)){
										straddnc(addmodes,chr,sizeof(addmodes));
										//a++;
									}else if(type=='-' && !strcin(chr,delmodes)){
										straddnc(delmodes,chr,sizeof(delmodes));
										//b++;
									}
								}
							}
						}else{
							send_lang(thread,language,howtosend,snick,nick,"NICK_NOT_REGISTERED",param[5]);
							return MOD_CONT;
						}
					}else if(strcor(thread->c.chan.access.ownlevel,"m")){
						if(strcor(param[6],"nm")) send_lang(thread,language,howtosend,snick,nick,"Q_RIGHT_OWNER_TO_CHANGE_FLAGS");
						else{
							if(thread->c.aliases.fields && thread->c.aliases.authnick[0]!='\0'){
								type='\0';
								for(i=0;i<dynlen(param[6]);i++){
									chr=param[6][i];
									if(chr=='+' || chr=='-'){type=param[6][i];}
									else if(strcin(chr,"abhoptv")){
										if(type=='+' && !strcin(chr,addmodes) && !strcin(chr,thread->c.chan.access.targetlevel)){
											straddnc(addmodes,chr,sizeof(addmodes));
											//a++;
										}else if(type=='-' && !strcin(chr,delmodes)){
											straddnc(delmodes,chr,sizeof(delmodes));
											//b++;
										}
									}
								}
							}else{
								send_lang(thread,language,howtosend,snick,nick,"NICK_NOT_REGISTERED",param[5]);
								return MOD_CONT;
							}
						}
					}

					/****************************************************/
					/**            	 Merge new ChannelLevel            **/
					bzero(modes,sizeof(modes));
					if(thread->c.chan.access.targetlevel[0]!='\0')
                        straddns(modes,thread->c.chan.access.targetlevel,sizeof(modes));
                        
					straddns(modes,addmodes,sizeof(modes));

					for(i=0;i<dynlen(delmodes);i++)
						strdelc(modes,delmodes[i]);
					
					/****************************************************/
					
					if(modes[0]=='\0'){
                        /* Update Memory, for caching */
#ifndef USE_THREADS
						bzero(&thread->c.chan.access.targetlevel,sizeof(thread->c.chan.access.targetlevel));
#endif

						/* Update SQL as Live Update */
						domysql(thread,"DELETE FROM `orion_cs_access` WHERE `authnick`='%s' AND `channel`='%s' AND `serv`='Q'",mysql_quote(thread,thread->c.aliases.authnick),mysql_quote(thread,param[4]));
					}
					else{
						sort(modes);
						
						/* Update Memory, for caching */
#ifndef USE_THREADS
						dynncpy(thread->c.chan.access.targetlevel,modes,sizeof(thread->c.chan.access.targetlevel));
#endif

						/* Update SQL as Live Update */
						if(thread->c.chan.access.targetauthnick[0]=='\0')
							domysql(thread,"INSERT INTO `orion_cs_access` (`serv`,`authnick`,`channel`,`level`) VALUES ('Q','%s','%s','%s')",mysql_quote(thread,thread->c.aliases.authnick),mysql_quote(thread,param[4]),mysql_quote(thread,modes));
						else{
							domysql(thread,"UPDATE `orion_cs_access` LEFT OUTER JOIN `orion_cs_core` ON (`orion_cs_access`.`channel`=`orion_cs_core`.`channel`) SET `orion_cs_core`.`last_used`='%ld', `orion_cs_access`.`level`='%s' WHERE `orion_cs_access`.`serv`='Q' AND `orion_cs_access`.`authnick`='%s' AND `orion_cs_access`.`channel`='%s'",time(NULL),mysql_quote(thread,modes),mysql_quote(thread,thread->c.aliases.authnick),mysql_quote(thread,param[4]));
							if(strcor(modes,"b")){
								do_mode(thread,snick,param[4],thread->c.chan.datecreate,"+b %s",thread->c.Cdata.vhost);
								do_kick(thread,snick,param[4],thread->c.Cdata.nick,"Q-banned");
							}
						}
					}
					send_lang(thread,language,howtosend,snick,nick,"Q_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else send_lang(thread,language,howtosend,snick,nick,"MISSING_PARAMETER");
			}
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_EXISTING");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the Q CHANLEV Command (/msg Q HELP CHANLEV)
 * 
 * When the user tries to get help for this command, using /msg Q HELP CHANLEV,<br/>
 * this function is executed.
 * 
 * Syntax: /msg Q HELP CHANLEV
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_chanlev_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Chanlev Help
	 */
	if(thread->sql.access.qlevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help chanlev")){
		send_lang(thread,language,howtosend,snick,nick,"Q_HELP_CHANLEV",snick,snick,snick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}

	else send_lang(thread,language,howtosend,snick,nick,"Q_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Q SHOWCOMMANDS Command - CHANLEV part (/msg Q showcommands)
 * 
 * When the user tries to get general Q help, using /msg Q showcommands, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to<br/>
 * include the information for the CHANLEV command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_chanlev_showcommands(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=0, Users_Q_Level=0;
	command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"showcommands");
	Users_Q_Level=(thread->sql.aliases.active>0 ? thread->sql.access.qlevel : is_user);
	/*
	 * Chanlev (Showcommands)
	 */
	if(Users_Q_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"Q_SHOWCOMMANDS_CHANLEV",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"chanlev"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
