/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file q_set.c
 *   @brief This is the Module file for Q SET Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the Q SET command.
 *
 *   The Q SET Command allows the user to set various SET-Options<br/>
 *   for target Channel. (Controls FOUNDER/CHANTYPE/URL etc.)
 *
 *   Syntax is: /msg Q SET #channel OPTION ...
 *
 *<table border="0">
 *    <tr><td> FOUNDER	</td><td> Syntax is: /msg Q SET #channel FOUNDER	</td><td> < nick >		</td></tr>
 *    <tr><td> CHANTYPE	</td><td> Syntax is: /msg Q SET #channel CHANTYPE	</td><td> < chantype >	</td></tr>
 *    <tr><td> URL		</td><td> Syntax is: /msg Q SET #channel URL		</td><td> < new-Url >	</td></tr>
 *</table>
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"
#include "webform.h"

/* Define Q Ownermode */
#define OWNER_MODE "ahmnoptv"

#define SERVICE "Q"
#define MODUL "set"
#define MODUL_FILE "q_set"
#define MODUL_VERSION "1.2"

MOD_BOOL Q_set_founder(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL Q_set_chantype(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL Q_set_url(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
//--------------------------------------------------------------------------------------

MOD_BOOL Q_set_founder_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL Q_set_chantype_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL Q_set_url_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL Q_set_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL Q_set_showcommands(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"set * option[1]","set option[1]");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,4,WEBACCESS,LOGINONLY,OPER_IF_DEFCON,READONLY);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addLeastModuleLevels(thread,&Module,"set",700,1); /* just to add a displaylevel for /msg Q showcommands
															  this should always be the lowest used Module level! */
															  
	addCommand(thread,&Module,"set founder",Q_set_founder,700,1,6,6);
	addCommand(thread,&Module,"set chantype",Q_set_chantype,700,1,6,0);
	addCommand(thread,&Module,"set url",Q_set_url,700,1,6,6);
	
	addCommand(thread,&Module,"help set founder",Q_set_founder_help,100,1,5,5);
	addCommand(thread,&Module,"help set chantype",Q_set_chantype_help,100,1,5,5);
	addCommand(thread,&Module,"help set url",Q_set_url_help,100,1,5,5);
	
	addCommand(thread,&Module,"help set",Q_set_help,100,1,4,4);
	addCommand(thread,&Module,"showcommands",Q_set_showcommands,100,1,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */
//:shock PRIVMSG Q :set #channel FOUNDER la
/**
 * @brief Q SET Founder function for the ChanServ SET Command.
 * 
 * The Q SET Founder Command allows the user to change the FounderNick<br/>
 *   to a new FounderNick
 *  
 * Syntax is: /msg Q SET #channel FOUNDER < nick >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_set_founder(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//Channel c;
	//Aliases aliases5;

	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));
	//bzero(&aliases5,sizeof(aliases5));

	/*
	 * Set Founder Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set founder")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],param[6]);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_FORBID_BIT))		 send_lang(thread,language,howtosend,snick,nick,"CHANNEL_FORBIDDEN",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"q")){
			if(streqnc(thread->sql.aliases.authnick,thread->c.chan.founder) || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set founder")){
				//aliases5 = sql_aliases(thread,param[6]);
				
				if(thread->c.aliases.authnick[0]!='\0'){
    				if(!thread->c.aliases.forbid){
						/* Update Memory, for caching */
#ifndef	USE_THREADS			
						dynncpy(thread->c.chan.founder,thread->c.aliases.authnick,sizeof(thread->c.chan.founder));
						dynncpy(thread->c.chan.access.targetlevel,OWNER_MODE,sizeof(thread->c.chan.access.targetlevel));
#endif
						
						/* Update SQL as Live Update */
    					domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`founder`='%s' WHERE `channel`='%s' AND `serv`='Q'",time(NULL),mysql_quote(thread,thread->c.aliases.authnick),mysql_quote(thread,thread->c.chan.name));
    					
    					if(stoi(thread->c.chan.access.targetlevel)!=0)
    						domysql(thread,"UPDATE `orion_cs_access` SET `level`='%s' WHERE `authnick`='%s' AND `channel`='%s' AND `serv`='Q'",mysql_quote(thread,OWNER_MODE),mysql_quote(thread,thread->c.aliases.authnick),mysql_quote(thread,thread->c.chan.name));
    					else
    						domysql(thread,"INSERT INTO `orion_cs_access` (`serv`,`authnick`,`channel`,`level`) VALUES ('Q','%s','%s','%s')",mysql_quote(thread,thread->c.aliases.authnick),mysql_quote(thread,thread->c.chan.name),mysql_quote(thread,OWNER_MODE));
    						
    					send_lang(thread,language,howtosend,snick,nick,"Q_SET_FOUNDER_X_FOR_CHANNEL_X",thread->c.chan.name,thread->c.aliases.authnick);
    					
    					setWebSuccess(thread); /* Indentify webform that changes were applied */
    				}else send_lang(thread,language,howtosend,snick,nick,"NICK_SUSPENDED_OR_FORBIDDEN",thread->c.aliases.nick);
                }else send_lang(thread,language,howtosend,snick,nick,"NO_AUTHNICK");
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_Q");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Q SET Chantype function for the Q SET Command.
 * 
 * The Q SET Chantype Command allows the user to change<br/>
 *   the Channel's Chantype
 *  
 * Syntax is: /msg Q SET #channel CHANTYPE < new-chantype >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_set_chantype(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;

	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));

	/*
	 * Set Chantype Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set chantype")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_FORBID_BIT))		 send_lang(thread,language,howtosend,snick,nick,"CHANNEL_FORBIDDEN",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"q")){
			if(streqnc(thread->sql.aliases.authnick,thread->c.chan.founder) || strcor(thread->c.chan.access.ownlevel,"mn") || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set desc")){
				/* Update Memory, for caching */
#ifdef USE_THREADS
				my_pthread_mutex_lock(&threads_lock);
#endif
				//bzero(&thread->c.chan.description,sizeof(thread->c.chan.description));
				dynncpy(thread->c.chan.description,_dstrjoin(' ',param,6),sizeof(thread->c.chan.description));
#ifdef USE_THREADS
				my_pthread_mutex_unlock(&threads_lock);
#endif
				
				/* Update SQL as Live Update */
				domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`description`='%s' WHERE `channel`='%s' AND `serv`='Q'",time(NULL),mysql_quote(thread,thread->c.chan.description),mysql_quote(thread,thread->c.chan.name));
				send_lang(thread,language,howtosend,snick,nick,"Q_COMMAND_DONE");
				
				setWebSuccess(thread); /* Indentify webform that changes were applied */
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_Q");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Q SET Url function for the Q SET Command.
 * 
 * The Q SET Url Command allows the user to change<br/>
 *   the Channel's Url
 *  
 * Syntax is: /msg Q SET #channel URL < new-Url >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_set_url(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;

	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));

	/*
	 * Set Url Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set url")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_FORBID_BIT))		 send_lang(thread,language,howtosend,snick,nick,"CHANNEL_FORBIDDEN",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"q")){
			if(streqnc(thread->sql.aliases.authnick,thread->c.chan.founder) || strcor(thread->c.chan.access.ownlevel,"mn") || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set url")){
				if(!param[6]){
					/* Update Memory, for caching */
#ifndef USE_THREADS
					bzero(&thread->c.chan.url,sizeof(thread->c.chan.url));
#endif

					/* Update SQL as Live Update */
					domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`url`='' WHERE `channel`='%s' AND `serv`='Q'",time(NULL),mysql_quote(thread,thread->c.chan.name));
					send_lang(thread,language,howtosend,snick,nick,"Q_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else if(!is_valid_chars(param[6],"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ+-_.:/%")){
					/* Update Memory, for caching */
#ifndef USE_THREADS
					//bzero(&thread->c.chan.url,sizeof(thread->c.chan.url));
					dynncpy(thread->c.chan.url,param[6],sizeof(thread->c.chan.url));
#endif

					/* Update SQL as Live Update */
					domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`url`='%s' WHERE `channel`='%s' AND `serv`='Q'",time(NULL),mysql_quote(thread,param[6]),mysql_quote(thread,thread->c.chan.name));
					send_lang(thread,language,howtosend,snick,nick,"Q_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else send_lang(thread,language,howtosend,snick,nick,"INVALID_PARAMETER",param[6]);
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_Q");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Q SET Email function for the Q SET Command.
 * 
 * The Q SET Email Command allows the user to change<br/>
 *   the Channel's Email-Address.
 *  
 * Syntax is: /msg Q SET #channel EMAIL < new-Email >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_set_email(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */

	/* Initialise Variables to (NULL) */
	

	/*
	 * Set Email Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set email")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_FORBID_BIT))		 send_lang(thread,language,howtosend,snick,nick,"CHANNEL_FORBIDDEN",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"q")){
			if(streqnc(thread->sql.aliases.authnick,thread->c.chan.founder) || streqnc(thread->sql.aliases.authnick,thread->c.chan.successor) || stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.set || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set email")){
				if(param[6]){
					/*dynncpy(email,param[6],sizeof(email));

					if(!strcsea(email,"@&.")){
						send_lang(thread,language,howtosend,snick,nick,"Q_SET_EMAIL_INVALID_EMAIL");
						return MOD_CONT;
					}
		
					strsplit(email,domain,'@');
					dynncpy(emailname,domain[0],sizeof(emailname));
					strsplit(domain[1],parts,'.');
					domain[0] = emailname;

					dlog(thread,3,"domain[0] = %s\n",domain[0]);
					dlog(thread,3,"domain[1] = %s\n",domain[1]);
		
					if(domain[0]=='\0' || parts[0]=='\0' || parts[1]=='\0' || is_valid_chars(parts[0],"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-_.") || is_valid_chars(parts[1],"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-_.") || is_valid_chars(domain[0],"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-_.")){
						dlog(thread,3,"domain[0] (begin) = %s\n",domain[0]);
						dlog(thread,3,"parts[0] validation = %d\n",is_valid_chars(parts[0],"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-_."));
						dlog(thread,3,"parts[1] validation = %d\n",is_valid_chars(parts[1],"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-_."));
						dlog(thread,3,"domain[0] validation = %d\n",is_valid_chars(domain[0],"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-_."));

						dlog(thread,3,"parts[0] = %s\n",parts[0]);
						dlog(thread,3,"parts[1] = %s\n",parts[1]);
						dlog(thread,3,"domain[0] = %s\n",domain[0]);

						send_lang(thread,language,howtosend,snick,nick,"Q_SET_EMAIL_INVALID_EMAIL");
						return MOD_CONT;
					}
		
					else */
					if(MailValidate(param[6])/*dynlen(parts[0])>=ns->min_emaildomainlen && dynlen(parts[1])>=ns->min_emaildomainlen*/){ //email ok and accepted
						/* Update Memory, for caching */
#ifndef USE_THREADS
						//bzero(&thread->c.chan.email,sizeof(thread->c.chan.email));
						dynncpy(thread->c.chan.email,param[6],sizeof(thread->c.chan.email));
#endif

						/* Update SQL as Live Update */
						domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`email`='%s' WHERE `channel`='%s' AND `serv`='Q'",time(NULL),mysql_quote(thread,param[6]),mysql_quote(thread,thread->c.chan.name));
						send_lang(thread,language,howtosend,snick,nick,"Q_COMMAND_DONE");
						
						setWebSuccess(thread); /* Indentify webform that changes were applied */
					}
					else send_lang(thread,language,howtosend,snick,nick,"Q_SET_EMAIL_INVALID_EMAIL");
				}else{
					/* Update Memory, for caching */
#ifndef USE_THREADS
					bzero(&thread->c.chan.email,sizeof(thread->c.chan.email));
#endif

					/* Update SQL as Live Update */
					domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`email`='' WHERE `channel`='%s' AND `serv`='Q'",time(NULL),mysql_quote(thread,thread->c.chan.name));
					send_lang(thread,language,howtosend,snick,nick,"Q_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_Q");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the ChanServ SET Command (/msg Q HELP SET)
 * 
 * When the user tries to get help for this command, using /msg Q HELP SET,<br/>
 * this function is executed.
 * 
 * Syntax: /msg Q HELP SET
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_set_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set")){
		send_lang(thread,language,howtosend,snick,nick,"Q_HELP_SET",snick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Q SET Founder Help function for the ChanServ SET Command (/msg Q HELP SET FOUNDER)
 * 
 * When the user tries to get help for this command, using /msg Q HELP SET FOUNDER,<br/>
 * this function is executed.
 * 
 * Syntax: /msg Q HELP SET FOUNDER
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_set_founder_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Founder Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set founder")){
		send_lang(thread,language,howtosend,snick,nick,"Q_HELP_SET_FOUNDER");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Q SET Chantype Help function for the Q SET Command (/msg Q HELP SET CHANTYPE)
 * 
 * When the user tries to get help for this command, using /msg Q HELP SET CHANTYPE,<br/>
 * this function is executed.
 * 
 * Syntax: /msg Q HELP SET CHANTYPE
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_set_chantype_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Chantype Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set chantype")){
		send_lang(thread,language,howtosend,snick,nick,"Q_HELP_SET_CHANTYPE");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Q SET Url Help function for the Q SET Command (/msg Q HELP SET URL)
 * 
 * When the user tries to get help for this command, using /msg Q HELP SET URL,<br/>
 * this function is executed.
 * 
 * Syntax: /msg Q HELP SET URL
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_set_url_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Url Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set url")){
		send_lang(thread,language,howtosend,snick,nick,"Q_HELP_SET_URL");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Q SET Email Help function for the Q SET Command (/msg Q HELP SET EMAIL)
 * 
 * When the user tries to get help for this command, using /msg Q HELP SET EMAIL,<br/>
 * this function is executed.
 * 
 * Syntax: /msg Q HELP SET EMAIL
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_set_email_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Email Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set email")){
		send_lang(thread,language,howtosend,snick,nick,"Q_HELP_SET_EMAIL");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief Q Showcommands Set Command - SET part (/msg Q Showcommands)
 * 
 * When the user tries to get general Q Showcommands, using /msg Q Showcommands, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the SET command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_set_showcommands(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"showcommands");
	int Users_Q_Level=(thread->sql.aliases.active>0 ? thread->sql.access.qlevel : is_user);
	/*
	 * Set (Help listing)
	 */
	if(Users_Q_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"Q_SHOWCOMMANDS_SET");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
