/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file gline.c
 *   @brief This is the Module file for GLINE (Server Side) Command.
 *
 *   It contains all functions which are neccessary to process and use 
 *   the GLINE command.
 *
 *   The GLINE File is used to process GLINES sent from Servers
 *   and to Servers. (Protocol-Process-File -> PPF)
 *   (Used to set GLINES or remove them as sent from the services or received from the server as well.)
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "global.h"
#include "services.h"
#include "modules.h"
#include "osocket.h"
#include "webform.h"

/**
 * @brief do_gline:  Sends out a GLINE (depending on the Loaded Protocol)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param servername The ServerName (services-server) which sends the gline information
 * @param mask The Mask which we like to put a gline on
 * @param nick The Nick which added the gline
 * @param in_expiretime The Time when the gline should expire
 * @param in_hubtime The Time when the gline was set
 * @param in_reason The Reason why we gline that Host / IP
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_gline(Threads *thread, char *servername, char *mask, char *nick, time_t in_expiretime, time_t in_hubtime, char *in_reason){
	PROTOCOL_BOOL analyzer;

	/* Declare variables */
	char addedby[MAX_NICK_LEN];
	char ident_at_host[MAX_IDENT_LEN+1+MAX_HOST_LEN];
	time_t expire = 0;
	
	/* Initialize variables to NULL */
	bzero(&addedby,sizeof(addedby));
	bzero(&ident_at_host,sizeof(ident_at_host));
	
	/* Clean the delay in our Duration */
							expire = in_expiretime-in_hubtime;
	if(in_expiretime > 0) 	expire+=time(NULL);
	else 					expire = 0;

	/*
	 * Build response.
	 */

	/* Analyze Command - is it supported from the IRCd !? */
	analyzer = orion_WRCtranslate_cmd_gline(thread,servername,mask,nick,in_hubtime,expire,in_reason);
	if(analyzer >= CMD_NOT_SUPPORTED)
		return analyzer;

	/* Command supported .. we go on running it */
	sscanf(nick,"%[^!]!%s",addedby,ident_at_host);
	//addedby = NICK ohne !ident@host.net ....

	domysql(thread,"INSERT INTO `orion_core_lines` (`type`,`mask`,`addedby`,`duration`,`reason`) VALUES ('G','%s','%s','%ld','%s') ON DUPLICATE KEY UPDATE "
	"`addedby`=VALUES(`addedby`),"
	"`duration`=VALUES(`duration`),"
	"`reason`=VALUES(`reason`)"
	,mysql_quote(thread,mask),mysql_quote(thread,addedby),expire,mysql_quote(thread,in_reason));
	
	if(streqnc(addedby,O->nick) || streqnc(addedby,SC->nick) || streqnc(addedby,Q->nick))
		domysql(thread,"INSERT INTO `orion_core_glines` (`rng`,`line`,`gline`,`duration`,`addedby`,`reason`) VALUES ('0','G','%s','%ld','%s','%s') ON DUPLICATE KEY UPDATE "
		"`duration`=VALUES(`duration`),"
		"`addedby`=VALUES(`addedby`),"
		"`reason`=VALUES(`reason`)"
		,mysql_quote(thread,mask),expire,mysql_quote(thread,addedby),mysql_quote(thread,in_reason));
		
	return analyzer;
}

/*  ************************************************************************************************  */

/**
 * @brief do_ungline:  Sends out an UNGLINE (depending on the Loaded Protocol)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param servername The ServerName (services-server) which sends the ungline information
 * @param mask The Mask which we like to ungline
 * @param nick The Nick which added the gline
 * @param in_hubtime The Time when the gline was set
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_ungline(Threads *thread, char *servername, char *mask, char *nick, time_t in_hubtime){
	PROTOCOL_BOOL analyzer;
	
	/*
	 * Build response.
	 */
	 
	/* Analyze Command - is it supported from the IRCd !? */
	analyzer = orion_WRCtranslate_cmd_ungline(thread,servername,mask,nick,in_hubtime);
	if(analyzer >= CMD_NOT_SUPPORTED)
		return analyzer;
	
	/* Command supported .. we go on running it */
	domysql(thread,
		"DELETE `orion_core_glines`,`orion_core_lines` FROM `orion_core_glines` "
		"LEFT OUTER JOIN `orion_core_lines` ON (`orion_core_lines`.`mask`=`orion_core_glines`.`gline` AND `orion_core_lines`.`type`='G') "
		"WHERE `orion_core_glines`.`gline`='%s' AND `orion_core_glines`.`rng`='0'"
		,mysql_quote(thread,mask));
	return analyzer;
}

/*  ************************************************************************************************  */

/**
 * @brief on_gline:  Receives a GLINE (depending on the Loaded Protocol)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param servername The ServerName which sends us the gline information
 * @param mask The Mask which was glined
 * @param nick The Nick which added the gline
 * @param in_expiretime The Time when the gline should expire
 * @param in_hubtime The Time when the gline was set
 * @param in_reason The Reason why we that Host / IP was glined
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL on_gline(Threads *thread, char *servername, char *mask, char *nick, time_t in_expiretime, time_t in_hubtime, char *in_reason){
	/* Declare variables */
	char addedby[MAX_NICK_LEN];
	char ident_at_host[MAX_IDENT_LEN+1+MAX_HOST_LEN];
	time_t expire = 0;

	/* Initialize variables to NULL */
	bzero(&addedby,sizeof(addedby));
	bzero(&ident_at_host,sizeof(ident_at_host));

	/* Clean the delay in our Duration */
	expire = in_expiretime == 0 ? 0 : (time(NULL)+(in_expiretime-in_hubtime));

	/* Get the Glines setter */
	sscanf(nick,"%[^!]!%s",addedby,ident_at_host);
	
	if(DuringServerlink)
		MySQL_FlushInserts(thread,SQL_BUFFER_SJOIN,SQL_BUFFER_SJOIN); /* Flush Inserts - Since on serverlink or not; the SJOIN
																Event comes before TKL! */
	
	/*
	 > :markus.deepirc.net TKL + s * 35.35.35.35 shock!nMe@netadmin.deepirc.net 0 1077828248 :shun 2
	 > :markus.deepirc.net TKL + s * 36.36.36.36 shock!nMe@netadmin.deepirc.net 1077846231 1077828231 :shun 1
	 > :markus.deepirc.net TKL + Z * 36.36.36.36 shock!nMe@netadmin.deepirc.net 1077846221 1077828221 :gzline 2
	 > :markus.deepirc.net TKL + Z * 35.35.35.35 shock!nMe@netadmin.deepirc.net 0 1077828215 :gzline 1
	 > :markus.deepirc.net TKL + G * 36.36.36.36 shock!nMe@netadmin.deepirc.net 1077846209 1077828209 :gline 2
	 > :markus.deepirc.net TKL + G * 35.35.35.35 shock!nMe@netadmin.deepirc.net 0 1077828192 :gline 1
	*/
	domysql(thread,"INSERT INTO `orion_core_lines` (`type`,`mask`,`addedby`,`duration`,`reason`) VALUES ('G','%s','%s','%ld','%s') ON DUPLICATE KEY UPDATE "
	"`addedby`=VALUES(`addedby`),"
	"`duration`=VALUES(`duration`),"
	"`reason`=VALUES(`reason`)"
	,mysql_quote(thread,mask),mysql_quote(thread,addedby),expire,mysql_quote(thread,in_reason));
	
	if(streqnc(addedby,O->nick) || streqnc(addedby,SC->nick) || streqnc(addedby,Q->nick))
		domysql(thread,"INSERT INTO `orion_core_glines` (`rng`,`line`,`gline`,`duration`,`addedby`,`reason`) VALUES ('0','G','%s','%ld','%s','%s') ON DUPLICATE KEY UPDATE "
		"`duration`=VALUES(`duration`),"
		"`addedby`=VALUES(`addedby`),"
		"`reason`=VALUES(`reason`)"
		,mysql_quote(thread,mask),expire,mysql_quote(thread,addedby),mysql_quote(thread,in_reason));
	return CMD_EXECUTED;
}

/*  ************************************************************************************************  */

/**
 * @brief on_ungline:  Receives an UNGLINE (depending on the Loaded Protocol)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param servername The ServerName (services-server) which sends the ungline information
 * @param mask The Mask which we like to ungline
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL on_ungline(Threads *thread, char *servername, char *mask){
	/* Declare variables */
	deepSQL getSQL;
	
	/* Initialize variables to NULL */
	bzero(&getSQL,sizeof(getSQL));
	
	if(DuringServerlink)
		MySQL_FlushInserts(thread,SQL_BUFFER_SJOIN,SQL_BUFFER_SJOIN); /* Flush Inserts - Since on serverlink or not; the SJOIN
																Event comes before TKL! */
	
	/*
	 > :markus.deepirc.net TKL + s * 35.35.35.35 shock!nMe@netadmin.deepirc.net 0 1077828248 :shun 2
	 > :markus.deepirc.net TKL + s * 36.36.36.36 shock!nMe@netadmin.deepirc.net 1077846231 1077828231 :shun 1
	 > :markus.deepirc.net TKL + Z * 36.36.36.36 shock!nMe@netadmin.deepirc.net 1077846221 1077828221 :gzline 2
	 > :markus.deepirc.net TKL + Z * 35.35.35.35 shock!nMe@netadmin.deepirc.net 0 1077828215 :gzline 1
	 > :markus.deepirc.net TKL + G * 36.36.36.36 shock!nMe@netadmin.deepirc.net 1077846209 1077828209 :gline 2
	 > :markus.deepirc.net TKL + G * 35.35.35.35 shock!nMe@netadmin.deepirc.net 0 1077828192 :gline 1
	*/
	domysql(thread,"DELETE FROM `orion_core_lines` WHERE `type`='G' AND `mask`='%s'",mysql_quote(thread,mask));

	/* MySQL */
	getSQL.res    = doSelectMySQL(thread,"SELECT `line`,`gline`,`addedby`,`duration`,`reason` FROM `orion_core_glines` WHERE `gline`='%s' AND `line`='G' AND `rng`='0'",mysql_quote(thread,mask));
	getSQL.fields = mysql_num_rows(getSQL.res);
	getSQL.row    = mysql_fetch_row(getSQL.res);
	
	//dlog(thread,2,"mysql soweit leiwand\n");
	if(getSQL.fields){
		//dlog(thread,2,"getSQL.row[2] = %s\n",getSQL.row[2]);
		if(streqnc(getSQL.row[2],O->nick))
			do_gline(thread,m->servername,getSQL.row[1],_dynadd("%s!%s@%s",O->nick,O->ident,O->host),stol(getSQL.row[3]),time(NULL),getSQL.row[4]);
	
		else if(streqnc(getSQL.row[2],Q->nick))
			do_gline(thread,m->servername,getSQL.row[1],_dynadd("%s!%s@%s",Q->nick,Q->ident,Q->host),stol(getSQL.row[3]),time(NULL),getSQL.row[4]);
	
		else if(streqnc(getSQL.row[2],SC->nick))
			do_gline(thread,m->servername,getSQL.row[1],_dynadd("%s!%s@%s",SC->nick,SC->ident,SC->host),stol(getSQL.row[3]),time(NULL),getSQL.row[4]);
	}
	/* MySQL Free Result */
	mysql_free_result_myr(getSQL.res);
	return CMD_EXECUTED;
}

/*  ************************************************************************************************  */
