/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file log.c
 *   @brief This is the Module file for handling Logs
 *
 *   It contains all functions which are neccessary to
 *   handle Logfiles. (LOGFILES HAVE A THREAD_INDEX DEPENDENCY!)
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "global.h"
#include "services.h"
#include "modules.h"
#include "osocket.h"

#include <time.h>

static int curday = 0;

#ifdef _WIN32
/**
 * @brief gettimeofday:  Returns the time of day
 *
 * @param date The Date-String (an empty should be given, cauze we work on it)
 * @param count The size of the Date-String (sizeof(date))
 * @param tm The Time struct
 * @return int Returns 1
 **/
int gettimeofday(struct timeval *tv, struct timezone *tz)
{
  FILETIME ft;
  unsigned __int64 tmpres = 0;
  static int tzflag;
 
  if (NULL != tv)
  {
    GetSystemTimeAsFileTime(&ft);
 
    tmpres |= ft.dwHighDateTime;
    tmpres <<= 32;
    tmpres |= ft.dwLowDateTime;
 
    /*converting file time to unix epoch*/
    tmpres /= 10;  /*convert into microseconds*/
    tmpres -= DELTA_EPOCH_IN_MICROSECS; 
    tv->tv_sec = (long)(tmpres / 1000000UL);
    tv->tv_usec = (long)(tmpres % 1000000UL);
  }
 
  if (NULL != tz)
  {
    if (!tzflag)
    {
      _tzset();
      tzflag++;
    }
    tz->tz_minuteswest = _timezone / 60;
    tz->tz_dsttime = _daylight;
  }
 
  return 0;
}
#endif

/*  ************************************************************************************************  */

/**
 * @brief get_logdate:  Returns the Date we should use for the logs
 *
 * @param date The Date-String (an empty should be given, cauze we work on it)
 * @param count The size of the Date-String (sizeof(date))
 * @param tm The Time struct
 * @return int Returns 1
 **/
int get_logdate(char *date, int count, struct tm *tm)
{
    if (!tm) {
        time_t t;

        time(&t);
        tm = localtime(&t);
    }

    strftime(date, count, "%Y%m%d", tm);
    curday = tm->tm_yday;

    return 1;
}

/*  ************************************************************************************************  */

/**
 * @brief checkday:  Updates the Logs 
 *               (saves/closes/removes and opens the new Logs if necessary)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return static void
 **/
static void checkday(Threads *thread)
{
    time_t t;
    struct tm tm;

    time(&t);
    tm = *localtime(&t);

    if (curday != tm.tm_yday) {
        close_logs(thread,NULL);
        remove_logs(thread);
        open_logs(thread);
    }
}

/*  ************************************************************************************************  */

/**
 * @brief open_logs:  Opens the log files.
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return bool Returns -1 if the log file could not be opened, else
 **/
bool open_logs(Threads *thread)
{
    char name[150];
    char date[50];
    bzero(&name,sizeof(name));
	bzero(&date,sizeof(date));

    if (!get_logdate(date, sizeof(date), NULL))
        return 0;
    
    if(logging){
		snprintf(name,100,"logs/%s.%s.",m->logfile,thread->index);
		straddns(name,date,sizeof(name));
		thread->loghandle    = fopen(name   ,"a+");
		bzero(&name,sizeof(name));
	}
    if(sqllogging){
		snprintf(name,100,"logs/%s.%s.",m->sqllogfile,thread->index);
		straddns(name,date,sizeof(name));
		thread->sqlloghandle    = fopen(name   ,"a+");
		bzero(&name,sizeof(name));
	}
    if(m->debuglogging){
		snprintf(name,100,"logs/%s.%s.",m->debuglog,thread->index);
		straddns(name,date,sizeof(name));
		thread->debughandle    = fopen(name   ,"a+");
		bzero(&name,sizeof(name));
	}
	
    return 1;
}

/*  ************************************************************************************************  */

/**
 * @brief close_logs:  Closes all Logs (debug-log, SQL-Log, default-Log)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param lastmsg The last message to dumo to all logs
 * @return void
 **/
void close_logs(Threads *thread, char *lastmsg)
{
    if (thread->loghandle){
		if(lastmsg != NULL)
		    alog(thread,lastmsg);
		    
		fflush(thread->loghandle);
	    fclose(thread->loghandle);
	    thread->loghandle = NULL;
	}
    
    if (thread->sqlloghandle){
		if(lastmsg != NULL)
		    sqllog(thread,lastmsg);
		    
		fflush(thread->sqlloghandle);
	    fclose(thread->sqlloghandle);
	    thread->sqlloghandle = NULL;
	}
    
    if (thread->debughandle){
		if(lastmsg != NULL)
		    dlog(thread,0,lastmsg);
		    
		fflush(thread->debughandle);
	    fclose(thread->debughandle);
	    thread->debughandle = NULL;
	}
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief remove_logs:  Removes all Logs older than "m->KeepLogs" 
 *               (debug-log, SQL-Log, default-Log)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return void
 **/
void remove_logs(Threads *thread)
{
    time_t t;
    struct tm tm;

    char name[150];
    char date[50];
    bzero(&name,sizeof(name));
	bzero(&date,sizeof(date));

    if (!m->KeepLogs)
        return;

    time(&t);
    t -= (60 * 60 * 24 * m->KeepLogs);
    tm = *localtime(&t);

    if (!get_logdate(date, sizeof(date), &tm))
        return;
        
    if(logging){
		snprintf(name,100,"logs/%s.%s.",m->logfile,thread->index);
		straddns(name,date,sizeof(name));
	#ifndef _WIN32
	    unlink(name);
	#else
	    DeleteFile(name);
	#endif
		bzero(&name,sizeof(name));
	}
    if(sqllogging){
		snprintf(name,100,"logs/%s.%s.",m->sqllogfile,thread->index);
		straddns(name,date,sizeof(name));
	#ifndef _WIN32
	    unlink(name);
	#else
	    DeleteFile(name);
	#endif
		bzero(&name,sizeof(name));
	}
    if(m->debuglogging){
		snprintf(name,100,"logs/%s.%s.",m->debuglog,thread->index);
		straddns(name,date,sizeof(name));
	#ifndef _WIN32
	    unlink(name);
	#else
	    DeleteFile(name);
	#endif
		bzero(&name,sizeof(name));
	}
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief open_all_logs:  Opens all logs for ALL Threads
 *               (debug-log, SQL-Log, default-Log) - Logs are Thread-Index dependent!
 *
 * @return bool If opening logs succeded 1, otherwise 0
 **/
bool open_all_logs(void){
	Threads *thread = NULL;
	
	for (thread = threads; thread; thread = thread->next) {
		if(!thread){
			if(!open_logs(thread))
				return 0;
		}
	}
	return 1;
}

/*  ************************************************************************************************  */

/**
 * @brief close_all_logs:  Closes all logs of ALL Threads.
 *               (debug-log, SQL-Log, default-Log) - Logs are Thread-Index dependent!
 *
 * @param message The last message before closing logs
 * @return void
 **/
void close_all_logs(char *message, ...){
	Threads *thread = NULL;
	char tmp[BUFFER + 1];
	
	bzero(&tmp,sizeof(tmp));

	if(message != NULL){
		va_list va;
		va_start(va, message);
		vsnprintf(tmp, sizeof(tmp)-1, message, va);
		va_end(va);
	}
	
	for (thread = threads; thread; thread = thread->next) {
		if(!thread){
			if(tmp[0] == '\0')
				close_logs(thread,NULL);
			else
				close_logs(thread,tmp);
		}
	}
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief remove_all_logs:  Removes all logs of ALL Threads
 *               (debug-log, SQL-Log, default-Log) - Logs are Thread-Index dependent!
 *
 * @return void
 **/
void remove_all_logs(void){
	Threads *thread = NULL;
	
	for (thread = threads; thread; thread = thread->next) {
		if(!thread){
			remove_logs(thread);
		}
	}
	return;
}


/*  ************************************************************************************************  */

/**
 * @brief alog:  Writes to the common-log
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param message The message for the logfile
 * @return void
 **/
void alog(Threads *thread, const char *message, ...){
	if(message==NULL)
		return;

	else if(logging){
		checkday(thread);
		
		if(thread->loghandle){
			char tmp[BUFFER + 1];
			bzero(&tmp,sizeof(tmp));
			
			va_list va;
			va_start(va, message);
			vsnprintf(tmp, sizeof(tmp)-1, message, va);
			va_end(va);

#ifdef USE_THREADS
			my_pthread_mutex_lock(&alog_lock);
#endif
			fprintf(thread->loghandle, "%s",tmp);
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&alog_lock);
#endif
		}
	}
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief sqllog:  Writes to the sql-log
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param message The message for the sqllog
 * @return void
 **/
void sqllog(Threads *thread, const char *message, ...){
	time_t t;
	struct tm tm;
	int steps;
	if(message==NULL)
		return;

	else if(sqllogging){
		char buf[BUFFER + 1];
		va_list va;
		
		checkday(thread);
		
		va_start(va, message);
		time(&t);
		tm = *localtime(&t);
		
		char *s;
		struct timeval tv;
		gettimeofday(&tv, NULL);
		strftime(buf, sizeof(buf) - 1, "[%b %d %H:%M:%S", &tm);
		s = buf + strlen(buf);
		s += snprintf(s, sizeof(buf) - (s - buf), ".%06d",
					(int) tv.tv_usec);
		strftime(s, sizeof(buf) - (s - buf) - 1, " %Y] ", &tm);
		
		if(thread->sqlloghandle){
			fputs(buf, thread->sqlloghandle);
	
			// backtrace function name...
			bzero(&buf,sizeof(buf));
			if(message[0]=='S') steps=3; 
			else steps=4;
			
			get_last_function(buf,steps);
			fputs(buf, thread->sqlloghandle);
			// ---
			
#ifdef USE_THREADS
			my_pthread_mutex_lock(&sqllog_lock);
#endif
			vfprintf(thread->sqlloghandle, message, va);
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&sqllog_lock);
#endif
			va_end(va);
		}
	}
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief dlog:  Writes to the debuglog
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param priority The priority for debugging
 * @param message The message for the debuglog
 * @return void
 **/
void dlog(Threads *thread, int priority, const char *message, ...){
	if(message==NULL)
		return;

	else if(m->debug>=priority){
		checkday(thread);
		
		if(thread->debughandle){
			char tmp[BUFFER + 1];
			bzero(&tmp,sizeof(tmp));
			
			va_list va;
			va_start(va, message);
			vsnprintf(tmp, sizeof(tmp)-1, message, va);
			va_end(va);
			
			//printf("tmp = %s",tmp);	
#ifdef USE_THREADS
			my_pthread_mutex_lock(&globvar_lock);
			my_pthread_mutex_lock(&dlog_lock);
#endif
			if(m->debug>=1 && !dofork) fprintf(stderr,"%s",tmp);
				//printf("%s",tmp);
			if(m->debuglogging) fprintf(thread->debughandle, "%s",tmp);
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&dlog_lock);
			my_pthread_mutex_unlock(&globvar_lock);
#endif
		}
	}
	return;
}

/*  ************************************************************************************************  */
