/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 371                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-12-02 00:51:13 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file modules.c
 *   @brief This is the Module file for managing Modules.
 *
 *   It contains all functions which are neccessary to
 *   load/unload/call modules or commands from a specific Service.
 *
 *	 Includes the getmodules(); function - reads in all modules, that are defined in "etc/modules.conf"
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "global.h"
#include "services.h"
#include "modules.h"
#include "osocket.h"

char modul[20][MAX_NICK_LEN]; 						/* 20 modules | each MAX_NICK_LEN chars long */
void* languages[NUM_LANGS];
static ModCo *modulelist = NULL;
static ModConfig *ModuleConfig = NULL;

/**
 * @brief getmodules:  Reads the a config in etc/ and saves the modul handles
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return size_t Index of added module
 **/
size_t getmodules(Threads *thread){
	FILE *in;
	char line[100];
	char path[MAX_FILEPATH_LEN];
	size_t x=0;
	bzero(line,sizeof(line));
	bzero(path,sizeof(path));

	snprintf(path,sizeof(path),"%s/%s",SERVICES_CONF_DIR,m->LoadModulesConf);

#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_read_lock);
#endif
	in=fopen(path,"r");
	while(fgets(line,100-1,in)){
		bzero(&modul[x],sizeof(modul[x]));
		
		if(line[0] != '\0' && line[0] != '\n' && line[0] != '#'){
			strclean(line);
			dynncpy(modul[x],line,sizeof(modul[x]));
			x++;
		}
	}
	
	bzero(&modul[x],sizeof(modul[x]));
	fclose(in);
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&modules_read_lock);
#endif
	return x;
}

/*  ************************************************************************************************  */

/**
 * @brief dynopen:  synonym of dlopen() - used to open shared object-files
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param lib File to load - has to be a .so file (shared object)
 * @param flags Flags for dlopen()
 * @return handle Pointer to the opened file
 **/
void *dynopen(Threads *thread, char *lib, int flags){ //
	static void *handle = NULL;
	dlog(thread,2,"  *  Opening module: %s\n",lib);
	
	handle = _dynopen(lib);
	
	if (handle == NULL){
		dlog(thread,0,"  *  ERROR from dynopen();\n");
#ifdef _dynerror
		dlog(thread,0,"  *  ERROR [dynopen] - %s\n", _dynerror());
#endif
		remove_pidfile();
		_orion_exit();
	}
	
	dlog(thread,2,"  *  Library opened.\n");
	return handle;
}

/*  ************************************************************************************************  */

/**
 * @brief dynsym:  synonym of dlsym() - used to search loaded .so files for a symbol
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param handle Pointer to the shared object file received from dynopen();
 * @param func The symbol to look for (a function in most cases)
 * @return funcptr Pointer to the function found by dynsym();
 **/
void *dynsym (Threads *thread, void *handle, char *func){ //
#ifndef _WIN32
  #ifdef DL_PREFIX
	char newfunc[200];
	bzero(&newfunc,sizeof(newfunc));
	
	stradds(newfunc,DL_PREFIX);
	stradds(newfunc,func);
	
	void *funcptr = _dynsym(handle,newfunc);
  #else
	void *funcptr = _dynsym(handle,func);
  #endif
#else
	void *funcptr = _dynsym(handle,func);
#endif
	
	if (funcptr == NULL){
		dlog(thread,0,"  *  ERROR from dynsym();\n");
#ifdef _dynerror
		dlog(thread,0,"  *  ERROR [dynsym] - %s\n", _dynerror());
#endif
		dynclose(thread,handle);
	}
	return funcptr;
}

/*  ************************************************************************************************  */

/**
 * @brief dynclose:  Closes a .so library, by using the object handle (received from dynopen())
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param handle Pointer to the shared object file received from dynopen();
 * @return void
 **/
void dynclose (Threads *thread, void *handle){ //
	if(_dynclose (handle)){
		dlog(thread,0,"  *  ERROR from dynclose();\n");
#ifdef _dynerror
		dlog(thread,0,"  *  ERROR [dynclose] - %s\n", _dynerror());
#endif
		remove_pidfile();
		_orion_exit();
	}
	dlog(thread,2,"  *  Library closed.\n");
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief init_langs:  Opens all language files
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return void
 **/
void init_langs(Threads *thread){
	char langfile[MAX_FILEPATH_LEN];
	bzero(&langfile,sizeof(langfile));
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&globvar_lock);
#endif
	snprintf(langfile,sizeof(langfile),"%s/en%s",SERVICES_LANG_DIR,MODULE_EXT);
	languages[EN]=dynopen(thread,langfile,RTLD_NOW);
	bzero(langfile,sizeof(langfile));
	
	snprintf(langfile,sizeof(langfile),"%s/de%s",SERVICES_LANG_DIR,MODULE_EXT);
	languages[DE]=dynopen(thread,langfile,RTLD_NOW);
	bzero(langfile,sizeof(langfile));

	snprintf(langfile,sizeof(langfile),"%s/fr%s",SERVICES_LANG_DIR,MODULE_EXT);
	languages[FR]=dynopen(thread,langfile,RTLD_NOW);
	bzero(langfile,sizeof(langfile));

	snprintf(langfile,sizeof(langfile),"%s/es%s",SERVICES_LANG_DIR,MODULE_EXT);
	languages[ES]=dynopen(thread,langfile,RTLD_NOW);
	bzero(langfile,sizeof(langfile));

	snprintf(langfile,sizeof(langfile),"%s/it%s",SERVICES_LANG_DIR,MODULE_EXT);
	languages[IT]=dynopen(thread,langfile,RTLD_NOW);
	bzero(langfile,sizeof(langfile));

	snprintf(langfile,sizeof(langfile),"%s/pl%s",SERVICES_LANG_DIR,MODULE_EXT);
	languages[PL]=dynopen(thread,langfile,RTLD_NOW);
	bzero(langfile,sizeof(langfile));
	languages[LASTLANG]=NULL;
	
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&globvar_lock);
#endif
	return;	
}

/*  ************************************************************************************************  */

/**
 * @brief reinit_langs:  Opens all language files
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return void
 **/
void reinit_langs(Threads *thread){
	char langfile[MAX_FILEPATH_LEN];
	bzero(&langfile,sizeof(langfile));

#ifdef USE_THREADS
	my_pthread_mutex_lock(&globvar_lock);
#endif
	dynclose(thread,languages[EN]);
	snprintf(langfile,sizeof(langfile),"%s/en%s",SERVICES_LANG_DIR,MODULE_EXT);
	languages[EN]=dynopen(thread,langfile,RTLD_NOW);
	bzero(langfile,sizeof(langfile));
				
	dynclose(thread,languages[DE]);
	snprintf(langfile,sizeof(langfile),"%s/de%s",SERVICES_LANG_DIR,MODULE_EXT);
	languages[DE]=dynopen(thread,langfile,RTLD_NOW);
	bzero(langfile,sizeof(langfile));
				
	dynclose(thread,languages[FR]);
	snprintf(langfile,sizeof(langfile),"%s/fr%s",SERVICES_LANG_DIR,MODULE_EXT);
	languages[FR]=dynopen(thread,langfile,RTLD_NOW);
	bzero(langfile,sizeof(langfile));
				
	dynclose(thread,languages[ES]);
	snprintf(langfile,sizeof(langfile),"%s/es%s",SERVICES_LANG_DIR,MODULE_EXT);
	languages[ES]=dynopen(thread,langfile,RTLD_NOW);
	bzero(langfile,sizeof(langfile));
				
	dynclose(thread,languages[IT]);
	snprintf(langfile,sizeof(langfile),"%s/it%s",SERVICES_LANG_DIR,MODULE_EXT);
	languages[IT]=dynopen(thread,langfile,RTLD_NOW);
	bzero(langfile,sizeof(langfile));
				
	dynclose(thread,languages[PL]);
	snprintf(langfile,sizeof(langfile),"%s/pl%s",SERVICES_LANG_DIR,MODULE_EXT);
	languages[PL]=dynopen(thread,langfile,RTLD_NOW);
	bzero(langfile,sizeof(langfile));
				
	languages[LASTLANG]=NULL;
	
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&globvar_lock);
#endif
}

/*  ************************************************************************************************  */

/**
 * @brief ModuleError:  Returns a readable Error
 *
 * @param MOD_ERROR The ErrorEnum
 * @return char *
 **/
char *ModuleError(MOD_ERROR ModErr){
	const char* ModuleErrors[] = {
/* 00 */"Module, Okay - No Error",																			/* MOD_ERR_OK */
/* 01 */"Module Error, Allocating memory",																	/* MOD_ERR_MEMORY */
/* 02 */"Module Error, Not enough parameters",																/* MOD_ERR_PARAMS */
/* 03 */"Module Error, Already loaded",																		/* MOD_ERR_EXISTS */
/* 04 */"Module Error, Modul does not exist",																/* MOD_ERR_NOEXIST */
/* 05 */"Module Error, Command Already loaded",																/* MOD_ERR_CMD_EXIST */
/* 06 */"Module Error, Command not loaded",																	/* MOD_ERR_CMD_NOEXIST */
/* 07 */"Module Error, Options don't match",																/* MOD_ERR_OPTIONS */
/* 08 */"Module Error, Unable to load module",																/* MOD_ERR_NOTLOADED */
/* 09 */"Module Error, Unable to unload module (possible error in MODUL_NAME)",								/* MOD_ERR_NOTUNLOADED */
/* 10 */"Module Error, First loaded module of this command (and not unique)",								/* MOD_ERR_NOUNIQUE */
/* 11 */"Module Error, During load - Module cancelled loading",												/* MOD_ERR_CANCELLED */
/* 12 */"Module Error, No target for given function",														/* MOD_ERR_NO_FUNCTARGET */
/* 13 */"Module Error, Moduleconfig not found (no modules loaded)",											/* MOD_ERR_FILE_IO */
/* 14 */"Module Error, Creating Callback failed!",															/* MOD_ERR_CALLBACK	*/
/* 15 */"Module Error, Setting is already set (you can't overwrite it)!",      								/* MOD_ERR_SETTING_EXISTS */
/* 16 */"Module Error, You haven't used ModuleInit(); - services shutdown, to avoid further problems!"		/* MOD_ERR_NOTUSEDINIT */
	};
    return (char *) ModuleErrors[ModErr];
}

/*  ************************************************************************************************  */

/**
 * @brief is_same_prototype:  Proofes if the Moduls prototype format is the same as the one we want to load
 *
 * @param prototype_a The prototype of modul A
 * @param prototype_b The prototype of modul B
 * @return bool
 **/
bool is_same_prototype(char *prototype_a, char *prototype_b){
	int params_in_a = 0;
	int params_in_b = 0;
	
	params_in_a = findchar(prototype_a,' ')+1;
	params_in_b = findchar(prototype_b,' ')+1;
	
	params_in_a -= findchar(prototype_a,'*');
	params_in_b -= findchar(prototype_b,'*');
	
	return (params_in_a == params_in_b ? true : false);
}

/*  ************************************************************************************************  */

/**
 * @brief ModuleInit:  Initialises a modul
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The object handle (received by using dlopen())
 * @param MODposition Position where to add the module (head of list?, tail of list?)
 * @param Service_in Service to which the modul belongs to
 * @param Filepath Path where the object-file could be found
 * @param Module_in Call parameter - identifier on which the module has to react (IRCkeyword)
 * @param prototype The protocol prototype for calling commands included in this modul
 * @param helpprototype The protocol helpprototype for calling HELP commands included in this modul
 * @return ModInfo ModInfo for our modulholder
 **/
ModInfo ModuleInit(Threads *thread, void *dl_handle, MODposition ModPos, char *Service_in, char *Filepath, char *Module_in, char *prototype, char *helpprototype){
	ModInfo ModHandle;
	
	ModCo *tail		= NULL;
	ModHandle.module= NULL;

	ModHandle.module = findModule(thread, Service_in, Module_in);
	
	if(!ModHandle.module && ModPos!=MOD_UNIQUE){
		dlog(thread,0,
			"Module Error: Module [%s] for Service [%s] is not a UNIQUE module (but the first loaded of it's setup)\n"
			"(possibly you loaded a 3rd party module BEFORE loading the unique module)\n"
			"NOTE: Take in mind that 3rd party (or self written modules shouldn't be UNIQUE too, only ours should be)\n"
			,Module_in,Service_in
		);
		ModHandle.ModErr 	= MOD_ERR_NOUNIQUE;
		ModHandle.module	= NULL;
		return ModHandle;
	}
	
	if(ModHandle.module && ModPos==MOD_UNIQUE){
		dlog(thread,0,
			"Module Error: Module [%s] for Service [%s] already has a UNIQUE module.\n"
			"(possibly you loaded a 3rd party module BEFORE loading the unique module)\n"
			"NOTE: Take in mind that 3rd party (or self written modules shouldn't be UNIQUE too, only ours should be)\n"
			,Module_in,Service_in
		);
		ModHandle.ModErr 	= MOD_ERR_EXISTS;
		ModHandle.module	= NULL;
		return ModHandle;
	}
		
    if (ModPos < 0 || ModPos >= KNOWN_POSITIONS){
		ModHandle.ModErr 	= MOD_ERR_PARAMS;
		ModHandle.module	= NULL;
		return ModHandle;
    }

	/* Set us to default - just to ensure we don't produce errors in this part */
	ModHandle.module = NULL;
	ModHandle.ModErr = MOD_ERR_OK;
	
	/* Start adding our new Module */
#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
#endif
	ModHandle.module = (ModCo*)malloc(sizeof(ModCo));
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&modules_lock);
#endif

	/* First of all we clear our new Module, to ensure we really from scratch */
    bzero(ModHandle.module, sizeof(ModCo));

	/* We set our handle (makes us able to close the .so/.dll file again */
	ModHandle.module->dl_handle = dl_handle;
	ModHandle.module->commands 	= NULL;
	
	/* We set the module it's needed Information */
	dynncpy(ModHandle.module->servicetag, 		Service_in,		sizeof(ModHandle.module->servicetag));
	dynncpy(ModHandle.module->modulename, 		Module_in,		sizeof(ModHandle.module->modulename));
	dynncpy(ModHandle.module->filename, 		Filepath,		sizeof(ModHandle.module->filename));
	dynncpy(ModHandle.module->prototype,		prototype,		sizeof(ModHandle.module->prototype));
	dynncpy(ModHandle.module->helpprototype,	helpprototype,	sizeof(ModHandle.module->helpprototype));
		
	/* TERMINATE OTHER STRUCTURE MEMBERS!! - VERY IMPORTANT, DO NOT DELETE */
	bzero(ModHandle.module->Version, 							sizeof(ModHandle.module->Version));
	bzero(ModHandle.module->Author, 							sizeof(ModHandle.module->Author));
	bzero(ModHandle.module->configfile, 						sizeof(ModHandle.module->configfile));
	ModHandle.module->type 				= (MODType)		0;
	ModHandle.module->access 			= (MODACCESS)	0;
	ModHandle.module->options 			= 0;
	ModHandle.module->func_finish 		= NULL;
	
	ModHandle.module->LeastUserLevel 	= INFINIT;
	ModHandle.module->LeastRootLevel 	= INFINIT;
	/***********************************************************************/

#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
#endif
    
    /* Now we add our Module to the list - but where? */
    if (ModPos == MOD_HEAD || ModPos == MOD_UNIQUE) {
		
		/* If we add it to the HEAD or make it UNIQUE we go on processing here */
		/* Head modules are called before others (even before the UNIQUE, that's the sense ... */
		
		ModHandle.module->next = modulelist;
		ModHandle.module->prev = NULL;
		if (modulelist)
			modulelist->prev = ModHandle.module;
    
		modulelist = ModHandle.module;

        dlog(thread,3,"debug: existing cmd: (0x%p), new cmd (0x%p)\n",
                    (void *) ModHandle.module->next, (void *) ModHandle.module);

        ModHandle.ModErr = MOD_ERR_OK;
    } else if (ModPos == MOD_TAIL) {
		/* If we add it to the TAIL we go on processing here */

		/* Am Anfang der Liste starten */
		tail = modulelist;

		/* Das ende der Liste suchen */
       	while (tail->next)
            tail = tail->next;

		/* Der Nachfolger des letzten Knotens
			ist nicht definiert, also NULL */
 		ModHandle.module->next = NULL;

		/* Der Vorgaenger des neuen Knotens
			ist der bislang letzte Knoten */
		ModHandle.module->prev = tail;

		if (modulelist == NULL)
		/* Falls noch kein Listenanfang definiert ist,
			dann ist der gerade angelegte Knoten der
			Anfang der Liste */
			modulelist =  ModHandle.module;

		else{

			/* Der Nachfolger des bislang letzten Knotens
				ist der gerade neu angelegte Knoten */
			tail->next = ModHandle.module;

		}

		/* Der gerade neu angelegte Knoten
			ist nun der letzte Knoten */
		tail = ModHandle.module;
		

        dlog(thread,3,"debug: existing cmd: (0x%p), new cmd (0x%p)\n",
                    (void *) tail, (void *) ModHandle.module);


        ModHandle.ModErr = MOD_ERR_OK;
    }
    
    ModHandle.ModErr = MOD_ERR_OK;
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&modules_lock);
#endif
	return ModHandle;
}

/*  ************************************************************************************************  */

/**
 * @brief addMODType:  Adds a type to the modul, CORE / EVENT
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param ModHandle The ModInfo Pointer to the modul where the type should be set to
 * @param itype Modultyp to be set to target modul
 * @return MOD_ERROR
 **/
MOD_ERROR addMODType(Threads *thread, ModInfo *ModHandle, MODType itype){
	if(ModHandle->module == NULL)
		return MOD_ERR_NOEXIST;
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
	if(ModHandle->module->type)
		return MOD_ERR_SETTING_EXISTS;

	ModHandle->module->type = itype;
	my_pthread_mutex_unlock(&modules_lock);
#else
	if(ModHandle->module->type)
		return MOD_ERR_SETTING_EXISTS;

	ModHandle->module->type = itype;
#endif
	return MOD_ERR_OK;
}

/*  ************************************************************************************************  */

/**
 * @brief addOptions:  Adds Options to target command
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param ModHandle The ModInfo* Pointer to the modul where the type should be set to
 * @param parameters The Number of options to be added
 * @return MOD_ERROR
 **/
MOD_ERROR addOptions(Threads *thread, ModInfo *ModHandle, int parameters, ...){
	int options = 0;
	int arg 	= 0;
	int i 		= 0;
	
	if(ModHandle->module == NULL)
		return MOD_ERR_NOEXIST;
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
#endif
	if(ModHandle->module->options){
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&modules_lock);
#endif
    	return MOD_ERR_SETTING_EXISTS;
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&modules_lock);
#endif
    	
	va_list function;
	va_start(function, parameters);
    
	arg = va_arg(function,int);
	for(i=0;i<parameters;i++){
		options += (1<<arg);
		arg = va_arg(function,int);
	}
	va_end(function);
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
#endif
	ModHandle->module->options = options;
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&modules_lock);
#endif
	return MOD_ERR_OK;
}

/*  ************************************************************************************************  */

/**
 * @brief addAuthor:  Adds an author to target modul
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param ModHandle The ModInfo* Pointer to the modul where the author should be set to
 * @param Author The Author of target module
 * @return MOD_ERROR
 **/
MOD_ERROR addAuthor(Threads *thread, ModInfo *ModHandle, const char *Author, ...){
	if(ModHandle->module == NULL)
		return MOD_ERR_NOEXIST;
		
#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
	if(ModHandle->module->Author[0] != '\0'){
		my_pthread_mutex_unlock(&modules_lock);
		return MOD_ERR_SETTING_EXISTS;
	}
	my_pthread_mutex_unlock(&modules_lock);
#else
	if(ModHandle->module->Author[0] != '\0'){
		return MOD_ERR_SETTING_EXISTS;
	}
#endif

    char auth[70];
    va_list args;
    
    bzero(&auth,sizeof(auth));
    
    va_start	(args, Author);
    vsnprintf	(auth, sizeof(auth)-1, Author, args);
    va_end		(args);
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
#endif
	dynncpy(ModHandle->module->Author, auth, sizeof(ModHandle->module->Author));
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&modules_lock);
#endif
	return MOD_ERR_OK;
}

/*  ************************************************************************************************  */

/**
 * @brief addVersion:  Adds a version for target modul
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param ModHandle The ModInfo* Pointer to the modul where the author should be set to
 * @param Version The Version of target module
 * @return MOD_ERROR
 **/
MOD_ERROR addVersion(Threads *thread, ModInfo *ModHandle, const char *Version, ...){
#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
#endif
	if(ModHandle->module == NULL)
		return MOD_ERR_NOEXIST;

	if(ModHandle->module->Version[0] != '\0'){
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&modules_lock);
#endif
		return MOD_ERR_SETTING_EXISTS;
	}
#ifdef USE_THREADS
	else my_pthread_mutex_unlock(&modules_lock);
#endif

    char vers[200];
    va_list args;
    
    bzero(&vers,sizeof(vers));
    
    va_start	(args, Version);
    vsnprintf	(vers, sizeof(vers)-1, Version, args);
    va_end		(args);
    
#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
#endif
	dynncpy(ModHandle->module->Version, vers, sizeof(ModHandle->module->Version));
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&modules_lock);
#endif
	return MOD_ERR_OK;
}

/*  ************************************************************************************************  */

/**
 * @brief addAccess:  Adds an access-type for target modul
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param ModHandle The ModInfo* Pointer to the modul where the author should be set to
 * @param access The access type for target module [USER/ADMIN/BOTH]
 * @return MOD_ERROR
 **/
MOD_ERROR addAccess(Threads *thread, ModInfo *ModHandle, MODACCESS access){
#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
#endif

	if(ModHandle->module == NULL)
		return MOD_ERR_NOEXIST;

	if(ModHandle->module->access){
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&modules_lock);
#endif
    	return MOD_ERR_SETTING_EXISTS;
	}
	
	ModHandle->module->access = access;
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&modules_lock);
#endif
	return MOD_ERR_OK;
}

/*  ************************************************************************************************  */

/**
 * @brief CheckOptions:  Adds Options to target command
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param modul The ModCo* Pointer to the modul where the type should be set to
 * @param snick The ServiceNick
 * @param rnick The receiver (target user)
 * @return MOD_REPLY
 **/
MOD_REPLY CheckOptions(Threads *thread, ModCo *module, char *snick, char *rnick){
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
#endif
	if(thread->Webform){
		
		if(~module->options & WEBACCESS_BIT){
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&modules_lock);
#endif
			return MOD_REPLY_OPTIONS_NO_WEBACCESS_GRANTED;
		}
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&modules_lock);
#endif
		return MOD_REPLY_NONE;
	}
	
	
	if(module->options & LOGINONLY_BIT){
		if(!thread->sql.aliases.active && !thread->Webform/* Webform extension*/){

#ifdef USE_THREADS
			my_pthread_mutex_unlock(&modules_lock);
#endif
			return MOD_REPLY_OPTIONS_NOT_LOGGED_IN;
		}
	}
	if(!strcor(thread->sql.Cdata.modes,"o")){
		if(module->options & OPER_IF_DEFCON_BIT && defcon<=3){

#ifdef USE_THREADS
			my_pthread_mutex_unlock(&modules_lock);
#endif
			return MOD_REPLY_OPTIONS_DEFCON_TOO_HIGH;
		}
		else if(module->options & READONLY_BIT && readonly){

#ifdef USE_THREADS
			my_pthread_mutex_unlock(&modules_lock);
#endif
			return MOD_REPLY_OPTIONS_READONLY;
		}
		else if(module->options & OPERONLY_BIT && !thread->Webform){
			
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&modules_lock);
#endif
			return MOD_REPLY_OPTIONS_NOT_AN_IRCOP;
		}
	}
	
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&modules_lock);
#endif
	return MOD_REPLY_NONE;
}

/*  ************************************************************************************************  */

/**
 * @brief CheckOption:  Checks a option to be added for target modul
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param snick The ServiceNick
 * @param rnick The receiver (target user)
 * @param option The option we want to proof
 * @return bool
 **/
bool CheckOption(Threads *thread, char *snick, char *rnick, int option){
	if(thread->Webform) return true;
	
	switch(option){
		case LOGINONLY:
			if(thread->sql.aliases.active ||  thread->Webform/* Webform extension*/)
				return true;
			else{
				send_lang(thread,languages[thread->sql.access.language],thread->sql.access.settings&NS_SET_MSG ? 1 : 0,snick,rnick,"NOT_LOGGED_IN");
				return false;
			}
			break;
		case READONLY:
			if(!readonly || (readonly && strcor(thread->sql.Cdata.modes,"o")))
				return true;
			else{
				send_lang(thread,languages[thread->sql.access.language],thread->sql.access.settings&NS_SET_MSG ? 1 : 0,snick,rnick,"READONLY_MODE");
				return false;
			}
			break;
		case OPERONLY:
			if(strcor(thread->sql.Cdata.modes,"o"))
				return true;
			else{
				send_lang(thread,languages[thread->sql.access.language],thread->sql.access.settings&NS_SET_MSG ? 1 : 0,snick,rnick,"NOT_AN_IRCOP");
				return false;
			}
			break;
		case OPER_IF_DEFCON:
			if(defcon<=3 && !strcor(thread->sql.Cdata.modes,"o"))
				return true;
			else{
				send_lang(thread,languages[thread->sql.access.language],thread->sql.access.settings&NS_SET_MSG ? 1 : 0,snick,rnick,"DEFCON_TOO_HIGH",defcon);
				return false;
			}
			break;
		case WEBACCESS:
			/* not needed */
			break;
	}
	return false;
}

/*  ************************************************************************************************  */

/**
 * @brief CheckParamters:  Checks if there are too many or too few arguments 
 *                   to a command
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param module The module which we have to proof
 * @param command The command which we have to proof
 * @param param The parameter-array
 * @param snick The ServiceNick
 * @param rnick The receiver (target user)
 * @return MOD_REPLY
 **/
MOD_REPLY CheckParamters(Threads *thread, ModCo *module, char *command, char *param[], char *snick, char *rnick){
	CMD *c = findCommand(thread,module,command);
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
#endif
	if(c != NULL){
		if(module->type==CORE || module->type==THIRD){

			if(c->high_param != 0 && param[c->high_param+1]){

#ifdef USE_THREADS
				my_pthread_mutex_unlock(&modules_lock);
#endif
				return MOD_REPLY_TOO_MANY_PARAMS;
			}
			else if(!param[c->low_param]){

#ifdef USE_THREADS
				my_pthread_mutex_unlock(&modules_lock);
#endif
				return MOD_REPLY_LESS_PARAMS;
			}
			
		}
		return MOD_REPLY_NONE;
	}
	
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&modules_lock);
#endif
	return MOD_REPLY_NONE;
}

/*  ************************************************************************************************  */

/**
 * @brief ModReply:  Checks for errors occured while executing
 *                   and tell us on IRC about it
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param modcheck The modulcheck struct (here we save the errors)
 * @param snick The ServiceNick
 * @param nick The receiver (target user)
 * @param param The parameter-array
 * @return bool
 **/
bool ModReply(Threads *thread, ModCheck *modcheck, char *snick, char *nick, char *param[]){
	
	/* Now we perform the Options */
	if(modcheck->ModErrOptions == MOD_REPLY_OPTIONS_NOT_LOGGED_IN){
		send_lang(thread,languages[thread->sql.access.language],thread->sql.access.settings&NS_SET_MSG ? 1 : 0,snick,nick,"NOT_LOGGED_IN");
		return true;
	}
	
	else if(modcheck->ModErrOptions == MOD_REPLY_OPTIONS_DEFCON_TOO_HIGH){
		send_lang(thread,languages[thread->sql.access.language],thread->sql.access.settings&NS_SET_MSG ? 1 : 0,snick,nick,"DEFCON_TOO_HIGH",defcon);
		return true;
	}
	
	else if(modcheck->ModErrOptions == MOD_REPLY_OPTIONS_READONLY){
		send_lang(thread,languages[thread->sql.access.language],thread->sql.access.settings&NS_SET_MSG ? 1 : 0,snick,nick,"READONLY_MODE");
		return true;
	}
	
	else if(modcheck->ModErrOptions == MOD_REPLY_OPTIONS_NOT_AN_IRCOP){
		send_lang(thread,languages[thread->sql.access.language],thread->sql.access.settings&NS_SET_MSG ? 1 : 0,snick,nick,"NOT_AN_IRCOP");
		return true;
	}
	
	else if(modcheck->ModErrOptions == MOD_REPLY_OPTIONS_NO_WEBACCESS_GRANTED){
		send_lang(thread,languages[thread->sql.access.language],thread->sql.access.settings&NS_SET_MSG ? 1 : 0,snick,nick,"MODULE_NOT_ACCESSABLE_VIA_WEB");
		return true;
	}
	
	/* Now we perform the Parameters */
	else if(modcheck->ModErrParams == MOD_REPLY_TOO_MANY_PARAMS){
		send_lang(thread,languages[thread->sql.access.language],thread->sql.access.settings&NS_SET_MSG ? 1 : 0,snick,nick,"TOO_MANY_ARGUMENTS");
		return true;
	}
	
	else if(modcheck->ModErrParams == MOD_REPLY_LESS_PARAMS){
		send_lang(thread,languages[thread->sql.access.language],thread->sql.access.settings&NS_SET_MSG ? 1 : 0,snick,nick,"MISSING_PARAMETER");
		return true;
	}
	
	else if(modcheck->executed > 0)
		return true;
		
	return false;
}

/*  ************************************************************************************************  */

/**
 * @brief addFinish:  Adds a Finish Function to target modul
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param ModHandle The ModInfo* Pointer to the modul where the author should be set to
 * @param Finish_Func The pointer to the finish function
 * @return MOD_ERROR
 **/
MOD_ERROR addFinish(Threads *thread, ModInfo *ModHandle, MOD_BOOL (*Finish_Func) (Threads *thread, DO_ACTION Action)){
#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
#endif

	if(ModHandle->module == NULL)
		return MOD_ERR_NOEXIST;
    
	if(ModHandle->module->func_finish != NULL){
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&modules_lock);
#endif
    	return MOD_ERR_NO_FUNCTARGET;
	}
    	
	ModHandle->module->func_finish = Finish_Func;
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&modules_lock);
#endif
	return MOD_ERR_OK;
}

/*  ************************************************************************************************  */

/**
 * @brief addCommand:  Adds a command to target modul
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param ModHandle The ModInfo* Pointer to the modul where the command should be added to
 * @param call The identifier for calling the command
 * @param func The Callback function if our command is executed
 * @param root_level The root command level for that command
 * @param user_level The user command level for that command
 * @param low_param The minimum of parameters for this command
 * @param high_param The maximum of parameters for this command
 * @return MOD_ERROR
 **/
MOD_ERROR addCommand(Threads *thread, ModInfo *ModHandle,
	const char *call,
	MOD_BOOL (*func) (Threads *thread, void* language, bool howtosend, char *snick, char *nick, char *param[]),
	int root_level,
	int user_level,
	size_t low_param,
	size_t high_param
	)
{
	CMD *c 		= NULL;
	
    if(ModHandle->module == NULL)
		return MOD_ERR_NOEXIST;
		
#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
#endif
	c = (CMD*)malloc(sizeof(CMD));
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&modules_lock);
#endif
    bzero(c,sizeof(CMD));
    
	c->root_level 	= root_level;
	c->user_level 	= user_level;
	c->low_param 	= low_param;
	c->high_param 	= high_param;
	
	c->func = func;
	strcpy(c->call, call);
	
	if(func == NULL){
		ModHandle->module->LeastUserLevel = user_level;
		ModHandle->module->LeastRootLevel = root_level;
	}
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
#endif
    c->next = ModHandle->module->commands;
    c->prev = NULL;
    if (ModHandle->module->commands)
        ModHandle->module->commands->prev = c;
        
    ModHandle->module->commands = c;
    
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&modules_lock);
#endif
    return MOD_ERR_OK;
}


/*  ************************************************************************************************  */

/**
 * @brief addEventCallback:  This is to add an EventHook, after we already finished
 *                  introducing the Eventhandler and want an additional Event Hook
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param ModHandle The ModInfo* Pointer to the module where the command should be added to
 * @param call The identifier for calling the command
 * @param func The Callback function to be executed if our added event occurs
 * @return MOD_ERROR
 **/
MOD_ERROR addEventCallback(Threads *thread, ModInfo *ModHandle, MODposition ModPos,
	const char *call,
	MOD_BOOL (*func) (Threads *thread, void* language, bool howtosend, char *snick, char *nick, char *param[])
	)
{
	
	CMD *tail 	= NULL;
	CMD *c 		= NULL;
	
    if(ModHandle->module == NULL)
		return MOD_ERR_NOEXIST;
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
#endif
	c = (CMD*)malloc(sizeof(CMD));
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&modules_lock);
#endif
    bzero(c,sizeof(CMD));
    
    if(func == NULL){
		return MOD_ERR_PARAMS;
	}
    
    /* It's a Callback so we leave it 0 */
	c->root_level 	= 0;
	c->user_level 	= 0;
	c->low_param 	= 0;
	c->high_param 	= 0;
    /* ******************************** */
	
	c->func = func;
	strcpy(c->call, call);
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
#endif
    if (ModPos == MOD_HEAD) {	
    	c->next = ModHandle->module->commands;
    	c->prev = NULL;
    	if (ModHandle->module->commands)
        	ModHandle->module->commands->prev = c;
        
    	ModHandle->module->commands = c;

		dlog(thread,3,"debug: existing cmd: (0x%p), new cmd (0x%p)\n",
                    (void *) c->next, (void *) c);
                    
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&modules_lock);
#endif
        return MOD_ERR_OK;
    } else if (ModPos == MOD_TAIL) {

		/* Am Anfang der Liste starten */
		tail = ModHandle->module->commands;

		/* Das ende der Liste suchen */
        while (tail->next)
            tail = tail->next;

		/* Der Nachfolger des letzten Knotens
			ist nicht definiert, also NULL */
 		c->next = NULL;
 		
		/* Der Vorgaenger des neuen Knotens
			ist der bislang letzte Knoten */
		c->prev = tail;
		
		if (ModHandle->module->commands == NULL)
		/* Falls noch kein Listenanfang definiert ist,
			dann ist der gerade angelegte Knoten der
			Anfang der Liste */
			ModHandle->module->commands = c;
			
		else{
		/* Der Nachfolger des bislang letzten Knotens
			ist der gerade neu angelegte Knoten */
			tail->next = c;

		}
		
		/* Der gerade neu angelegte Knoten
			ist nun der letzte Knoten */
		tail = c;

		dlog(thread,3,"debug: existing cmd: (0x%p), new cmd (0x%p)\n",
                    (void *) tail, (void *) c);

#ifdef USE_THREADS
		my_pthread_mutex_unlock(&modules_lock);
#endif
        return MOD_ERR_OK;
    }

#ifdef USE_THREADS
	my_pthread_mutex_unlock(&modules_lock);
#endif
    return MOD_ERR_CALLBACK;
}

/*  ************************************************************************************************  */

/**
 * @brief addLeastModuleLevels:  Tells a module it's least levels to be executed
 *   								(used to make it visible on i.e. /cs help overview helps)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param ModHandle The ModInfo* Pointer to the modul where the command should be added to
 * @param call The identifier for calling the command
 * @param root_level The root command level for that command
 * @param user_level The user command level for that command
 * @return MOD_ERROR
 **/
MOD_ERROR addLeastModuleLevels(Threads *thread, ModInfo *ModHandle,
	const char *call,
	int root_level,
	int user_level
	)
{
	return addCommand(thread,ModHandle,call,NULL,root_level,user_level,0,0);
}


/*  ************************************************************************************************  */

/**
 * @brief addEventHook:  Adds a new eventhandling function (listening for a specific Event)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param ModHandle The ModInfo* Pointer to the modul where the command should be added to
 * @param call The identifier for calling the command
 * @param func The Callback function to be executed if our added event occurs
 * @return MOD_ERROR
 **/
MOD_ERROR addEventHook(Threads *thread, ModInfo *ModHandle,
	const char *call,
	MOD_BOOL (*func) (Threads *thread, void* language, bool howtosend, char *snick, char *nick, char *param[])
	)
{
	return addCommand(thread,ModHandle,call,func,INFINIT,INFINIT,0,0);
}

/*  ************************************************************************************************  */

/**
 * @brief addModuleConfig:  Adds a new Configstring
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param item The item to identify it (the configvariables name)
 * @param value The value
 * @param to_modulefile The modulefile to which we like to add these strings.
 * @return bool
 **/
bool addModuleConfig(Threads *thread, char *item, char *value, char *to_modulefile){
    if(item == NULL || value == NULL || thread == NULL || to_modulefile == NULL)
		return false;

	if(dynlen(item)>sizeof(ModuleConfig->item))
	    return false;
	    
	if(dynlen(value)>sizeof(ModuleConfig->value))
	    return false;

	ModConfig *Config = NULL;

#ifdef USE_THREADS
	my_pthread_mutex_lock(&shconfig_lock);
#endif
	Config = (ModConfig*)malloc(sizeof(ModConfig));
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&shconfig_lock);
#endif
    bzero(Config,sizeof(ModConfig));

    dynncpy(Config->item,item,sizeof(Config->item));
    dynncpy(Config->value,value,sizeof(Config->value));

    dynncpy(Config->modulefile,to_modulefile,sizeof(Config->modulefile));

#ifdef USE_THREADS
	my_pthread_mutex_lock(&shconfig_lock);
#endif
    Config->next = ModuleConfig;
    Config->prev = NULL;
    if (ModuleConfig)
        ModuleConfig->prev = Config;

    ModuleConfig = Config;

#ifdef USE_THREADS
	my_pthread_mutex_unlock(&shconfig_lock);
#endif
	return true;
}

/*  ************************************************************************************************  */

/**
 * @brief delModuleConfig:  Deletes all config entries for a modulefile
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param module The pointer to the moduleinformation
 * @return void
 **/
void delModuleConfig(Threads *thread, ModCo *module){
	ModConfig *Config = NULL;

#ifdef USE_THREADS
	my_pthread_mutex_lock(&shconfig_lock);
#endif
	for (Config = ModuleConfig; Config; Config = Config->next) {

		if (!Config){
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&shconfig_lock);
#endif
			return;
		}

		if(streqnc(Config->modulefile,module->filename)){

			if (Config->prev)
				Config->prev->next = Config->next;

			else
				ModuleConfig = Config->next;

			if (Config->next)
				Config->next->prev = Config->prev;

			free(Config);
		}
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&shconfig_lock);
#endif

	bzero(module->configfile,sizeof(module->configfile));
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief rehashModuleConfigs:  Rehashes all external configs
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return void
 **/
void rehashModuleConfigs(Threads *thread){
	ModConfig 	*Config = NULL;
	ModCo 		*module = NULL;

#ifdef USE_THREADS
	my_pthread_mutex_lock(&shconfig_lock);
#endif
	for (Config = ModuleConfig; Config; Config = Config->next) {

		if (!Config){
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&shconfig_lock);
#endif
			return;
		}


		if (Config->prev)
			Config->prev->next = Config->next;

		else
			ModuleConfig = Config->next;

		if (Config->next)
			Config->next->prev = Config->prev;

		free(Config);
	}
	
	for (module = modulelist; module; module = module->next) {

		if (!module){
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&shconfig_lock);
#endif
			return;
		}

		if(module->configfile[0]!='\0'){
            moduleLoadconfig(thread, module, module->configfile);
		}
	}
	
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&shconfig_lock);
#endif
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief getConfigVariable:  Returns the value of a config variable
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param item The item for which we want it's value
 * @return char *
 **/
char *getConfigVariable(Threads *thread, char *item){
	ModConfig *Config = NULL;

#ifdef USE_THREADS
	my_pthread_mutex_lock(&shconfig_lock);
#endif
	for (Config = ModuleConfig; Config; Config = Config->next) {
		if (!Config){
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&shconfig_lock);
#endif
			return NULL;
		}

		if(streqnc(Config->item,item)){
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&shconfig_lock);
#endif
			if(Config->item[0]=='\0')
			    return (char*)1;
			else
				return Config->value;
		}
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&shconfig_lock);
#endif
	return NULL;
}

/*  ************************************************************************************************  */

/**
 * @brief delModule:  Deletes target modul
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param servicetag The tag of the service for which the modul was added
 * @param modulename The modul which should be deleted (includes all commands added for this modul)
 * @return MOD_ERROR
 **/
MOD_ERROR delModule(Threads *thread, char *servicetag, char *modulename){
	ModCo *module = NULL;

#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
#endif
	module = findModule(thread,servicetag,modulename);
    
	if (!module){
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&modules_lock);
#endif
        return MOD_ERR_NOEXIST;
	}
        
    if (module->prev)
        module->prev->next = module->next;
        
    else
        modulelist = module->next;
        
    if (module->next)
        module->next->prev = module->prev;
        
    free(module);
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&modules_lock);
#endif
    return MOD_ERR_OK;
}

/*  ************************************************************************************************  */

/**
 * @brief delCommand:  Deletes a command from a modul (or all commands)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param servicetag The tag of the service for which the modul was added
 * @param modulename The modul for which the command was added
 * @param command The command to delete (or NULL to delete all commands for target modul)
 * @return MOD_ERROR
 **/
MOD_ERROR delCommand(Threads *thread, char *servicetag, char *modulename, char *command){
    ModCo *module 	= NULL;
    CMD   *c 		= NULL;

#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
#endif
    module = findModule(thread,servicetag,modulename);
    
    if(command != NULL){
        c = findCommand(thread,module,command);
		if (!c){
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&modules_lock);
#endif
			return MOD_ERR_CMD_NOEXIST;
		}
            
        if (c->prev)
            c->prev->next = c->next;
            
        else
            module->commands = c->next;
            
        if (c->next)
            c->next->prev = c->prev;
            
        free(c);
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&modules_lock);
#endif
	}else{
		for (c = module->commands; c; c = c->next) {
			if (!c){
#ifdef USE_THREADS
				my_pthread_mutex_unlock(&modules_lock);
#endif
				return MOD_ERR_CMD_NOEXIST;
			}
				
			if (c->prev)
				c->prev->next = c->next;
				
			else
				module->commands = c->next;
				
			if (c->next)
				c->next->prev = c->prev;
				
			free(c);
		}
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&modules_lock);
#endif
    return MOD_ERR_OK;
}


/*  ************************************************************************************************  */

/**
 * @brief delModulebyFile:  Deletes a modul
 *								The Modul is identified by it's unique filename (so only one will match)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param modulefile The modulefile we want to delete
 * @return MOD_ERROR
 **/
MOD_ERROR delModulebyFile(Threads *thread, char *modulefile){
	ModCo *module = NULL;

#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
#endif
	module = FilefindModule(thread,modulefile);
    
	if (!module){
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&modules_lock);
#endif
        return MOD_ERR_NOEXIST;
	}
        
    if (module->prev)
        module->prev->next = module->next;
        
    else
        modulelist = module->next;
        
    if (module->next)
        module->next->prev = module->prev;
        
    free(module);
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&modules_lock);
#endif
    return MOD_ERR_OK;
}

/*  ************************************************************************************************  */

/**
 * @brief delCommandbyFile:  Deletes a command from a modul (or all commands)
 *								The Modul is identified by it's unique filename (so only one will match)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param modulefile The modulefile for which the command was added
 * @param command The command to delete (or NULL to delete all commands for target modul)
 * @return MOD_ERROR
 **/
MOD_ERROR delCommandbyFile(Threads *thread, char *modulefile, char *command){
    ModCo *module 	= NULL;
    CMD   *c 		= NULL;

#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
#endif
    module = FilefindModule(thread,modulefile);
    if(module == NULL)
    	return MOD_ERR_NOEXIST;
    
    if(command != NULL){
        c = findCommand(thread,module,command);
		if (!c){
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&modules_lock);
#endif
			return MOD_ERR_CMD_NOEXIST;
		}
            
        if (c->prev)
            c->prev->next = c->next;
            
        else
            module->commands = c->next;
            
        if (c->next)
            c->next->prev = c->prev;
            
        free(c);
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&modules_lock);
#endif
	}else{
		for (c = module->commands; c; c = c->next) {
			if (!c){
#ifdef USE_THREADS
				my_pthread_mutex_unlock(&modules_lock);
#endif
				return MOD_ERR_CMD_NOEXIST;
			}
				
			if (c->prev)
				c->prev->next = c->next;
				
			else
				module->commands = c->next;
				
			if (c->next)
				c->next->prev = c->prev;
				
			free(c);
		}
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&modules_lock);
#endif
    return MOD_ERR_OK;
}

/*  ************************************************************************************************  */

/**
 * @brief User_Command_Level:  Returns the user level for target command
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param servicetag The tag for the service for which the module was added
 * @param modulename The module for which the command was added
 * @param command The command from which we want to know the user_level
 * @return user_level The user level for that command (int)
 **/
int User_Command_Level(Threads *thread, char *servicetag, char *modulename, char *command){
	ModCo *module 	= NULL;
	CMD  *c 		= NULL;

#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
#endif
	module = findModule(thread,servicetag,modulename);
	if(module != NULL)
    	c = findCommand(thread,module,command);
	
	if(c != NULL){
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&modules_lock);
#endif
		return c->user_level;
	}

#ifdef USE_THREADS
	my_pthread_mutex_unlock(&modules_lock);
#endif
	return INFINIT;
}

/*  ************************************************************************************************  */

/**
 * @brief Root_Command_Level:  Returns the root level for target command
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param servicetag The tag for the service for which the module was added
 * @param modulename The module for which the command was added
 * @param command The command from which we want to know the root_level
 * @return root_level The root_level level for that command (int)
 **/
int Root_Command_Level(Threads *thread, char *servicetag, char *modulename, char *command){
	ModCo *module 	= NULL;
	CMD   *c 		= NULL;

#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
#endif
	module = findModule(thread,servicetag,modulename);
	if(module != NULL)
    	c = findCommand(thread,module,command);
	
	if(c != NULL){
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&modules_lock);
#endif
		return c->root_level;
	}

#ifdef USE_THREADS
	my_pthread_mutex_unlock(&modules_lock);
#endif
	return INFINIT;
}

/*  ************************************************************************************************  */

/**
 * @brief Dynamic_Command_Level:  Returns the command level for target command
 *                            depending on the access-type
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param servicetag The tag for the service for which the modul was added
 * @param modulename The module for which the command was added
 * @param command The command from which we want to know the level
 * @return level The level for that command (int) (Depends on the Access-Type of the module)
 */
int Dynamic_Command_Level(Threads *thread, char *servicetag, char *modulename, char *command){
	ModCo *module 	= NULL;
	CMD   *c 		= NULL;

#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
#endif
	
	module = findModule(thread,servicetag,modulename);
	if(module != NULL)
    	c = findCommand(thread,module,command);
	
	if(c != NULL){
                
		if(module->access == USER || module->access == BOTH){
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&modules_lock);
#endif
			return c->user_level;
		}
			
		else if(module->access == ADMIN){
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&modules_lock);
#endif
			return c->root_level;
		}		
	}

#ifdef USE_THREADS
	my_pthread_mutex_unlock(&modules_lock);
#endif
    return INFINIT;
}

/*  ************************************************************************************************  */

/**
 * @brief ModuleFile_UserCommandLevel:  Returns the user level for target command
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param modulefile The module's Filename
 * @param command The command from which we want to know the user_level
 * @return user_level The user level for that command (int)
 **/
int ModuleFile_UserCommandLevel(Threads *thread, char *modulefile, char *command){
	ModCo *module 	= NULL;
	CMD   *c 		= NULL;

#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
#endif
	module = FilefindModule(thread,modulefile);
	if(module != NULL)
    	c = findCommand(thread,module,command);
	
	if(c != NULL){
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&modules_lock);
#endif
		return c->user_level;
	}

#ifdef USE_THREADS
	my_pthread_mutex_unlock(&modules_lock);
#endif
	return INFINIT;
}

/*  ************************************************************************************************  */

/**
 * @brief ModuleFile_RootCommandLevel:  Returns the root level for target command
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param service The service for which the modul was added
 * @param modulefile The module's Filename
 * @param command The command from which we want to know the root_level
 * @return root_level The root_level level for that command (int)
 **/
int ModuleFile_RootCommandLevel(Threads *thread, char *modulefile, char *command){
	ModCo *module 	= NULL;
	CMD   *c 		= NULL;

#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
#endif
	module = FilefindModule(thread,modulefile);
	if(module != NULL)
    	c = findCommand(thread,module,command);
	
	if(c != NULL){
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&modules_lock);
#endif
		return c->root_level;
	}

#ifdef USE_THREADS
	my_pthread_mutex_unlock(&modules_lock);
#endif
	return INFINIT;
}

/*  ************************************************************************************************  */

/**
 * @brief ModuleFile_DynamicCommandLevel:  Returns the command level for target command
 *                            depending on the access-type
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param modulefile The module's Filename
 * @param command The command from which we want to know the level
 * @return level The level for that command (int) (Depends on the Access-Type of the modul)
 */
int ModuleFile_DynamicCommandLevel(Threads *thread, char *modulefile, char *command){
	ModCo *module 	= NULL;
	CMD   *c 		= NULL;

#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
#endif
	module = FilefindModule(thread,modulefile);
	if(module != NULL)
    	c = findCommand(thread,module,command);
	
	if(c != NULL){
                
		if(module->access == USER || module->access == BOTH){
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&modules_lock);
#endif
			return c->user_level;
		}
			
		else if(module->access == ADMIN){
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&modules_lock);
#endif
			return c->root_level;
		}		
	}

#ifdef USE_THREADS
	my_pthread_mutex_unlock(&modules_lock);
#endif
    return INFINIT;
}


/*  ************************************************************************************************  */

/**
 * @brief VerifyLevels:  Verifies the Level for the Modul
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param ModHandle The handle for target module
 * @return MOD_ERROR
 */
MOD_ERROR VerifyLevels(Threads *thread, ModInfo *ModHandle){
    if(ModHandle->module == NULL)
		return MOD_ERR_NOEXIST;
	
	if(ModHandle->module->LeastUserLevel == INFINIT)
		ModHandle->module->LeastUserLevel = ModuleFile_UserCommandLevel(thread,ModHandle->module->filename,ModHandle->module->modulename);
		
	if(ModHandle->module->LeastRootLevel == INFINIT)
		ModHandle->module->LeastRootLevel = ModuleFile_RootCommandLevel(thread,ModHandle->module->filename,ModHandle->module->modulename);

    return MOD_ERR_OK;
}

/*  ************************************************************************************************  */

/**
 * @brief FilefindModule:  Returns the pointer to target module information (from a given Modulfile)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param modulefile The Modulefile we are searching for
 * @return (ModCo*)module The pointer to target module (ModCo*)
 **/
ModCo *FilefindModule(Threads *thread, char *modulefile){
	ModCo *module = NULL;
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
#endif
	for (module = modulelist; module; module = module->next) {
		//dlog(thread,3,"module->modulename = %s, module->filename = %s\n",module->modulename,module->filename);
		if (module->filename[0]!='\0' && streqnc(module->filename,modulefile)){
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&modules_lock);
#endif
            //dlog(thread,3,"MATCHED: module->filename = modulefile, %s\n",module->filename,modulefile);
			return module;
		}
	}
	
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&modules_lock);
#endif
	return NULL;
}

/*  ************************************************************************************************  */

/**
 * @brief findModul:  Returns the pointer to target module information
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param servicetag The servicetag of the Service for which the module was added
 * @param modulename The module for which the address is needed
 * @return (ModCo*)module The pointer to target modul (ModCo*)
 **/
ModCo *findModule(Threads *thread, char *servicetag, char *modulename){
	ModCo *module = NULL;
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
#endif
	for (module = modulelist; module; module = module->next) {
		if (module->servicetag[0]!='\0' && module->modulename[0]!='\0' && streqnc(module->servicetag,servicetag) && streqnc(module->modulename,modulename)){
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&modules_lock);
#endif
			return module;
		}
	}
	
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&modules_lock);
#endif
	return NULL;
}

/*  ************************************************************************************************  */

/**
 * @brief findModul_next:  Returns the next module (starting from the old one)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param servicetag The servicetag of the Service for which the module was added
 * @param modulename The modulename for which the address is needed
 * @param module_start The modulpointer where we want to start searching
 * @return (ModCo*)module The pointer to target module (ModCo*)
 **/
ModCo *findModule_next(Threads *thread, char *servicetag, char *modulename, ModCo *module_start){
	ModCo *module = NULL;
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
#endif
	for (module = module_start->next; module; module = module->next) {
		if (module->servicetag[0]!='\0' && module->modulename[0]!='\0' && streqnc(module->servicetag,servicetag) && streqnc(module->modulename,modulename)){
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&modules_lock);
#endif
			return module;
		}
	}
	
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&modules_lock);
#endif
	return NULL;
}

/*  ************************************************************************************************  */

/**
 * @brief findCommand:  Returns the pointer to target command
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param module The pointer referencing target module (receive that with findModule())
 * @param calling The identifier for the command
 * @return (CMD*)cmd The pointer to target command (CMD*)
 **/
CMD *findCommand(Threads *thread, ModCo *module, char *calling){
	CMD *c = NULL;
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
#endif
	for (c = module->commands; c; c = c->next) {
		if(streqnc(c->call,calling)){
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&modules_lock);
#endif
        	return c;
		}
	}

#ifdef USE_THREADS
	my_pthread_mutex_unlock(&modules_lock);
#endif
	return NULL;
}

/*  ************************************************************************************************  */

/**
 * @brief ExecuteEvent:  Executes an Event - sends the Information to all loaded modules
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The nick of the executor
 * @param param The default parameters as used in all modules
 * @param event The Event that should be sended
 * @return void
 **/
void ExecuteEvent(Threads *thread, char *nick, char *param[], char *event){
	ModCo *module = NULL;

#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
#endif
	for (module = modulelist; module; module = module->next) {
		if(module!=NULL){
			
			if(streqnc(module->modulename,"events")){
				CMD *c = NULL;
				c = findCommand(thread,module,event);
				if(c != NULL){
					c->func(thread,languages[m->language],0,m->servername,nick,param);
				}
			}
		}
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&modules_lock);
#endif
	return;
}


/*  ************************************************************************************************  */

/**
 * @brief ExecuteSingleEvent:  Executes an Event - sends the Information to ONLY one module (UNIQUE)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The nick of the executor
 * @param param The default parameters as used in all modules
 * @param servicetag The service for which we like to execute the event
 * @param event The Event that should be sended
 * @return void
 **/
MOD_ERROR ExecuteSingleEvent(Threads *thread, char *nick, char *param[], char *servicetag, char *event){
	ModCo *module = NULL;

#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
#endif
	for (module = modulelist; module; module = module->next) {
		if(module!=NULL){
			if(streqnc(module->modulename,"events") && streqnc(module->servicetag,servicetag)){

				CMD *c = NULL;
				c = findCommand(thread,module,event);
				if(c != NULL){
					c->func(thread,languages[m->language],0,m->servername,nick,param);
					return MOD_ERR_OK;
				}
				else return MOD_ERR_CMD_NOEXIST;
			}
		}
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&modules_lock);
#endif
	return MOD_ERR_NOEXIST;
}

/*  ************************************************************************************************  */

/**
 * @brief OrionLoadModule:  Loads a modul
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param modulefile The modulefile which should be loaded
 * @return ModInfo
 **/
ModInfo OrionLoadModule(Threads *thread, char *modulefile){
	char library[MAX_FILEPATH_LEN];
	ModInfo (*func) (Threads *, void *);
	void *dl_handle = NULL;
	ModInfo ModHandle;

	snprintf(library,sizeof(library),"%s/%s%s",MODULE_PATH,modulefile,MODULE_EXT);
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
#endif

	ModHandle.module = FilefindModule(thread,modulefile);
	ModHandle.ModErr = MOD_ERR_OK;
	
	if(ModHandle.module == NULL){
		
		dl_handle	= dynopen(thread,library,RTLD_NOW);
		if(dl_handle==NULL){
			ModHandle.ModErr = MOD_ERR_NOTLOADED;
			
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&modules_lock);
#endif
			return ModHandle;
		}

		func		= (ModInfo (*)(Threads *,void *))dynsym (thread,dl_handle,"orion_Init");

		if(func){
			ModHandle = (*func) (thread,dl_handle);
			
			if(ModHandle.ModErr == MOD_ERR_CANCELLED){
				dynclose(thread,dl_handle);
				
#ifdef USE_THREADS
				my_pthread_mutex_unlock(&modules_lock);
#endif
				return ModHandle;
			}
			else{
#ifdef USE_THREADS
				my_pthread_mutex_unlock(&modules_lock);
#endif
				/* Ensure that Error isn't higher than END and lower than "0" */
				if(ModHandle.ModErr >= MOD_ERR_END || ModHandle.ModErr < 0){
					dlog(thread,0,"Module Error for \"%s\" on loading - ErrorNo(%d): %s\n",modulefile,ModHandle.ModErr,ModuleError(MOD_ERR_NOTUSEDINIT));
					orion_exit(thread);
				}
					
				return ModHandle;
			}
		}
		
		ModHandle.ModErr = MOD_ERR_MEMORY;
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&modules_lock);
#endif
		return ModHandle;
        
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&modules_lock);
#endif
	ModHandle.ModErr = MOD_ERR_EXISTS;
	return ModHandle;
}

/*  ************************************************************************************************  */

/**
 * @brief OrionUnloadModule:  Unloads a modul
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param modulefile The modulfile which should be unloaded
 * @param unloadtype The module is only deleted by using DO_UNLOAD (used to keep Services up with DO_RELOAD)
 * @return MOD_ERROR
 **/
MOD_ERROR OrionUnloadModule(Threads *thread, char *modulefile, DO_ACTION unloadtype){
	ModCo *module = NULL;
	MOD_ERROR moderr;
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
#endif

	module = FilefindModule(thread,modulefile);

	if(module != NULL && module->dl_handle != NULL){
		module->func_finish(thread,unloadtype);
		
		if(unloadtype == DO_UNLOAD || unloadtype == DO_RELOAD){
			dlog(thread,2,"Modul %s, Filename: %s\n",module->modulename,module->filename);
		
			dynclose    (thread,module->dl_handle);
		
			if(module->configfile[0]!='\0')
				delModuleConfig(thread, module);
			delSharedLanguage(thread,modulefile);
		
			moderr = delCommandbyFile(thread,modulefile,NULL);
			if(moderr != MOD_ERR_OK){
#ifdef USE_THREADS
				my_pthread_mutex_unlock(&modules_lock);
#endif
				return moderr;
			}
		
			moderr = delModulebyFile(thread,modulefile);
			if(moderr != MOD_ERR_OK){
#ifdef USE_THREADS
				my_pthread_mutex_unlock(&modules_lock);
#endif
				return moderr;
			}

#ifdef USE_THREADS
			my_pthread_mutex_unlock(&modules_lock);
#endif
			return MOD_ERR_OK;
		}
		return MOD_ERR_OK;
	}
	
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&modules_lock);
#endif
	return MOD_ERR_NOTUNLOADED;
}

/*  ************************************************************************************************  */

/**
 * @brief IRCLoadModule:  Loads a modul
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @param service The service for which the modul should be loaded
 * @param modul The modul which should be loaded
 * @return ModInfo
 **/
ModInfo IRCLoadModule(Threads *thread, void* language, bool howtosend, char *snick, char *nick, char *param[], char *service, char *modul){
	char library[MAX_FILEPATH_LEN];
	char modulefile[MAX_MODULEFILE_LEN];
	ModInfo (*func) (Threads *, void *) 	= NULL;
	void *dl_handle 						= NULL;
	ModInfo ModHandle;

	snprintf(modulefile,sizeof(modulefile),"%s_%s",_lc(service),modul);
	snprintf(library,sizeof(library),"%s/%s_%s%s",MODULE_PATH,service,modul,MODULE_EXT);
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
#endif
	ModHandle.module = FilefindModule(thread,modulefile);
	
	if(ModHandle.module == NULL){

		dl_handle = _dynopen(library);
		
		if(dl_handle != NULL){
			func = (ModInfo (*)(Threads *,void *))_dynsym(dl_handle,"orion_Init");
		}
		else{		
#ifdef _dynerror
			do_privmsg(thread,snick,nick,"Modul ERROR - Modul unknown (%s)",_dynerror());
#else
			do_privmsg(thread,snick,nick,"Modul ERROR - Modul unknown");
#endif
			
			ModHandle.ModErr = MOD_ERR_NOTLOADED;
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&modules_lock);
#endif
			return ModHandle;
		}
		
		if(func != NULL){
			ModHandle = (*func) (thread,dl_handle);
			
			if(ModHandle.ModErr != MOD_ERR_OK){
				_dynclose(dl_handle);
				do_privmsg(thread,snick,nick,"Modul ERROR - %s",ModuleError(ModHandle.ModErr));
				
#ifdef USE_THREADS
				my_pthread_mutex_unlock(&modules_lock);
#endif
				return ModHandle;
			}
			else{
				do_privmsg(thread,snick,nick,"Modul SUCCESS - Modul loaded successful.");
#ifdef USE_THREADS
				my_pthread_mutex_unlock(&modules_lock);
#endif
				return ModHandle;
			}
		}else{
			_dynclose(dl_handle);
			do_privmsg(thread,snick,nick,"Modul ERROR - Cannot find orion_Init()! (check your modul).");
	
			ModHandle.ModErr = MOD_ERR_MEMORY;
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&modules_lock);
#endif
			return ModHandle;
		}
	}else{
		do_privmsg(thread,snick,nick,"Modul WARNING - Modul already loaded (use reload).");

		ModHandle.ModErr = MOD_ERR_EXISTS;
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&modules_lock);
#endif
		return ModHandle;
	}

	ModHandle.ModErr = MOD_ERR_EXISTS;
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&modules_lock);
#endif
	return ModHandle;
}

/*  ************************************************************************************************  */

/**
 * @brief IRCUnloadModule:  Unloads a modul
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @param service The service for which the modul should be unloaded
 * @param modulname The module which should be unloaded
 * @param unloadtype The module is only deleted by using DO_UNLOAD (used to keep Services up with DO_RELOAD)
 * @return MOD_ERROR
 **/
MOD_ERROR IRCUnloadModule(Threads *thread, void* language, bool howtosend, char *snick, char *nick, char *param[], char *service, char *modulename, DO_ACTION unloadtype){
	ModCo *module = NULL;
	char modulefile[MAX_MODULEFILE_LEN];
	
	snprintf(modulefile,sizeof(modulefile),"%s_%s",_lc(service),modulename);

#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
#endif
	module = FilefindModule(thread,modulefile);

	if(module != NULL && module->dl_handle != NULL){
		dlog(thread,3,"found modul\n");
		
		module->func_finish(thread,unloadtype);
		
		dlog(thread,3,"executed fini\n");
		
		if(unloadtype == DO_UNLOAD || unloadtype == DO_RELOAD){
			if(_dynclose(module->dl_handle)){
#ifdef _dynerror
				do_privmsg(thread,snick,nick,"Modul ERROR - Modul wasn't unloaded successfully (%s).",_dynerror());
#else
				do_privmsg(thread,snick,nick,"Modul ERROR - Modul wasn't unloaded successfully.");
#endif

#ifdef USE_THREADS
				my_pthread_mutex_unlock(&modules_lock);
#endif
				return MOD_ERR_NOTUNLOADED;
			}
			else{
				dlog(thread,3,"dlclose() worked\n");
				if(module->configfile[0]!='\0')
					delModuleConfig(thread,module);
					
			    delSharedLanguage(thread,modulefile);
				delCommandbyFile(thread,modulefile,NULL);
				delModulebyFile(thread,modulefile);
			
				do_privmsg(thread,snick,nick,"Modul SUCCESS - Modul unloaded successfully.");
			
#ifdef USE_THREADS
				my_pthread_mutex_unlock(&modules_lock);
#endif
				return MOD_ERR_OK;
			}
		}
		return MOD_ERR_OK;
	}
	else{
		do_privmsg(thread,snick,nick,"Modul ERROR - Modul not found (not loaded - remember that modules and service are case sensivity).");

#ifdef USE_THREADS
		my_pthread_mutex_unlock(&modules_lock);
#endif
		return MOD_ERR_NOEXIST;
	}

#ifdef USE_THREADS
	my_pthread_mutex_unlock(&modules_lock);
#endif
	return MOD_ERR_NOEXIST;
}

/*  ************************************************************************************************  */

/**
 * @brief do_run_cmd: Runs a Command through all commands in a module
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param CMD The command struct where our command is in (any possible others)
 * @param language The language the receiver had choosen
 * @param howtosend The way the receiver likes to get informed (notice = 0/privmsg = 1)
 * @param snick The Service nick for sending the information
 * @param nick The nick of the executor
 * @param param The original parameter array
 * @return void
 */
static void do_run_cmd(Threads *thread, ModCo *module, CMD *c, ModCheck *modcheck, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    int runned 		= 0;
    CMD *current	= NULL;
    MOD_BOOL retVal = MOD_CONT;

    if (c){
		if(c->func) {
            current = c;
            
            while (current && retVal == MOD_CONT) {
				if(streqnc(current->call,c->call)){
					dlog(thread,0,"if(streqnc(current->call,c->call)) -> (looping) -> c->call = \"%s\"\n",c->call);
					modcheck->ModErrParams = (MOD_ERROR)CheckParamters(thread,module,current->call,param,snick,nick);
					
					if(modcheck->ModErrParams == MOD_REPLY_NONE){
                        dlog(thread,0,"before func() current == %p\n",current);
						dlog(thread,0,"CheckParamters==MOD_REPLY_NONE -> executing\n");
                		retVal = current->func(thread,language,howtosend,snick,nick,param);
                		dlog(thread,0,"after func() current == %p\n",current);
                		runned++;
					}
            	}
            	dlog(thread,0,"after block current == %p\n",current);
            	current = current->next;
        	}
    	}
    }
    modcheck->status = retVal;
    modcheck->executed += runned;
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief call_command: Executes a command ina module
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param modul The modul we found
 * @param cmd The command we want to execute
 * @param snick The Service nick for sending the information
 * @param nick The nick of the executor
 * @param param The original parameter array
 * @return ModCheck Includes OptionsError, ParameterError and number of executed modules
 */
ModCheck call_command(Threads *thread, ModCo *module, char *cmd, char *snick, char *nick, char *param[]){
	ModCheck modcheck;
	
	char call      [BUFSIZE];
    char prototype [MAX_COMMANDNAME_LEN*4];
    char *Mprototype[10];
    CMD *command 		= NULL;
	int i 				= 0;
	int values 			= 0;

    bzero(&modcheck,sizeof(&modcheck));

	while(module!=NULL){
   		bzero(&call,sizeof(&call));
   		bzero(Mprototype,sizeof(Mprototype));
		
		dynncpy(prototype,module->prototype,sizeof(prototype));
		values = strsplit(prototype,Mprototype,' ');

		straddns(call,cmd,sizeof(call));
		straddnc(call,' ',sizeof(call));
		values--;

		for(i=4;i-4<values;i++){
    		if(!streqnc(Mprototype[i-3],"*")){
				if(param[i]){
					straddns(call,param[i],sizeof(call));
					straddnc(call,' ',sizeof(call));
				}else
					break;
    		}else continue;
    	}

		call[dynlen(call)-1]='\0';
    	strclean(call);
		dlog(thread,4,"call = %s\n",call);
		
    	command = findCommand(thread,module,call);
    	if(command){
			modcheck.ModErrOptions = (MOD_ERROR)CheckOptions(thread,module,snick,nick);
			
			if(modcheck.ModErrOptions == MOD_REPLY_NONE){
				do_run_cmd(thread,module,command,&modcheck,languages[thread->sql.access.language],thread->sql.access.settings&NS_SET_MSG ? 1 : 0,snick,nick,param);
			}
		}
		else
		/* If the command couldn't be found we look for another module to find it (so we continue... ) */
		    modcheck.status = MOD_CONT;
		
		/* Look if we are able to go on processing, or if we should stop here .. */
		if(modcheck.status == MOD_CONT)
			/* We continue processing modules */
			module = findModule_next(thread,module->servicetag,cmd,module);
			
		/* We skip processing others .. */
		else break;
	}
	return modcheck;
}

/*  ************************************************************************************************  */

/**
 * @brief call_helpcommand: Executes a help command in a module
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param modul The modul we found
 * @param cmd The command we want to execute
 * @param snick The Service nick for sending the information
 * @param nick The nick of the executor
 * @param param The original parameter array
 * @return ModCheck Includes OptionsError, ParameterError and number of executed modules
 */
ModCheck call_helpcommand(Threads *thread, ModCo *module, char *cmd, char *snick, char *nick, char *param[]){
	ModCheck modcheck;
	
	char call      	[BUFSIZE];
    char prototype 	[MAX_COMMANDNAME_LEN*4];
    char *Mprototype[10];
    CMD *command 		= NULL;
	int i 				= 0;
	int values 			= 0;
	
    bzero(&modcheck,sizeof(&modcheck));

	while(module!=NULL){
   		bzero(&call,sizeof(&call));
   		bzero(Mprototype,sizeof(Mprototype));
		
		dynncpy(prototype,module->helpprototype,sizeof(prototype));
		values = strsplit(prototype,Mprototype,' ');
		
		straddns(call,cmd,sizeof(call));
		straddnc(call,' ',sizeof(call));
		
		for(i=4;i-4<values;i++){
	    	if(!streqnc(Mprototype[i-3],"*")){
				if(param[i]){
					straddns(call,param[i],sizeof(call));
					straddnc(call,' ',sizeof(call));
				}else
					break;
	    	}else continue;
	    }
	    
		call[dynlen(call)-1]='\0';
	    strclean(call);
		dlog(thread,2,"call = %s\n",call);	
		
    	command = findCommand(thread,module,call);
    	if(command){
			modcheck.ModErrOptions = (MOD_ERROR)CheckOptions(thread,module,snick,nick);
			
			if(modcheck.ModErrOptions == MOD_REPLY_NONE){
				do_run_cmd(thread,module,command,&modcheck,languages[thread->sql.access.language],thread->sql.access.settings&NS_SET_MSG ? 1 : 0,snick,nick,param);
			}
		}
		else
		/* If the command couldn't be found we look for another module to find it (so we continue... ) */
			modcheck.status = MOD_CONT;
		
		/* Look if we are able to go on processing, or if we should stop here .. */
		if(modcheck.status == MOD_CONT)
			/* We continue processing modules */
			module = findModule_next(thread,module->servicetag,param[4],module);
			
		/* We skip processing others .. */
		else break;
	}
	return modcheck;
}

/*  ************************************************************************************************  */

/**
 * @brief call_wildview: Executes a help for "help *add*" - allows users to search for commands easily
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param snick The Service nick for sending the information
 * @param nick The nick of the executor
 * @param servicetag The servicetag
 * @param help_overview_tag The helpoverview tag "help" or "showcommands" (at Q)
 * @param param The original parameter array
 * @return void
 */
void call_wildview(Threads *thread, char *snick, char *nick, char *servicetag, char *help_overview_tag, char *param[]){
	ModCheck 	modcheck;
	deepSQL 	mCMD;
	
    ModCo *module = NULL;
    CMD *command = NULL;
    
	char wildmask  [BUFSIZE];
	
    bzero(&modcheck,sizeof(modcheck));
    bzero(&mCMD,sizeof(mCMD));
    bzero(&wildmask,sizeof(wildmask));
    
	if(!param[5]){
	    					
		dynncpy(wildmask,param[4],sizeof(wildmask));
		strrepc(wildmask,'*','%');
	    			
	    mCMD.res = doSelectMySQL(thread,"SELECT `modul` FROM `orion_core_commands` WHERE `modul` LIKE '%s' AND `service`='%s'",mysql_quote(thread,wildmask),mysql_quote(thread,servicetag));
	    while((mCMD.row = mysql_fetch_row(mCMD.res))){
	    	module = findModule(thread,servicetag,mCMD.row[0]);
						
			modcheck.ModErrOptions = (MOD_ERROR)CheckOptions(thread,module,snick,nick);
			if(modcheck.ModErrOptions == MOD_REPLY_NONE){
				
		    	if(module  != NULL && !streqnc(module->modulename,help_overview_tag))
					command = findCommand(thread,module,help_overview_tag);
					
		    	if(command != NULL && !streqnc(module->modulename,help_overview_tag))
					command->func(thread,languages[thread->sql.access.language],
							thread->sql.access.settings&NS_SET_MSG ? 1 : 0,snick,nick,param);

		    }else{
				ModReply(thread, &modcheck, snick, nick, param);
				break;
			}
	    }
		mysql_free_result_myr(mCMD.res);
	}
	else
		send_lang(thread,languages[thread->sql.access.language],thread->sql.access.settings&NS_SET_MSG ? 1 : 0,snick,nick,"CS_UNKNOWN_COMMAND",snick);

	return;
}

/*  ************************************************************************************************  */

/**
 * @brief call_overviewhelp: Executes a common overview help for a service
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The Service nick for sending the information
 * @param servicetag The Servicetag we have to use for that Service
 * @param help_overview_tag The helpoverview tag "help" or "showcommands" (at Q)
 * @param nick The nick of the executor
 * @param param The original parameter array
 * @return void
 */
void call_overviewhelp(Threads *thread, void *language, bool howtosend, char *snick, char *servicetag, char *help_overview_tag, char *nick, char *param[]){
	deepSQL MySQL;
	ModCo *module = NULL;
	CMD *command  = NULL;
	
	int i = 0;
	bzero(&MySQL,sizeof(MySQL));

	MySQL.res    = doSelectMySQL(thread,"SELECT `modul` FROM `orion_core_commands` WHERE `service`='%s' ORDER BY `modul`,`user_level`,`root_level` ASC",servicetag); //prepare Query
	MySQL.fields = mysql_num_rows(MySQL.res);

	if(MySQL.fields){
		for(i=0;i<MySQL.fields;i++){
			MySQL.row = mysql_fetch_row(MySQL.res);
			
			module = findModule(thread,servicetag,MySQL.row[0]);

			if(module != NULL && !streqnc(module->modulename,help_overview_tag)){
				command = findCommand(thread,module,help_overview_tag);
				
				if(command != NULL){
					if((module->options&OPERONLY_BIT && strcor(thread->sql.Cdata.modes,"o")) || ~module->options&OPERONLY_BIT)
						command->func(thread,language,howtosend,snick,nick,param);
				}
			}
		}
	}
	mysql_free_result_myr(MySQL.res); //free MySQL result
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief ListModules:  Sends a list of loaded modules by PRIVMSG
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param snick The servicesnick which should send the information
 * @param rnick The receiver of the listing
 * @param servicetag The servicetag of the Service which's information we are searching
 * @return void
 **/
void ListModules(Threads *thread, char* snick, char *rnick, char *servicetag){
	ModCo *module = NULL;
	
	if(servicetag == NULL){
#ifdef USE_THREADS
		my_pthread_mutex_lock(&modules_lock);
#endif
		for (module = modulelist; module; module = module->next) {
			if(streqnc(module->modulename,"events"))
				do_privmsg(thread,snick,rnick,"Service: %s",module->servicetag);
		}
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&modules_lock);
#endif
	}
	
	else{
#ifdef USE_THREADS
		my_pthread_mutex_lock(&modules_lock);
#endif
    	for (module = modulelist; module; module = module->next) {
    		if (streqnc(module->servicetag,servicetag))
                do_privmsg(thread,snick,rnick,"Modul %s (%s) [Author: %s, Version: %s]",module->modulename,module->filename,module->Author,module->Version);
    	}
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&modules_lock);
#endif
	}
	
	return;
}


/*  ************************************************************************************************  */

/**
 * @brief DumpModules:  prints all loaded modules
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return void
 **/
void DumpModules(Threads *thread){
	ModCo *module = NULL;

#ifdef USE_THREADS
	my_pthread_mutex_lock(&modules_lock);
#endif
    for (module = modulelist; module; module = module->next) {
        dlog(thread,0,"Modul %s (%s) [Version: %s]",module->modulename,module->filename,module->Version);
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&modules_lock);
#endif
	return;
}

/*  ************************************************************************************************  */
